@@ -53,6 +53,26 @@ public class AlterTableOptions {
     return this;
   }
 
+  /**
+   * Change a table's owner.
+   * @param owner the new table owner
+   * @return this instance
+   */
+  public AlterTableOptions setOwner(String owner) {
+    pb.setNewTableOwner(owner);
+    return this;
+  }
+
+  /**
+   * Change a table's comment.
+   * @param comment the new table comment
+   * @return this instance
+   */
+  public AlterTableOptions setComment(String comment) {
+    pb.setNewTableComment(comment);
+    return this;
+  }
+
   /**
    * Add a new column.
    * @param colSchema the schema of the new column
