@@ -21,7 +21,6 @@ import java.util.List;
 import java.util.Map;
 
 import com.google.common.base.Preconditions;
-import com.google.common.base.Strings;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Lists;
 import org.apache.yetus.audience.InterfaceAudience;
@@ -260,6 +259,17 @@ public class CreateTableOptions {
     return this;
   }
 
+  /**
+   * Set the table comment.
+   *
+   * @param comment the table comment
+   * @return this instance
+   */
+  public CreateTableOptions setComment(String comment) {
+    pb.setComment(comment);
+    return this;
+  }
+
   Master.CreateTableRequestPB.Builder getBuilder() {
     if (!splitRows.isEmpty() || !rangePartitions.isEmpty()) {
       pb.setSplitRowsRangeBounds(new Operation.OperationsEncoder()
