@@ -59,6 +59,15 @@ public class KuduScanner implements Iterable<RowResult> {
     asyncScanner.setReuseRowResult(reuseRowResult);
   }
 
+  /**
+   * Optionally set expected row data format.
+   *
+   * @param rowDataFormat Row data format to be expected.
+   */
+  public void setRowDataFormat(AsyncKuduScanner.RowDataFormat rowDataFormat) {
+    asyncScanner.setRowDataFormat(rowDataFormat);
+  }
+
   /**
    * Scans a number of rows.
    * <p>
