@@ -43,6 +43,7 @@ import org.junit.Test;
 import org.apache.kudu.client.Client.AuthenticationCredentialsPB;
 import org.apache.kudu.master.Master.ConnectToMasterResponsePB;
 import org.apache.kudu.test.CapturingLogAppender;
+import org.apache.kudu.test.KuduTestHarness;
 import org.apache.kudu.test.cluster.FakeDNS;
 import org.apache.kudu.test.cluster.MiniKuduCluster;
 import org.apache.kudu.test.cluster.MiniKuduCluster.MiniKuduClusterBuilder;
@@ -55,6 +56,10 @@ public class TestSecurity {
   private static final String TABLE_NAME = "TestSecurity-table";
   private static final int TICKET_LIFETIME_SECS = 10;
   private static final int RENEWABLE_LIFETIME_SECS = 20;
+  public static final String CUSTOM_PRINCIPAL = "oryx";
+
+  @Rule
+  public KuduTestHarness harness = new KuduTestHarness();
 
   private CapturingLogAppender cla;
   private MiniKuduCluster miniCluster;
@@ -64,6 +69,7 @@ public class TestSecurity {
     LONG_LEADER_ELECTION,
     SHORT_TOKENS_AND_TICKETS,
     START_TSERVERS,
+    CUSTOM_PRINCIPAL,
   }
 
   private static class KeyValueMessage {
@@ -89,6 +95,9 @@ public class TestSecurity {
               .kdcRenewLifetime(RENEWABLE_LIFETIME_SECS + "s")
               .kdcTicketLifetime(TICKET_LIFETIME_SECS + "s");
     }
+    if (opts.contains(Option.CUSTOM_PRINCIPAL)) {
+      mcb.principal(CUSTOM_PRINCIPAL);
+    }
     miniCluster = mcb.numMasterServers(3)
         .numTabletServers(opts.contains(Option.START_TSERVERS) ? 3 : 0)
         .build();
@@ -491,4 +500,28 @@ public class TestSecurity {
       }
     }
   }
+
+  @Test(timeout = 60000)
+  @KuduTestHarness.EnableKerberos(principal = CUSTOM_PRINCIPAL)
+  public void testNonDefaultPrincipal() throws Exception {
+    try {
+      KuduClient client = new KuduClient.KuduClientBuilder(harness.getMasterAddressesAsString())
+          .build();
+      client.createTable("TestSecurity-nondefault-principal-1",
+          getBasicSchema(),
+          getBasicCreateTableOptions());
+      Assert.fail("default client shouldn't be able to connect to the cluster.");
+    } catch (NonRecoverableException e) {
+      Assert.assertThat(e.getMessage(), CoreMatchers.containsString(
+          "this client is not authenticated"
+      ));
+    }
+    KuduClient client = new KuduClient.KuduClientBuilder(harness.getMasterAddressesAsString())
+            .saslProtocolName(CUSTOM_PRINCIPAL)
+            .build();
+    Assert.assertNotNull(client.createTable( "TestSecurity-nondefault-principal-2",
+        getBasicSchema(),
+        getBasicCreateTableOptions()));
+  }
+
 }
