@@ -130,6 +130,13 @@ public class KuduTestHarness extends ExternalResource {
       }
     }
 
+    // Enable Kerberos if needed and set the SPN.
+    EnableKerberos enableKerberos = description.getAnnotation(EnableKerberos.class);
+    if (enableKerberos != null) {
+      clusterBuilder.enableKerberos();
+      clusterBuilder.principal(enableKerberos.principal());
+    }
+
     // Generate the ExternalResource Statement.
     Statement statement = super.apply(base, description);
     // Wrap in the RetryRule to rerun flaky tests.
@@ -146,6 +153,7 @@ public class KuduTestHarness extends ExternalResource {
     LOG.info("Creating a new Kudu client...");
     asyncClient = new AsyncKuduClientBuilder(miniCluster.getMasterAddressesAsString())
         .defaultAdminOperationTimeoutMs(DEFAULT_SLEEP)
+        .saslProtocolName(miniCluster.getPrincipal())
         .build();
     client = asyncClient.syncClient();
   }
@@ -201,37 +209,42 @@ public class KuduTestHarness extends ExternalResource {
   }
 
   /**
-   * Helper method to kill a tablet server that serves the given tablet's
-   * leader. The currently running test case will be failed if the tablet has no
-   * leader after some retries, or if the tablet server was already killed.
-   *
+   * Helper method to kill a tablet server that hosts the given tablet's leader
+   * replica.
    * This method is thread-safe.
+   *
    * @param tablet a RemoteTablet which will get its leader killed
-   * @throws Exception
+   * @return the host and port of the tablet server which hosted the tablet's
+   *         leader replica
+   * @throws Exception if no leader replica found after a few retries,
+   *         or if the tablet server isn't running
    */
-  public void killTabletLeader(RemoteTablet tablet) throws Exception {
-    killTabletLeader(new LocatedTablet(tablet));
+  public HostAndPort killTabletLeader(RemoteTablet tablet) throws Exception {
+    return killTabletLeader(new LocatedTablet(tablet));
   }
 
   /**
-   * Helper method to kill a tablet server that serves the given tablet's
-   * leader. The currently running test case will be failed if the tablet has no
-   * leader after some retries, or if the tablet server was already killed.
+   * Helper method to kill a tablet server that serves the given tablet's leader
+   * replica.
    *
    * This method is thread-safe.
    * @param tablet a LocatedTablet which will get its leader killed
-   * @throws Exception
+   * @return the host and port of the tablet server which hosted the tablet's
+   *         leader replica
+   * @throws Exception if no leader replica found or if the tablet server isn't
+   *                   running
    */
-  public void killTabletLeader(LocatedTablet tablet) throws Exception {
+  public HostAndPort killTabletLeader(LocatedTablet tablet) throws Exception {
     HostAndPort hp = findLeaderTabletServer(tablet);
     miniCluster.killTabletServer(hp);
+    return hp;
   }
 
   /**
    * Finds the RPC port of the given tablet's leader tserver.
    * @param tablet a LocatedTablet
    * @return the host and port of the given tablet's leader tserver
-   * @throws Exception if we are unable to find the leader tserver
+   * @throws Exception if unable to find a tablet server with leader replica
    */
   public HostAndPort findLeaderTabletServer(LocatedTablet tablet)
       throws Exception {
@@ -254,14 +267,14 @@ public class KuduTestHarness extends ExternalResource {
   }
 
   /**
-   * Helper method to easily kill the leader master.
+   * Start tablet server which has previously been registered at the specified
+   * host and port.
    *
-   * This method is thread-safe.
-   * @throws Exception if there is an error finding or killing the leader master.
+   * @param hp host and port of the tablet server to start back
+   * @throws Exception
    */
-  public void killLeaderMasterServer() throws Exception {
-    HostAndPort hp = findLeaderMasterServer();
-    miniCluster.killMasterServer(hp);
+  public void startTabletServer(HostAndPort hp) throws Exception {
+    miniCluster.startTabletServer(hp);
   }
 
   /**
@@ -273,6 +286,19 @@ public class KuduTestHarness extends ExternalResource {
     return client.findLeaderMasterServer();
   }
 
+  /**
+   * Helper method to easily kill the leader master.
+   *
+   * This method is thread-safe.
+   * @return the host and port of the detected leader master
+   * @throws Exception if there is an error finding or killing the leader master.
+   */
+  public HostAndPort killLeaderMasterServer() throws Exception {
+    HostAndPort hp = findLeaderMasterServer();
+    miniCluster.killMasterServer(hp);
+    return hp;
+  }
+
   /**
    * Picks at random a tablet server that serves tablets from the passed table and restarts it.
    * @param table table to query for a TS to restart
@@ -296,22 +322,64 @@ public class KuduTestHarness extends ExternalResource {
   /**
    * Kills a tablet server that serves the given tablet's leader and restarts it.
    * @param tablet a RemoteTablet which will get its leader killed and restarted
+   * @return the host and port of the restarted tablet server
    * @throws Exception
    */
-  public void restartTabletServer(RemoteTablet tablet) throws Exception {
+  public HostAndPort restartTabletServer(RemoteTablet tablet) throws Exception {
     HostAndPort hp = findLeaderTabletServer(new LocatedTablet(tablet));
     miniCluster.killTabletServer(hp);
     miniCluster.startTabletServer(hp);
+    return hp;
   }
 
   /**
    * Kills and restarts the leader master.
+   * @return the host and port of the restarted master
    * @throws Exception
    */
-  public void restartLeaderMaster() throws Exception {
+  public HostAndPort restartLeaderMaster() throws Exception {
     HostAndPort hp = findLeaderMasterServer();
     miniCluster.killMasterServer(hp);
     miniCluster.startMasterServer(hp);
+    return hp;
+  }
+
+  /**
+   * Start master which has previously been registered at the specified
+   * host and port.
+   *
+   * @param hp host and port of the master to start back
+   * @throws Exception
+   */
+  public void startMaster(HostAndPort hp) throws Exception {
+    miniCluster.startMasterServer(hp);
+  }
+
+  /**
+   * Finds and pauses the leader master.
+   * @return the host and port of the paused master
+   * @throws Exception
+   */
+  public HostAndPort pauseLeaderMaster() throws Exception {
+    HostAndPort hp = findLeaderMasterServer();
+    miniCluster.pauseMasterServer(hp);
+    return hp;
+  }
+
+  /**
+   * Pauses the specified master.
+   * @throws Exception
+   */
+  public void pauseMaster(HostAndPort hp) throws Exception {
+    miniCluster.pauseMasterServer(hp);
+  }
+
+  /**
+   * Resumes the specified master.
+   * @throws Exception
+   */
+  public void resumeMaster(HostAndPort hp) throws Exception {
+    miniCluster.resumeMasterServer(hp);
   }
 
   /**
@@ -334,7 +402,7 @@ public class KuduTestHarness extends ExternalResource {
    * @return the list of tablet servers
    */
   public List<HostAndPort> getTabletServers() {
-    return miniCluster.getMasterServers();
+    return miniCluster.getTabletServers();
   }
 
   /**
@@ -344,6 +412,13 @@ public class KuduTestHarness extends ExternalResource {
     return miniCluster.getClusterRoot();
   }
 
+  /**
+   * @return the service principal name
+   */
+  public String getPrincipal() {
+    return miniCluster.getPrincipal();
+  }
+
   /**
    * Kills all the master servers.
    * Does nothing to the servers that are already dead.
@@ -460,4 +535,14 @@ public class KuduTestHarness extends ExternalResource {
   public @interface LocationConfig {
     String[] locations();
   }
+
+  /**
+   * An annotation that can be added to each test method to enable Kerberos.
+   * The service principal name can be configured by specifying 'principal'.
+   */
+  @Retention(RetentionPolicy.RUNTIME)
+  @Target({ElementType.METHOD})
+  public @interface EnableKerberos {
+    String principal() default "kudu";
+  }
 }
