@@ -18,6 +18,7 @@
 package org.apache.kudu.test.junit;
 
 import static org.junit.Assert.fail;
+import static org.junit.Assume.assumeTrue;
 
 import org.junit.Rule;
 import org.junit.Test;
@@ -45,4 +46,12 @@ public class TestRetryRule {
     // Pass the test (by not throwing) on the final retry.
   }
 
+  // Ensure that the RetryRule does not cause test failures when
+  // assumeTrue and other similar assumption statements are used.
+  @Test
+  public void testAssumeTrue() {
+    assumeTrue(false);
+    fail("This is unreachable!");
+  }
+
 }
