@@ -19,13 +19,13 @@
 
 #include <algorithm>
 #include <cstdint>
-#include <cstdlib>
 #include <functional>
 #include <limits>
 #include <map>
 #include <memory>
 #include <mutex>
 #include <ostream>
+#include <random>
 #include <string>
 #include <utility>
 #include <vector>
@@ -183,6 +183,22 @@ KuduClient::Data::~Data() {
   dns_resolver_.reset();
 }
 
+namespace {
+
+// This random integer is used when making any random choice for replica
+// selection. It is static to provide a deterministic selection for any given
+// process and therefore also better cache affinity while ensuring that we can
+// still benefit from spreading the load across replicas for other processes
+// and applications.
+int InitRandomSelectionInt() {
+  std::random_device rdev;
+  std::mt19937 gen(rdev());
+  return gen();
+}
+static const int kRandomSelectionInt = InitRandomSelectionInt();
+
+} // anonymous namespace
+
 RemoteTabletServer* KuduClient::Data::SelectTServer(
     const scoped_refptr<RemoteTablet>& rt,
     const ReplicaSelection selection,
@@ -249,11 +265,11 @@ RemoteTabletServer* KuduClient::Data::SelectTServer(
         }
       }
       if (!local.empty()) {
-        ret = local[rand() % local.size()];
+        ret = local[kRandomSelectionInt % local.size()];
       } else if (!same_location.empty()) {
-        ret = same_location[rand() % same_location.size()];
+        ret = same_location[kRandomSelectionInt % same_location.size()];
       } else if (!filtered.empty()) {
-        ret = filtered[rand() % filtered.size()];
+        ret = filtered[kRandomSelectionInt % filtered.size()];
       }
       break;
     }
@@ -501,6 +517,7 @@ Status KuduClient::Data::OpenTable(KuduClient* client,
   string table_name;
   int num_replicas;
   string owner;
+  string comment;
   PartitionSchema partition_schema;
   map<string, string> extra_configs;
   MonoTime deadline = MonoTime::Now() + default_admin_operation_timeout_;
@@ -513,6 +530,7 @@ Status KuduClient::Data::OpenTable(KuduClient* client,
                                &table_name,
                                &num_replicas,
                                &owner,
+                               &comment,
                                &extra_configs));
 
   // When the table name is specified, use the caller-provided table name.
@@ -526,7 +544,7 @@ Status KuduClient::Data::OpenTable(KuduClient* client,
   //                   map to reuse KuduTable instances.
   table->reset(new KuduTable(client->shared_from_this(),
                              effective_table_name, table_id, num_replicas,
-                             owner, schema, partition_schema, extra_configs));
+                             owner, comment, schema, partition_schema, extra_configs));
 
   // When opening a table, clear the existing cached non-covered range entries.
   // This avoids surprises where a new table instance won't be able to see the
@@ -545,6 +563,7 @@ Status KuduClient::Data::GetTableSchema(KuduClient* client,
                                         std::string* table_name,
                                         int* num_replicas,
                                         std::string* owner,
+                                        std::string* comment,
                                         map<string, string>* extra_configs) {
   GetTableSchemaRequestPB req;
   GetTableSchemaResponsePB resp;
@@ -588,6 +607,9 @@ Status KuduClient::Data::GetTableSchema(KuduClient* client,
   if (owner) {
     *owner = resp.owner();
   }
+  if (comment) {
+    *comment = resp.comment();
+  }
   if (extra_configs) {
     map<string, string> result(resp.extra_configs().begin(), resp.extra_configs().end());
     *extra_configs = std::move(result);
