@@ -16,8 +16,7 @@
 // under the License.
 //
 // This module is internal to the client and not a public API.
-#ifndef KUDU_CLIENT_META_CACHE_H
-#define KUDU_CLIENT_META_CACHE_H
+#pragma once
 
 #include <atomic>
 #include <map>
@@ -50,14 +49,15 @@ namespace kudu {
 class Sockaddr;
 
 namespace tserver {
-class TabletServerServiceProxy;
 class TabletServerAdminServiceProxy;
+class TabletServerServiceProxy;
 } // namespace tserver
 
 namespace master {
+class GetTableLocationsResponsePB;
+class GetTabletLocationsResponsePB;
 class TSInfoPB;
 class TabletLocationsPB;
-class GetTableLocationsResponsePB;
 } // namespace master
 
 namespace client {
@@ -146,6 +146,8 @@ struct RemoteReplica {
   bool failed;
 };
 
+typedef std::unordered_map<std::string, std::unique_ptr<RemoteTabletServer>>
+    TabletServerRegistry;
 typedef std::unordered_map<std::string, RemoteTabletServer*> TabletServerMap;
 
 // A ServerPicker for tablets servers, backed by the MetaCache.
@@ -188,7 +190,8 @@ class MetaCacheServerPicker : public rpc::ServerPicker<RemoteTabletServer> {
   // A ref to the meta cache.
   scoped_refptr<MetaCache> meta_cache_;
 
-  // The table we're writing to.
+  // The table we're writing to. If null, relies on tablet ID-based lookups
+  // instead of partition key-based lookups.
   const KuduTable* table_;
 
   // The tablet we're picking replicas for.
@@ -366,13 +369,12 @@ class MetaCacheEntry {
   // metadata.
   std::string DebugString(const KuduTable* table) const;
 
- private:
-
   // Returns true if the entry is initialized.
   bool Initialized() const {
     return expiration_time_.Initialized();
   }
 
+ private:
   // The expiration time of this cached entry.
   MonoTime expiration_time_;
 
@@ -394,7 +396,7 @@ class MetaCache : public RefCountedThreadSafe<MetaCache> {
  public:
   // The passed 'client' object must remain valid as long as MetaCache is alive.
   MetaCache(KuduClient* client, ReplicaController::Visibility replica_visibility);
-  ~MetaCache();
+  ~MetaCache() = default;
 
   // Determines what type of operation a MetaCache lookup is being done for.
   enum class LookupType {
@@ -423,12 +425,30 @@ class MetaCache : public RefCountedThreadSafe<MetaCache> {
                          scoped_refptr<RemoteTablet>* remote_tablet,
                          const StatusCallback& callback);
 
+  // Look up the locations of the given tablet, storing the result in
+  // 'remote_tablet' if not null, and calling 'lookup_complete_cb' once the
+  // lookup is complete. Only tablets with non-failed LEADERs are considered.
+  //
+  // NOTE: the callback may be called from an IO thread or inline with this
+  // call if the cached data is already available.
+  void LookupTabletById(KuduClient* client,
+                        const std::string& tablet_id,
+                        const MonoTime& deadline,
+                        scoped_refptr<RemoteTablet>* remote_tablet,
+                        const StatusCallback& lookup_complete_cb);
+
   // Lookup the given tablet by key, only consulting local information.
   // Returns true and sets *entry if successful.
   bool LookupEntryByKeyFastPath(const KuduTable* table,
                                 const std::string& partition_key,
                                 MetaCacheEntry* entry);
+  // Lookup the given tablet by tablet ID, only consulting local information.
+  // Returns true and sets *entry if successful.
+  bool LookupEntryByIdFastPath(const std::string& tablet_id,
+                               MetaCacheEntry* entry);
 
+  // Process the response for the given key-based lookup parameters, indexing
+  // the location information as appropriate.
   Status ProcessGetTableLocationsResponse(const KuduTable* table,
                                           const std::string& partition_key,
                                           bool is_exact_lookup,
@@ -459,6 +479,7 @@ class MetaCache : public RefCountedThreadSafe<MetaCache> {
 
  private:
   friend class LookupRpc;
+  friend class LookupRpcById;
 
   FRIEND_TEST(client::ClientTest, TestMasterLookupPermits);
   FRIEND_TEST(client::ClientTest, TestMetaCacheExpiry);
@@ -469,10 +490,16 @@ class MetaCache : public RefCountedThreadSafe<MetaCache> {
                                MetaCacheEntry* cache_entry,
                                int max_returned_locations);
 
+  // Process the response for the given id-based lookup parameters, indexing
+  // the location information as appropriate.
+  Status ProcessGetTabletLocationsResponse(const std::string& tablet_id,
+                                           const master::GetTabletLocationsResponsePB& resp,
+                                           MetaCacheEntry* cache_entry);
+
   // Perform the complete fast-path lookup. Returns:
-  //  - NotFound if the lookup hits a non-covering range.
-  //  - Incomplete if the fast path was not possible
-  //  - OK if the lookup was successful.
+  // - NotFound if the lookup hits a non-covering range.
+  // - Incomplete if the fast path was not possible
+  // - OK if the lookup was successful.
   //
   // If 'lookup_type' is kLowerBound, then 'partition_key' will be updated to indicate the
   // start of the range for the matched tablet.
@@ -481,38 +508,69 @@ class MetaCache : public RefCountedThreadSafe<MetaCache> {
                           LookupType lookup_type,
                           scoped_refptr<RemoteTablet>* remote_tablet);
 
+  // Perform the fast-path lookup by tablet ID. Returns:
+  // - Incomplete if there was no cache entry
+  // - OK if the lookup was successful
+  //
+  // If 'remote_tablet' isn't null, it is populated with a pointer to the
+  // RemoteTablet being looked up. Otherwise, just does the lookup, priming the
+  // cache with the location.
+  Status DoFastPathLookupById(const std::string& tablet_id,
+                              scoped_refptr<RemoteTablet>* remote_tablet);
+
   // Update our information about the given tablet server.
   //
   // This is called when we get some response from the master which contains
   // the latest host/port info for a server.
   //
   // NOTE: Must be called with lock_ held.
-  void UpdateTabletServer(const master::TSInfoPB& pb);
+  void UpdateTabletServerUnlocked(const master::TSInfoPB& pb);
 
   KuduClient* client_;
 
   percpu_rwlock lock_;
 
-  // Cache of Tablet Server locations: TS UUID -> RemoteTabletServer*.
+  // Registry of all tablet servers as a map of tablet server's
+  // UUID -> std::unique_ptr<RemoteTabletServer>.
   //
-  // Given that the set of tablet servers is bounded by physical machines, we never
-  // evict entries from this map until the MetaCache is destructed. So, no need to use
-  // shared_ptr, etc.
+  // Given that the set of tablet servers in a cluster is bounded by physical
+  // machines and every tablet server has its unique identifier, we never remove
+  // entries from this map until the MetaCache is destructed. Note that the
+  // ClearCache() method doesn't touch this registry, but updates ts_cache_ map
+  // below which contains raw pointers to the elements in this registry.
+  // So, there is no need to use shared_ptr and alike for the entries.
+  //
+  // Protected by lock_.
+  TabletServerRegistry ts_registry_;
+
+  // Cache of Tablet Server locations: TS UUID -> RemoteTabletServer*.
+  // The cache can be cleared by the ClearCache() method.
   //
   // Protected by lock_.
   TabletServerMap ts_cache_;
 
-  // Cache of tablets, keyed by partition key.
+  // Cache entries for tablets and non-covered ranges, keyed by table ID, used
+  // for key-based lookups.
   //
   // Protected by lock_.
   typedef std::map<std::string, MetaCacheEntry> TabletMap;
-
-  // Cache of tablets and non-covered ranges, keyed by table id.
-  //
-  // Protected by lock_.
   std::unordered_map<std::string, TabletMap> tablets_by_table_and_key_;
 
-  // Cache of tablets, keyed by tablet ID.
+  // Cache entries for tablets, keyed by tablet ID, used for ID-based lookups.
+  // NOTE: existence in 'tablets_by_table_and_key' does not imply existence in
+  // 'entry_by_tablet_id_', and vice versa.
+  //
+  // Protected by lock_.
+  //
+  // TODO(awong): it might be nice for ID-based lookups and table-based lookups
+  // to use the same entries. It's currently tricky to do so since ID-based
+  // lookups don't incur any table metadata, making lookups by table ID tricky.
+  std::unordered_map<std::string, MetaCacheEntry> entry_by_tablet_id_;
+
+  // The underlying remote tablets pointed to by the above cache entry
+  // containers, keyed by tablet ID. If an entry does not exist for a given
+  // tablet ID in this container, none can exist in either of the above
+  // containers.
   //
   // Protected by lock_
   std::unordered_map<std::string, scoped_refptr<RemoteTablet>> tablets_by_id_;
@@ -530,4 +588,3 @@ class MetaCache : public RefCountedThreadSafe<MetaCache> {
 } // namespace internal
 } // namespace client
 } // namespace kudu
-#endif /* KUDU_CLIENT_META_CACHE_H */
