@@ -45,6 +45,7 @@
 #include "kudu/common/wire_protocol.pb.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/stl_util.h"
+#include "kudu/gutil/strings/substitute.h"
 #include "kudu/master/master.h"
 #include "kudu/master/mini_master.h"
 #include "kudu/mini-cluster/internal_mini_cluster.h"
@@ -63,24 +64,23 @@ DECLARE_bool(tserver_enforce_access_control);
 METRIC_DECLARE_histogram(handler_latency_kudu_master_MasterService_GetTableSchema);
 METRIC_DECLARE_histogram(handler_latency_kudu_master_MasterService_GetTableLocations);
 
-namespace kudu {
-namespace client {
-
-using cluster::InternalMiniCluster;
-using cluster::InternalMiniClusterOptions;
-using sp::shared_ptr;
+using kudu::client::KuduTableCreator;
+using kudu::client::sp::shared_ptr;
+using kudu::cluster::InternalMiniCluster;
+using kudu::cluster::InternalMiniClusterOptions;
+using kudu::tserver::MiniTabletServer;
 using std::atomic;
 using std::string;
 using std::thread;
 using std::unique_ptr;
 using std::unordered_set;
 using std::vector;
-using tserver::MiniTabletServer;
+
+namespace kudu {
+namespace client {
 
 class ScanTokenTest : public KuduTest {
-
  protected:
-
   void SetUp() override {
     // Enable access control so we can validate the requests in secure environment.
     // Specifically that authz tokens in the scan tokens work.
@@ -104,13 +104,12 @@ class ScanTokenTest : public KuduTest {
 
       threads.emplace_back([this, &rows] (string serialized_token) {
         shared_ptr<KuduClient> client;
-        ASSERT_OK(cluster_->CreateClient(nullptr, &client));
+        CHECK_OK(cluster_->CreateClient(nullptr, &client));
         KuduScanner* scanner_ptr;
-        ASSERT_OK(KuduScanToken::DeserializeIntoScanner(client.get(),
-                                                        serialized_token,
-                                                        &scanner_ptr));
+        CHECK_OK(KuduScanToken::DeserializeIntoScanner(
+            client.get(), serialized_token, &scanner_ptr));
         unique_ptr<KuduScanner> scanner(scanner_ptr);
-        ASSERT_OK(scanner->Open());
+        CHECK_OK(scanner->Open());
 
         while (scanner->HasMoreRows()) {
           KuduScanBatch batch;
@@ -128,6 +127,28 @@ class ScanTokenTest : public KuduTest {
     return rows;
   }
 
+  // Similar to CountRows() above, but use the specified client handle
+  // and run all the scanners sequentially, one by one.
+  Status CountRowsSeq(KuduClient* client,
+                      vector<KuduScanToken*> tokens,
+                      int64_t* row_count) {
+    int64_t count = 0;
+    for (auto* t : tokens) {
+      unique_ptr<KuduScanToken> token(t);
+      unique_ptr<KuduScanner> scanner;
+      RETURN_NOT_OK(IntoUniqueScanner(client, *token, &scanner));
+
+      RETURN_NOT_OK(scanner->Open());
+      while (scanner->HasMoreRows()) {
+        KuduScanBatch batch;
+        RETURN_NOT_OK(scanner->NextBatch(&batch));
+        count += batch.NumRows();
+      }
+    }
+    *row_count = count;
+    return Status::OK();
+  }
+
   void VerifyTabletInfo(const vector<KuduScanToken*>& tokens) {
     unordered_set<string> tablet_ids;
     for (auto t : tokens) {
@@ -144,8 +165,7 @@ class ScanTokenTest : public KuduTest {
       // Check that the tserver associated with the replica is the sole tserver
       // started for this cluster.
       const MiniTabletServer* ts = cluster_->mini_tablet_server(0);
-      ASSERT_EQ(ts->server()->instance_pb().permanent_uuid(),
-                r->ts().uuid());
+      ASSERT_EQ(ts->server()->instance_pb().permanent_uuid(), r->ts().uuid());
       ASSERT_EQ(ts->bound_rpc_addr().host(), r->ts().hostname());
       ASSERT_EQ(ts->bound_rpc_addr().port(), r->ts().port());
     }
@@ -157,15 +177,28 @@ class ScanTokenTest : public KuduTest {
                                   const KuduScanToken& token,
                                   unique_ptr<KuduScanner>* scanner_ptr) {
     string serialized_token;
-    CHECK_OK(token.Serialize(&serialized_token));
+    RETURN_NOT_OK(token.Serialize(&serialized_token));
     KuduScanner* scanner_ptr_raw;
-    RETURN_NOT_OK(KuduScanToken::DeserializeIntoScanner(client,
-                                                        serialized_token,
-                                                        &scanner_ptr_raw));
+    RETURN_NOT_OK(KuduScanToken::DeserializeIntoScanner(
+        client, serialized_token, &scanner_ptr_raw));
     scanner_ptr->reset(scanner_ptr_raw);
     return Status::OK();
   }
 
+  // Create a table with the specified name and schema with replicaction factor
+  // of one and empty list of range partitions.
+  Status CreateAndOpenTable(const string& table_name,
+                            const KuduSchema& schema,
+                            shared_ptr<KuduTable>* table) {
+    unique_ptr<KuduTableCreator> table_creator(client_->NewTableCreator());
+    RETURN_NOT_OK(table_creator->table_name(table_name)
+                  .schema(&schema)
+                  .set_range_partition_columns({})
+                  .num_replicas(1)
+                  .Create());
+    return client_->OpenTable(table_name, table);
+  }
+
   uint64_t NumGetTableSchemaRequests() const {
     const auto& ent = cluster_->mini_master()->master()->metric_entity();
     return METRIC_handler_latency_kudu_master_MasterService_GetTableSchema
@@ -196,15 +229,15 @@ TEST_F(ScanTokenTest, TestScanTokens) {
   {
     unique_ptr<KuduPartialRow> split(schema.NewRow());
     ASSERT_OK(split->SetInt64("col", 0));
-    unique_ptr<client::KuduTableCreator> table_creator(client_->NewTableCreator());
+    unique_ptr<KuduTableCreator> table_creator(client_->NewTableCreator());
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
     ASSERT_OK(table_creator->table_name("table")
-                            .schema(&schema)
-                            .add_hash_partitions({ "col" }, 4)
-                            .split_rows({ split.release() })
-                            .num_replicas(1)
-                            .Create());
+              .schema(&schema)
+              .add_hash_partitions({ "col" }, 4)
+              .split_rows({ split.release() })
+              .num_replicas(1)
+              .Create());
 #pragma GCC diagnostic pop
     ASSERT_OK(client_->OpenTable("table", &table));
   }
@@ -216,9 +249,9 @@ TEST_F(ScanTokenTest, TestScanTokens) {
 
   // Insert rows
   for (int i = -100; i < 100; i++) {
-      unique_ptr<KuduInsert> insert(table->NewInsert());
-      ASSERT_OK(insert->mutable_row()->SetInt64("col", i));
-      ASSERT_OK(session->Apply(insert.release()));
+    unique_ptr<KuduInsert> insert(table->NewInsert());
+    ASSERT_OK(insert->mutable_row()->SetInt64("col", i));
+    ASSERT_OK(session->Apply(insert.release()));
   }
   ASSERT_OK(session->Flush());
 
@@ -286,9 +319,8 @@ TEST_F(ScanTokenTest, TestScanTokens) {
     vector<KuduScanToken*> tokens;
     ElementDeleter deleter(&tokens);
     KuduScanTokenBuilder builder(table.get());
-    unique_ptr<KuduPredicate> predicate(table->NewComparisonPredicate("col",
-                                                                      KuduPredicate::GREATER_EQUAL,
-                                                                      KuduValue::FromInt(0)));
+    unique_ptr<KuduPredicate> predicate(table->NewComparisonPredicate(
+        "col", KuduPredicate::GREATER_EQUAL, KuduValue::FromInt(0)));
     ASSERT_OK(builder.AddConjunctPredicate(predicate.release()));
     ASSERT_OK(builder.Build(&tokens));
 
@@ -301,9 +333,8 @@ TEST_F(ScanTokenTest, TestScanTokens) {
     vector<KuduScanToken*> tokens;
     ElementDeleter deleter(&tokens);
     KuduScanTokenBuilder builder(table.get());
-    unique_ptr<KuduPredicate> predicate(table->NewComparisonPredicate("col",
-                                                                      KuduPredicate::EQUAL,
-                                                                      KuduValue::FromInt(42)));
+    unique_ptr<KuduPredicate> predicate(table->NewComparisonPredicate(
+        "col", KuduPredicate::EQUAL, KuduValue::FromInt(42)));
     ASSERT_OK(builder.AddConjunctPredicate(predicate.release()));
     ASSERT_OK(builder.Build(&tokens));
 
@@ -365,10 +396,12 @@ TEST_F(ScanTokenTest, TestScanTokens) {
       string buf;
       ASSERT_OK(token->Serialize(&buf));
       KuduScanner* scanner_raw;
-      ASSERT_OK(KuduScanToken::DeserializeIntoScanner(client_.get(), buf, &scanner_raw));
+      ASSERT_OK(KuduScanToken::DeserializeIntoScanner(
+          client_.get(), buf, &scanner_raw));
       unique_ptr<KuduScanner> scanner(scanner_raw); // Caller gets ownership of the scanner.
       // Ensure the timeout configuration gets carried through the serialization process.
-      ASSERT_EQ(kTimeoutMillis, scanner->data_->configuration().timeout().ToMilliseconds());
+      ASSERT_EQ(kTimeoutMillis,
+                scanner->data_->configuration().timeout().ToMilliseconds());
     }
 
     ASSERT_GE(8, tokens.size());
@@ -391,7 +424,7 @@ TEST_F(ScanTokenTest, TestScanTokensWithNonCoveringRange) {
   {
     unique_ptr<KuduPartialRow> lower_bound(schema.NewRow());
     unique_ptr<KuduPartialRow> upper_bound(schema.NewRow());
-    unique_ptr<client::KuduTableCreator> table_creator(client_->NewTableCreator());
+    unique_ptr<KuduTableCreator> table_creator(client_->NewTableCreator());
     table_creator->table_name("table");
     table_creator->num_replicas(1);
     table_creator->schema(&schema);
@@ -422,14 +455,14 @@ TEST_F(ScanTokenTest, TestScanTokensWithNonCoveringRange) {
 
   // Insert rows
   for (int i = 0; i < 100; i++) {
-      unique_ptr<KuduInsert> insert(table->NewInsert());
-      ASSERT_OK(insert->mutable_row()->SetInt64("col", i));
-      ASSERT_OK(session->Apply(insert.release()));
+    unique_ptr<KuduInsert> insert(table->NewInsert());
+    ASSERT_OK(insert->mutable_row()->SetInt64("col", i));
+    ASSERT_OK(session->Apply(insert.release()));
   }
   for (int i = 200; i < 400; i++) {
-      unique_ptr<KuduInsert> insert(table->NewInsert());
-      ASSERT_OK(insert->mutable_row()->SetInt64("col", i));
-      ASSERT_OK(session->Apply(insert.release()));
+    unique_ptr<KuduInsert> insert(table->NewInsert());
+    ASSERT_OK(insert->mutable_row()->SetInt64("col", i));
+    ASSERT_OK(session->Apply(insert.release()));
   }
   ASSERT_OK(session->Flush());
 
@@ -447,9 +480,8 @@ TEST_F(ScanTokenTest, TestScanTokensWithNonCoveringRange) {
     vector<KuduScanToken*> tokens;
     ElementDeleter deleter(&tokens);
     KuduScanTokenBuilder builder(table.get());
-    unique_ptr<KuduPredicate> predicate(table->NewComparisonPredicate("col",
-                                                                      KuduPredicate::GREATER_EQUAL,
-                                                                      KuduValue::FromInt(200)));
+    unique_ptr<KuduPredicate> predicate(table->NewComparisonPredicate(
+        "col", KuduPredicate::GREATER_EQUAL, KuduValue::FromInt(200)));
     ASSERT_OK(builder.AddConjunctPredicate(predicate.release()));
     ASSERT_OK(builder.Build(&tokens));
 
@@ -462,9 +494,8 @@ TEST_F(ScanTokenTest, TestScanTokensWithNonCoveringRange) {
     vector<KuduScanToken*> tokens;
     ElementDeleter deleter(&tokens);
     KuduScanTokenBuilder builder(table.get());
-    unique_ptr<KuduPredicate> predicate(table->NewComparisonPredicate("col",
-                                                                      KuduPredicate::EQUAL,
-                                                                      KuduValue::FromInt(42)));
+    unique_ptr<KuduPredicate> predicate(table->NewComparisonPredicate(
+        "col", KuduPredicate::EQUAL, KuduValue::FromInt(42)));
     ASSERT_OK(builder.AddConjunctPredicate(predicate.release()));
     ASSERT_OK(builder.Build(&tokens));
 
@@ -515,16 +546,11 @@ TEST_F(ScanTokenTest, TestScanTokensWithNonCoveringRange) {
   }
 }
 
-const kudu::ReadMode read_modes[] = {
-    kudu::READ_LATEST,
-    kudu::READ_AT_SNAPSHOT,
-    kudu::READ_YOUR_WRITES,
-};
-
 class TimestampPropagationParamTest :
     public ScanTokenTest,
     public ::testing::WithParamInterface<kudu::ReadMode> {
 };
+
 // When building a scanner from a serialized scan token,
 // verify that the propagated timestamp from the token makes its way into the
 // latest observed timestamp of the client object.
@@ -549,7 +575,7 @@ TEST_P(TimestampPropagationParamTest, Test) {
     {
       unique_ptr<KuduPartialRow> split(schema.NewRow());
       ASSERT_OK(split->SetInt64(kKeyColumnName, 0));
-      unique_ptr<client::KuduTableCreator> creator(client_->NewTableCreator());
+      unique_ptr<KuduTableCreator> creator(client_->NewTableCreator());
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
       ASSERT_OK(creator->table_name(kTableName)
@@ -575,9 +601,8 @@ TEST_P(TimestampPropagationParamTest, Test) {
   EXPECT_EQ(ts_prev, client_->GetLatestObservedTimestamp());
 
   KuduScanner* scanner_raw;
-  ASSERT_OK(KuduScanToken::DeserializeIntoScanner(client_.get(),
-                                                  serialized_token,
-                                                  &scanner_raw));
+  ASSERT_OK(KuduScanToken::DeserializeIntoScanner(
+      client_.get(), serialized_token, &scanner_raw));
   // The caller of the DeserializeIntoScanner() is responsible for
   // de-allocating the result scanner object.
   unique_ptr<KuduScanner> scanner(scanner_raw);
@@ -606,8 +631,13 @@ TEST_P(TimestampPropagationParamTest, Test) {
   }
 }
 
-INSTANTIATE_TEST_CASE_P(Params, TimestampPropagationParamTest,
-                        testing::ValuesIn(read_modes));
+const kudu::ReadMode kReadModes[] = {
+  kudu::READ_LATEST,
+  kudu::READ_AT_SNAPSHOT,
+  kudu::READ_YOUR_WRITES,
+};
+INSTANTIATE_TEST_SUITE_P(Params, TimestampPropagationParamTest,
+                         testing::ValuesIn(kReadModes));
 
 // Tests the results of creating scan tokens, altering the columns being
 // scanned, and then executing the scan tokens.
@@ -624,15 +654,7 @@ TEST_F(ScanTokenTest, TestConcurrentAlterTable) {
 
   // Create table
   shared_ptr<KuduTable> table;
-  {
-    unique_ptr<client::KuduTableCreator> table_creator(client_->NewTableCreator());
-    ASSERT_OK(table_creator->table_name(kTableName)
-                            .schema(&schema)
-                            .set_range_partition_columns({})
-                            .num_replicas(1)
-                            .Create());
-    ASSERT_OK(client_->OpenTable(kTableName, &table));
-  }
+  ASSERT_OK(CreateAndOpenTable(kTableName, schema, &table));
 
   vector<KuduScanToken*> tokens;
   vector<KuduScanToken*> tokens_with_metadata;
@@ -703,7 +725,7 @@ TEST_F(ScanTokenTest, TestConcurrentAlterTable) {
 // Tests the results of creating scan tokens, renaming the table being
 // scanned, and then executing the scan tokens.
 TEST_F(ScanTokenTest, TestConcurrentRenameTable) {
-  const char* kTableName = "scan-token-rename";
+  constexpr const char* const kTableName = "scan-token-rename";
   // Create schema
   KuduSchema schema;
   {
@@ -715,15 +737,7 @@ TEST_F(ScanTokenTest, TestConcurrentRenameTable) {
 
   // Create table
   shared_ptr<KuduTable> table;
-  {
-    unique_ptr<client::KuduTableCreator> table_creator(client_->NewTableCreator());
-    ASSERT_OK(table_creator->table_name(kTableName)
-                            .schema(&schema)
-                            .set_range_partition_columns({})
-                            .num_replicas(1)
-                            .Create());
-    ASSERT_OK(client_->OpenTable(kTableName, &table));
-  }
+  ASSERT_OK(CreateAndOpenTable(kTableName, schema, &table));
 
   vector<KuduScanToken*> tokens;
   ASSERT_OK(KuduScanTokenBuilder(table.get()).Build(&tokens));
@@ -758,15 +772,7 @@ TEST_F(ScanTokenTest, TestMasterRequestsWithMetadata) {
 
   // Create table
   shared_ptr<KuduTable> table;
-  {
-    unique_ptr<client::KuduTableCreator> table_creator(client_->NewTableCreator());
-    ASSERT_OK(table_creator->table_name(kTableName)
-                            .schema(&schema)
-                            .set_range_partition_columns({})
-                            .num_replicas(1)
-                            .Create());
-    ASSERT_OK(client_->OpenTable(kTableName, &table));
-  }
+  ASSERT_OK(CreateAndOpenTable(kTableName, schema, &table));
 
   vector<KuduScanToken*> tokens;
   KuduScanTokenBuilder builder(table.get());
@@ -814,15 +820,7 @@ TEST_F(ScanTokenTest, TestMasterRequestsNoMetadata) {
 
   // Create table
   shared_ptr<KuduTable> table;
-  {
-    unique_ptr<client::KuduTableCreator> table_creator(client_->NewTableCreator());
-    ASSERT_OK(table_creator->table_name(kTableName)
-                  .schema(&schema)
-                  .set_range_partition_columns({})
-                  .num_replicas(1)
-                  .Create());
-    ASSERT_OK(client_->OpenTable(kTableName, &table));
-  }
+  ASSERT_OK(CreateAndOpenTable(kTableName, schema, &table));
 
   shared_ptr<KuduClient> new_client;
   ASSERT_OK(cluster_->CreateClient(nullptr, &new_client));
@@ -853,5 +851,214 @@ TEST_F(ScanTokenTest, TestMasterRequestsNoMetadata) {
   ASSERT_EQ(init_location_requests + 1, NumGetTableLocationsRequests());
 }
 
+enum FirstRangeChangeMode {
+  BEGIN = 0,
+  RANGE_DROPPED = 0,
+  RANGE_DROPPED_AND_PRECEDING_RANGE_ADDED = 1,
+  RANGE_DROPPED_AND_LARGER_ONE_ADDED = 2,
+  RANGE_DROPPED_AND_SMALLER_ONE_ADDED = 3,
+  RANGE_REPLACED_WITH_SAME = 4,
+  RANGE_REPLACED_WITH_SAME_AND_PRECEDING_RANGE_ADDED = 5,
+  RANGE_REPLACED_WITH_TWO_SMALLER_ONES = 6,
+  END = 7,
+};
+
+class StaleScanTokensParamTest :
+    public ScanTokenTest,
+    public ::testing::WithParamInterface<FirstRangeChangeMode> {
+};
+
+// Create scan tokens for one state of the table and store it for future use.
+// Use the tokens to scan the table. Alter the table dropping first range
+// partition, optionally replacing it according with FirstRangeChangeMode
+// enum. Open the altered table via the client handle which was used to run
+// the token-based scan prior. Now, attempt to scan the table using stale
+// tokens generated with the original state of the table.
+TEST_P(StaleScanTokensParamTest, DroppingFirstRange) {
+  constexpr const char* const kTableName = "stale-scan-tokens-dfr";
+  KuduSchema schema;
+  {
+    KuduSchemaBuilder builder;
+    builder.AddColumn("key")->
+        NotNull()->
+        Type(KuduColumnSchema::INT64)->
+        PrimaryKey();
+    ASSERT_OK(builder.Build(&schema));
+  }
+
+  shared_ptr<KuduTable> table;
+  {
+    unique_ptr<KuduTableCreator> table_creator(client_->NewTableCreator());
+    {
+      unique_ptr<KuduPartialRow> lower_bound(schema.NewRow());
+      ASSERT_OK(lower_bound->SetInt64("key", -100));
+      unique_ptr<KuduPartialRow> upper_bound(schema.NewRow());
+      ASSERT_OK(upper_bound->SetInt64("key", 0));
+      table_creator->add_range_partition(
+          lower_bound.release(), upper_bound.release());
+    }
+    {
+      unique_ptr<KuduPartialRow> lower_bound(schema.NewRow());
+      ASSERT_OK(lower_bound->SetInt64("key", 0));
+      unique_ptr<KuduPartialRow> upper_bound(schema.NewRow());
+      ASSERT_OK(upper_bound->SetInt64("key", 100));
+      table_creator->add_range_partition(
+          lower_bound.release(), upper_bound.release());
+    }
+
+    ASSERT_OK(table_creator->table_name(kTableName)
+        .schema(&schema)
+        .set_range_partition_columns({ "key" })
+        .num_replicas(1)
+        .Create());
+    ASSERT_OK(client_->OpenTable(kTableName, &table));
+  }
+
+  // Populate the table with data.
+  {
+    shared_ptr<KuduSession> session = client_->NewSession();
+    session->SetTimeoutMillis(10000);
+    ASSERT_OK(session->SetFlushMode(KuduSession::AUTO_FLUSH_SYNC));
+
+    for (int i = -50; i < 50; ++i) {
+      unique_ptr<KuduInsert> insert(table->NewInsert());
+      ASSERT_OK(insert->mutable_row()->SetInt64("key", i));
+      ASSERT_OK(session->Apply(insert.release()));
+    }
+  }
+
+  // Prepare two sets of scan tokens.
+  vector<KuduScanToken*> tokens_a;
+  {
+    KuduScanTokenBuilder builder(table.get());
+    ASSERT_OK(builder.IncludeTableMetadata(true));
+    ASSERT_OK(builder.IncludeTabletMetadata(true));
+    ASSERT_OK(builder.Build(&tokens_a));
+  }
+  ASSERT_EQ(2, tokens_a.size());
+
+  vector<KuduScanToken*> tokens_b;
+  {
+    KuduScanTokenBuilder builder(table.get());
+    ASSERT_OK(builder.IncludeTableMetadata(true));
+    ASSERT_OK(builder.IncludeTabletMetadata(true));
+    ASSERT_OK(builder.Build(&tokens_b));
+  }
+  ASSERT_EQ(2, tokens_b.size());
+
+  // Drop the first range partition, running the operation via the 'client_'
+  // handle.
+  {
+    unique_ptr<KuduPartialRow> lower_bound(schema.NewRow());
+    unique_ptr<KuduPartialRow> upper_bound(schema.NewRow());
+    ASSERT_OK(lower_bound->SetInt64("key", -100));
+    ASSERT_OK(upper_bound->SetInt64("key", 0));
+    unique_ptr<KuduTableAlterer> alterer(client_->NewTableAlterer(kTableName));
+    ASSERT_OK(alterer->DropRangePartition(
+        lower_bound.release(), upper_bound.release())->Alter());
+  }
+
+  shared_ptr<KuduClient> new_client;
+  ASSERT_OK(cluster_->CreateClient(nullptr, &new_client));
+
+  int64_t row_count_a = 0;
+  ASSERT_OK(CountRowsSeq(new_client.get(), std::move(tokens_a), &row_count_a));
+  ASSERT_EQ(50, row_count_a);
+
+  // Open the test table via 'new_client' handle to populate the metadata
+  // with actual table metadata, including non-covered ranges. This purges
+  // an entry for the [-100, 0) range from the 'tablet_by_key_' map, still
+  // keeping corresponding RemoteTable entry in the 'tablets_by_id_' map.
+  {
+    shared_ptr<KuduTable> t;
+    ASSERT_OK(new_client->OpenTable(kTableName, &t));
+  }
+
+  const auto range_adder = [&schema](
+      KuduClient* c, int64_t range_beg, int64_t range_end) {
+    unique_ptr<KuduPartialRow> lower_bound(schema.NewRow());
+    unique_ptr<KuduPartialRow> upper_bound(schema.NewRow());
+    RETURN_NOT_OK(lower_bound->SetInt64("key", range_beg));
+    RETURN_NOT_OK(upper_bound->SetInt64("key", range_end));
+    unique_ptr<KuduTableAlterer> alterer(c->NewTableAlterer(kTableName));
+    return alterer->AddRangePartition(lower_bound.release(),
+                                      upper_bound.release())->Alter();
+   };
+
+  // The bifurcation point.
+  const auto mode = GetParam();
+  switch (mode) {
+    case RANGE_DROPPED:
+      break;
+    case RANGE_DROPPED_AND_PRECEDING_RANGE_ADDED:
+      ASSERT_OK(range_adder(client_.get(), -200, -100));
+      break;
+    case RANGE_DROPPED_AND_LARGER_ONE_ADDED:
+      ASSERT_OK(range_adder(client_.get(), -200, 0));
+      break;
+    case RANGE_DROPPED_AND_SMALLER_ONE_ADDED:
+      ASSERT_OK(range_adder(client_.get(), -50, 0));
+      break;
+    case RANGE_REPLACED_WITH_SAME:
+      ASSERT_OK(range_adder(client_.get(), -100, 0));
+      break;
+    case RANGE_REPLACED_WITH_SAME_AND_PRECEDING_RANGE_ADDED:
+      ASSERT_OK(range_adder(client_.get(), -100, 0));
+      ASSERT_OK(range_adder(client_.get(), -200, -100));
+      break;
+    case RANGE_REPLACED_WITH_TWO_SMALLER_ONES:
+      ASSERT_OK(range_adder(client_.get(), -100, -50));
+      ASSERT_OK(range_adder(client_.get(), -50, 0));
+      break;
+    default:
+      FAIL() << strings::Substitute("$0: unsupported partition change mode",
+                                    static_cast<uint16_t>(mode));
+  }
+
+  shared_ptr<KuduSession> session = client_->NewSession();
+  session->SetTimeoutMillis(10000);
+  ASSERT_OK(session->SetFlushMode(KuduSession::AUTO_FLUSH_SYNC));
+
+  int64_t expected_row_count = 50;
+  switch (mode) {
+    case RANGE_DROPPED_AND_LARGER_ONE_ADDED:
+      expected_row_count += 100;
+      [[fallthrough]];
+    case RANGE_REPLACED_WITH_SAME_AND_PRECEDING_RANGE_ADDED:
+      for (int i = -200; i < -100; ++i) {
+        unique_ptr<KuduInsert> insert(table->NewInsert());
+        ASSERT_OK(insert->mutable_row()->SetInt64("key", i));
+        ASSERT_OK(session->Apply(insert.release()));
+      }
+      // The rows in the preceeding range should not be read if using the
+      // token for the [-100, 0) original range.
+      [[fallthrough]];
+    case RANGE_DROPPED_AND_SMALLER_ONE_ADDED:
+    case RANGE_REPLACED_WITH_SAME:
+    case RANGE_REPLACED_WITH_TWO_SMALLER_ONES:
+      for (int i = -25; i < 0; ++i) {
+        unique_ptr<KuduInsert> insert(table->NewInsert());
+        ASSERT_OK(insert->mutable_row()->SetInt64("key", i));
+        ASSERT_OK(session->Apply(insert.release()));
+      }
+      expected_row_count += 25;
+      break;
+    default:
+      break;
+  }
+
+  // Start another tablet scan using the other identical set of scan tokens.
+  // The client metacache should not produce any errors: it should re-fetch
+  // the information about the current partitioning scheme and scan the table
+  // within the range of the new partitions which correspond to the originally
+  // supplied range.
+  int64_t row_count_b = -1;
+  ASSERT_OK(CountRowsSeq(new_client.get(), std::move(tokens_b), &row_count_b));
+  ASSERT_EQ(expected_row_count, row_count_b);
+}
+
+INSTANTIATE_TEST_SUITE_P(FirstRangeDropped, StaleScanTokensParamTest,
+                         testing::Range(FirstRangeChangeMode::BEGIN,
+                                        FirstRangeChangeMode::END));
 } // namespace client
 } // namespace kudu
