@@ -20,6 +20,7 @@
 #include <algorithm>
 #include <cstring>
 #include <iterator>
+#include <memory>
 #include <set>
 #include <string>
 #include <unordered_set>
@@ -32,6 +33,8 @@
 #include "kudu/common/key_encoder.h"
 #include "kudu/common/partial_row.h"
 #include "kudu/common/row.h"
+#include "kudu/common/row_operations.h"
+#include "kudu/common/row_operations.pb.h"
 #include "kudu/common/types.h"
 #include "kudu/gutil/endian.h"
 #include "kudu/gutil/map-util.h"
@@ -81,11 +84,19 @@ Slice Partition::range_key(const string& partition_key) const {
   }
 }
 
-bool Partition::Equals(const Partition& other) const {
-  if (this == &other) return true;
-  if (partition_key_start() != other.partition_key_start()) return false;
-  if (partition_key_end() != other.partition_key_end()) return false;
-  if (hash_buckets_ != other.hash_buckets_) return false;
+bool Partition::operator==(const Partition& rhs) const {
+  if (this == &rhs) {
+    return true;
+  }
+  if (partition_key_start() != rhs.partition_key_start()) {
+    return false;
+  }
+  if (partition_key_end() != rhs.partition_key_end()) {
+    return false;
+  }
+  if (hash_buckets_ != rhs.hash_buckets_) {
+    return false;
+  }
   return true;
 }
 
@@ -177,9 +188,60 @@ Status PartitionSchema::ExtractHashBucketSchemasFromPB(
 Status PartitionSchema::FromPB(const PartitionSchemaPB& pb,
                                const Schema& schema,
                                PartitionSchema* partition_schema) {
+  return FromPB(pb, schema, schema, partition_schema);
+}
+
+Status PartitionSchema::FromPB(const PartitionSchemaPB& pb,
+                               const Schema& schema,
+                               const Schema& client_schema,
+                               PartitionSchema* partition_schema) {
   partition_schema->Clear();
   RETURN_NOT_OK(ExtractHashBucketSchemasFromPB(schema, pb.hash_bucket_schemas(),
-                             &partition_schema->hash_bucket_schemas_));
+                                               &partition_schema->hash_bucket_schemas_));
+  RangeHashSchema range_hash_schema;
+  range_hash_schema.resize(pb.range_hash_schemas_size());
+  for (int i = 0; i < pb.range_hash_schemas_size(); i++) {
+    RETURN_NOT_OK(ExtractHashBucketSchemasFromPB(schema, pb.range_hash_schemas(i).hash_schemas(),
+                                                 &range_hash_schema[i]));
+  }
+  vector<pair<KuduPartialRow, KuduPartialRow>> range_bounds;
+  for (int i = 0; i < pb.range_bounds_size(); i++) {
+    RowOperationsPBDecoder decoder(&pb.range_bounds(i), &client_schema, &schema, nullptr);
+    vector<DecodedRowOperation> ops;
+    RETURN_NOT_OK(decoder.DecodeOperations<DecoderMode::SPLIT_ROWS>(&ops));
+    if (ops.size() != 2) {
+      return Status::InvalidArgument(Substitute("$0 ops were provided; Only two ops are expected "
+                                                "for this pair of range bounds.", ops.size()));
+    }
+    const DecodedRowOperation& op1 = ops[0];
+    const DecodedRowOperation& op2 = ops[1];
+    switch (op1.type) {
+      case RowOperationsPB::RANGE_LOWER_BOUND:
+      case RowOperationsPB::EXCLUSIVE_RANGE_LOWER_BOUND: {
+        if (op2.type != RowOperationsPB::RANGE_UPPER_BOUND &&
+            op2.type != RowOperationsPB::INCLUSIVE_RANGE_UPPER_BOUND) {
+          return Status::InvalidArgument("missing upper range bound in request");
+        }
+
+        // Lower bound range partition keys are inclusive and upper bound range partition keys
+        // are exclusive by design. If the provided keys are not of this format, these keys
+        // will be transformed to their proper format.
+        if (op1.type == RowOperationsPB::EXCLUSIVE_RANGE_LOWER_BOUND) {
+          RETURN_NOT_OK(partition_schema->MakeLowerBoundRangePartitionKeyInclusive(
+              op1.split_row.get()));
+        }
+        if (op2.type == RowOperationsPB::INCLUSIVE_RANGE_UPPER_BOUND) {
+          RETURN_NOT_OK(partition_schema->MakeUpperBoundRangePartitionKeyExclusive(
+              op2.split_row.get()));
+        }
+        range_bounds.emplace_back(*op1.split_row, *op2.split_row);
+        break;
+      }
+      default:
+        return Status::InvalidArgument(
+            Substitute("Illegal row operation type in request: $0", op1.type));
+    }
+  }
 
   if (pb.has_range_schema()) {
     const PartitionSchemaPB_RangeSchemaPB& range_pb = pb.range_schema();
@@ -194,10 +256,15 @@ Status PartitionSchema::FromPB(const PartitionSchemaPB& pb,
     }
   }
 
+  if (!range_bounds.empty()) {
+    RETURN_NOT_OK(partition_schema->EncodeRangeBounds(
+        range_bounds, range_hash_schema, schema, &partition_schema->ranges_with_hash_schemas_));
+  }
+
   return partition_schema->Validate(schema);
 }
 
-void PartitionSchema::ToPB(PartitionSchemaPB* pb) const {
+Status PartitionSchema::ToPB(const Schema& schema, PartitionSchemaPB* pb) const {
   pb->Clear();
   pb->mutable_hash_bucket_schemas()->Reserve(hash_bucket_schemas_.size());
   for (const HashBucketSchema& hash_bucket : hash_bucket_schemas_) {
@@ -207,7 +274,34 @@ void PartitionSchema::ToPB(PartitionSchemaPB* pb) const {
     hash_bucket_pb->set_seed(hash_bucket.seed);
   }
 
+  if (!ranges_with_hash_schemas_.empty()) {
+    pb->mutable_range_hash_schemas()->Reserve(ranges_with_hash_schemas_.size());
+    pb->mutable_range_bounds()->Reserve(ranges_with_hash_schemas_.size());
+    Arena arena(256);
+    for (const auto& range_hash_schema : ranges_with_hash_schemas_) {
+      RowOperationsPBEncoder encoder(pb->add_range_bounds());
+      arena.Reset();
+      KuduPartialRow lower(&schema);
+      KuduPartialRow upper(&schema);
+      Slice s_lower = Slice(range_hash_schema.lower);
+      Slice s_upper = Slice(range_hash_schema.upper);
+      RETURN_NOT_OK(DecodeRangeKey(&s_lower, &lower, &arena));
+      RETURN_NOT_OK(DecodeRangeKey(&s_upper, &upper, &arena));
+      encoder.Add(RowOperationsPB::RANGE_LOWER_BOUND, lower);
+      encoder.Add(RowOperationsPB::RANGE_UPPER_BOUND, upper);
+
+      auto* range_hash_schema_pb = pb->add_range_hash_schemas();
+      for (const auto& hash_bucket : range_hash_schema.hash_schemas) {
+        auto* hash_bucket_pb = range_hash_schema_pb->add_hash_schemas();
+        SetColumnIdentifiers(hash_bucket.column_ids, hash_bucket_pb->mutable_columns());
+        hash_bucket_pb->set_num_buckets(hash_bucket.num_buckets);
+        hash_bucket_pb->set_seed(hash_bucket.seed);
+      }
+    }
+  }
+
   SetColumnIdentifiers(range_schema_.column_ids, pb->mutable_range_schema()->mutable_columns());
+  return Status::OK();
 }
 
 template<typename Row>
@@ -449,6 +543,11 @@ Status PartitionSchema::CreatePartitions(const vector<KuduPartialRow>& split_row
   RETURN_NOT_OK(EncodeRangeSplits(split_rows, schema, &splits));
   RETURN_NOT_OK(SplitRangeBounds(schema, std::move(splits), &bounds_with_hash_schemas));
 
+  // Maps each partition to its respective hash schemas within 'bounds_with_hash_schemas',
+  // needed for logic later in function for filling in holes in partition key space. Will be
+  // empty if no per range hash schemas are used.
+  vector<int> partition_idx_to_hash_schemas_idx;
+
   if (!range_hash_schemas.empty()) {
     // The number of ranges should match the size of range_hash_schemas.
     DCHECK_EQ(range_hash_schemas.size(), bounds_with_hash_schemas.size());
@@ -456,7 +555,8 @@ Status PartitionSchema::CreatePartitions(const vector<KuduPartialRow>& split_row
     DCHECK(split_rows.empty());
     vector<Partition> result_partitions;
     // Iterate through each bound and its hash schemas to generate hash partitions.
-    for (const auto& bound : bounds_with_hash_schemas) {
+    for (int i = 0; i < bounds_with_hash_schemas.size(); i++) {
+      const auto& bound = bounds_with_hash_schemas[i];
       const auto& current_range_hash_schemas = bound.hash_schemas;
       vector<Partition> current_bound_hash_partitions;
       // If current bound's HashBucketSchema is empty, implies use of default table-wide schema.
@@ -467,20 +567,23 @@ Status PartitionSchema::CreatePartitions(const vector<KuduPartialRow>& split_row
         current_bound_hash_partitions = GenerateHashPartitions(current_range_hash_schemas,
                                                                hash_encoder);
       }
-      // Adds range part to partition key.
+      // Add range part to partition key.
       for (Partition& partition : current_bound_hash_partitions) {
         partition.partition_key_start_.append(bound.lower);
         partition.partition_key_end_.append(bound.upper);
+        int index = current_range_hash_schemas.empty() ? -1 : i;
+        partition_idx_to_hash_schemas_idx.emplace_back(index);
       }
       result_partitions.insert(result_partitions.end(),
                                std::make_move_iterator(current_bound_hash_partitions.begin()),
                                std::make_move_iterator(current_bound_hash_partitions.end()));
     }
+    DCHECK_EQ(partition_idx_to_hash_schemas_idx.size(), result_partitions.size());
     *partitions = std::move(result_partitions);
   } else {
     // Create a partition per range bound and hash bucket combination.
     vector<Partition> new_partitions;
-    for (const Partition &base_partition : base_hash_partitions) {
+    for (const Partition& base_partition : base_hash_partitions) {
       for (const auto& bound : bounds_with_hash_schemas) {
         Partition partition = base_partition;
         partition.partition_key_start_.append(bound.lower);
@@ -508,7 +611,11 @@ Status PartitionSchema::CreatePartitions(const vector<KuduPartialRow>& split_row
   // the absolute start and end case, these holes are filled by clearing the
   // partition key beginning at the hash component. For a concrete example,
   // see PartitionTest::TestCreatePartitions.
-  for (Partition& partition : *partitions) {
+  const HashBucketSchema* hash_bucket_schema;
+  for (int j = 0; j < partitions->size(); j++) {
+    Partition& partition = (*partitions)[j];
+    // Find the first zero-valued bucket from the end and truncate the partition key
+    // starting from that bucket onwards for zero-valued buckets.
     if (partition.range_key_start().empty()) {
       for (int i = static_cast<int>(partition.hash_buckets().size()) - 1; i >= 0; i--) {
         if (partition.hash_buckets()[i] != 0) {
@@ -517,11 +624,26 @@ Status PartitionSchema::CreatePartitions(const vector<KuduPartialRow>& split_row
         partition.partition_key_start_.erase(kEncodedBucketSize * i);
       }
     }
+    // Starting from the last hash bucket, truncate the partition key until we hit the first
+    // non-max-valued bucket, at which point, replace the encoding with the next-incremented
+    // bucket value. For example, the following range end partition keys should be transformed,
+    // where the key is (hash_comp1, hash_comp2, range_comp):
+    //
+    // [ (0, 0, "a2b2") -> (0, 1, "a2b2") ]
+    // [ (0, 1, "a2b2") -> (1, _, "a2b2") ]
+    // [ (1, 0, "a2b2") -> (1, 1, "a2b2") ]
+    // [ (1, 1, "a2b2") -> (_, _, "a2b2") ]
     if (partition.range_key_end().empty()) {
       for (int i = static_cast<int>(partition.hash_buckets().size()) - 1; i >= 0; i--) {
         partition.partition_key_end_.erase(kEncodedBucketSize * i);
         int32_t hash_bucket = partition.hash_buckets()[i] + 1;
-        if (hash_bucket != hash_bucket_schemas_[i].num_buckets) {
+        if (range_hash_schemas.empty() || partition_idx_to_hash_schemas_idx[j] == -1) {
+          hash_bucket_schema = &hash_bucket_schemas_[i];
+        } else {
+          const auto& hash_schemas_idx = partition_idx_to_hash_schemas_idx[j];
+          hash_bucket_schema = &bounds_with_hash_schemas[hash_schemas_idx].hash_schemas[i];
+        }
+        if (hash_bucket != hash_bucket_schema->num_buckets) {
           hash_encoder.Encode(&hash_bucket, &partition.partition_key_end_);
           break;
         }
@@ -984,20 +1106,32 @@ string PartitionSchema::PartitionTableEntry(const Schema& schema,
   return entry;
 }
 
-bool PartitionSchema::Equals(const PartitionSchema& other) const {
-  if (this == &other) return true;
+bool PartitionSchema::operator==(const PartitionSchema& rhs) const {
+  if (this == &rhs) {
+    return true;
+  }
 
   // Compare range component.
-  if (range_schema_.column_ids != other.range_schema_.column_ids) return false;
+  if (range_schema_.column_ids != rhs.range_schema_.column_ids) {
+    return false;
+  }
 
   // Compare hash bucket components.
-  if (hash_bucket_schemas_.size() != other.hash_bucket_schemas_.size()) return false;
-  for (int i = 0; i < hash_bucket_schemas_.size(); i++) {
-    if (hash_bucket_schemas_[i].seed != other.hash_bucket_schemas_[i].seed) return false;
-    if (hash_bucket_schemas_[i].num_buckets
-        != other.hash_bucket_schemas_[i].num_buckets) return false;
-    if (hash_bucket_schemas_[i].column_ids
-        != other.hash_bucket_schemas_[i].column_ids) return false;
+  const auto& hb_schemas = hash_bucket_schemas_;
+  const auto& rhs_hb_schemas = rhs.hash_bucket_schemas_;
+  if (hb_schemas.size() != rhs_hb_schemas.size()) {
+    return false;
+  }
+  for (size_t i = 0; i < hb_schemas.size(); ++i) {
+    if (hb_schemas[i].seed != rhs_hb_schemas[i].seed) {
+      return false;
+    }
+    if (hb_schemas[i].num_buckets != rhs_hb_schemas[i].num_buckets) {
+      return false;
+    }
+    if (hb_schemas[i].column_ids != rhs_hb_schemas[i].column_ids) {
+      return false;
+    }
   }
 
   return true;
@@ -1075,11 +1209,14 @@ Status PartitionSchema::BucketForRow(const ConstContiguousRow& row,
 void PartitionSchema::Clear() {
   hash_bucket_schemas_.clear();
   range_schema_.column_ids.clear();
+  ranges_with_hash_schemas_.clear();
+
 }
 
-Status PartitionSchema::Validate(const Schema& schema) const {
+Status PartitionSchema::ValidateHashBucketSchemas(const Schema& schema,
+                                                  const HashBucketSchemas& hash_schemas) {
   set<ColumnId> hash_columns;
-  for (const PartitionSchema::HashBucketSchema& hash_schema : hash_bucket_schemas_) {
+  for (const PartitionSchema::HashBucketSchema& hash_schema : hash_schemas) {
     if (hash_schema.num_buckets < 2) {
       return Status::InvalidArgument("must have at least two hash buckets");
     }
@@ -1103,6 +1240,15 @@ Status PartitionSchema::Validate(const Schema& schema) const {
       }
     }
   }
+  return Status::OK();
+}
+
+Status PartitionSchema::Validate(const Schema& schema) const {
+  RETURN_NOT_OK(ValidateHashBucketSchemas(schema, hash_bucket_schemas_));
+
+  for (const auto& range_with_hash_schemas : ranges_with_hash_schemas_) {
+    RETURN_NOT_OK(ValidateHashBucketSchemas(schema, range_with_hash_schemas.hash_schemas));
+  }
 
   for (const ColumnId& column_id : range_schema_.column_ids) {
     int32_t column_idx = schema.find_column_by_id(column_id);
