@@ -29,6 +29,7 @@
 #include <gflags/gflags.h>
 
 #include "kudu/common/wire_protocol.h"
+#include "kudu/consensus/consensus.pb.h"
 #include "kudu/consensus/log_index.h"
 #include "kudu/consensus/log_metrics.h"
 #include "kudu/consensus/log_reader.h"
@@ -102,11 +103,6 @@ DEFINE_bool(log_inject_latency, false,
             "If true, injects artificial latency in log sync operations. "
             "Advanced option. Use at your own risk -- has a negative effect "
             "on performance for obvious reasons!");
-
-DEFINE_bool(skip_remove_old_recovery_dir, false,
-            "Skip removing WAL recovery dir after startup. (useful for debugging)");
-TAG_FLAG(skip_remove_old_recovery_dir, hidden);
-
 TAG_FLAG(log_inject_latency, unsafe);
 TAG_FLAG(log_inject_latency, runtime);
 
@@ -164,6 +160,10 @@ DEFINE_bool(fs_wal_use_file_cache, true,
 TAG_FLAG(fs_wal_use_file_cache, runtime);
 TAG_FLAG(fs_wal_use_file_cache, advanced);
 
+DEFINE_bool(skip_remove_old_recovery_dir, false,
+            "Skip removing WAL recovery dir after startup. (useful for debugging)");
+TAG_FLAG(skip_remove_old_recovery_dir, hidden);
+
 // Validate that log_min_segments_to_retain >= 1
 static bool ValidateLogsToRetain(const char* flagname, int value) {
   if (value >= 1) {
@@ -175,17 +175,17 @@ static bool ValidateLogsToRetain(const char* flagname, int value) {
 }
 DEFINE_validator(log_min_segments_to_retain, &ValidateLogsToRetain);
 
+using kudu::consensus::CommitMsg;
+using kudu::consensus::OpId;
+using kudu::consensus::ReplicateRefPtr;
+using std::string;
+using std::unique_ptr;
+using std::vector;
+using strings::Substitute;
+
 namespace kudu {
 namespace log {
 
-using consensus::CommitMsg;
-using consensus::ReplicateRefPtr;
-using std::shared_ptr;
-using std::string;
-using std::vector;
-using std::unique_ptr;
-using strings::Substitute;
-
 string LogContext::LogPrefix() const {
   return Substitute("T $0 P $1: ", tablet_id, fs_manager->uuid());
 }
@@ -247,9 +247,8 @@ class Log::AppendThread {
   // a new task was enqueued just as we were trying to go idle.
   bool GoIdle();
 
-  // Handle the actual appending of a group of entries. Responsible for deleting the
-  // LogEntryBatch* pointers.
-  void HandleBatches(vector<LogEntryBatch*> entry_batches);
+  // Handle the actual appending of a group of entries.
+  void HandleBatches(vector<unique_ptr<LogEntryBatch>> entry_batches);
 
   string LogPrefix() const;
 
@@ -350,7 +349,7 @@ void Log::AppendThread::ProcessQueue() {
   while (true) {
     MonoTime deadline = MonoTime::Now() +
         MonoDelta::FromMilliseconds(FLAGS_log_thread_idle_threshold_ms);
-    vector<LogEntryBatch*> entry_batches;
+    vector<unique_ptr<LogEntryBatch>> entry_batches;
     Status s = log_->entry_queue()->BlockingDrainTo(&entry_batches, deadline);
     if (PREDICT_FALSE(s.IsAborted())) {
       break;
@@ -364,7 +363,7 @@ void Log::AppendThread::ProcessQueue() {
   VLOG_WITH_PREFIX(2) << "WAL Appender going idle";
 }
 
-void Log::AppendThread::HandleBatches(vector<LogEntryBatch*> entry_batches) {
+void Log::AppendThread::HandleBatches(vector<unique_ptr<LogEntryBatch>> entry_batches) {
   if (log_->ctx_.metrics) {
     log_->ctx_.metrics->entry_batches_per_group->Increment(entry_batches.size());
   }
@@ -373,14 +372,14 @@ void Log::AppendThread::HandleBatches(vector<LogEntryBatch*> entry_batches) {
   SCOPED_LATENCY_METRIC(log_->ctx_.metrics, group_commit_latency);
 
   bool is_all_commits = true;
-  for (auto* entry_batch : entry_batches) {
-    TRACE_EVENT_FLOW_END0("log", "Batch", entry_batch);
-    Status s = log_->WriteBatch(entry_batch);
+  for (auto& entry_batch : entry_batches) {
+    TRACE_EVENT_FLOW_END0("log", "Batch", entry_batch.get());
+    Status s = log_->WriteBatch(entry_batch.get());
     if (PREDICT_FALSE(!s.ok())) {
       LOG_WITH_PREFIX(ERROR) << "Error appending to the log: " << s.ToString();
-      // TODO(af): If a single transaction fails to append, should we
-      // abort all subsequent transactions in this batch or allow
-      // them to be appended? What about transactions in future
+      // TODO(af): If a single op fails to append, should we
+      // abort all subsequent ops in this batch or allow
+      // them to be appended? What about ops in future
       // batches?
       entry_batch->SetAppendError(s);
     }
@@ -395,7 +394,7 @@ void Log::AppendThread::HandleBatches(vector<LogEntryBatch*> entry_batches) {
   }
   if (PREDICT_FALSE(!s.ok())) {
     LOG_WITH_PREFIX(ERROR) << "Error syncing log: " << s.ToString();
-    for (auto* entry_batch : entry_batches) {
+    for (const auto& entry_batch : entry_batches) {
       entry_batch->SetAppendError(s);
     }
   } else {
@@ -403,13 +402,12 @@ void Log::AppendThread::HandleBatches(vector<LogEntryBatch*> entry_batches) {
   }
   TRACE_EVENT0("log", "Callbacks");
   SCOPED_WATCH_STACK(100);
-  for (auto* entry_batch : entry_batches) {
+  for (auto& entry_batch : entry_batches) {
     entry_batch->RunCallback();
-
     // It's important to delete each batch as we see it, because
     // deleting it may free up memory from memory trackers, and the
     // callback of a later batch may want to use that memory.
-    delete entry_batch;
+    entry_batch.reset();
   }
 }
 
@@ -507,10 +505,9 @@ Status SegmentAllocator::Sync() {
     }
   }
 
-  if (opts_->force_fsync_all && !false) {
+  if (opts_->force_fsync_all) {
     LOG_SLOW_EXECUTION(WARNING, 50, Substitute("$0Fsync log took a long time", LogPrefix())) {
       RETURN_NOT_OK(active_segment_->Sync());
-
       if (hooks_) {
         RETURN_NOT_OK_PREPEND(hooks_->PostSyncIfFsyncEnabled(),
                               "PostSyncIfFsyncEnabled hook failed");
@@ -581,8 +578,8 @@ void SegmentAllocator::UpdateFooterForBatch(const LogEntryBatch& batch) {
   // immediately.
   if (batch.type_ == REPLICATE) {
     // Update the index bounds for the current segment.
-    for (const LogEntryPB& entry_pb : batch.entry_batch_pb_->entry()) {
-      UpdateFooterForReplicateEntry(entry_pb, &footer_);
+    for (const OpId& op_id : batch.replicate_op_ids_) {
+      UpdateFooterForReplicateEntry(op_id, &footer_);
     }
   }
 }
@@ -830,60 +827,69 @@ Status Log::Init() {
 }
 
 unique_ptr<LogEntryBatch> Log::CreateBatchFromPB(
-    LogEntryTypePB type,
-    unique_ptr<LogEntryBatchPB> entry_batch_pb,
+    LogEntryTypePB type, const LogEntryBatchPB& entry_batch_pb,
     StatusCallback cb) {
-  int num_ops = entry_batch_pb->entry_size();
-  unique_ptr<LogEntryBatch> new_entry_batch(new LogEntryBatch(
-      type, std::move(entry_batch_pb), num_ops, std::move(cb)));
-  new_entry_batch->Serialize();
+  unique_ptr<LogEntryBatch> new_entry_batch(
+      new LogEntryBatch(type, entry_batch_pb, std::move(cb)));
   TRACE("Serialized $0 byte log entry", new_entry_batch->total_size_bytes());
   return new_entry_batch;
 }
 
 Status Log::AsyncAppend(unique_ptr<LogEntryBatch> entry_batch) {
   TRACE_EVENT0("log", "Log::AsyncAppend");
-  TRACE_EVENT_FLOW_BEGIN0("log", "Batch", entry_batch.get());
-  if (PREDICT_FALSE(!entry_batch_queue_.BlockingPut(entry_batch.get()).ok())) {
-    TRACE_EVENT_FLOW_END0("log", "Batch", entry_batch.get());
+  // entry_batch_trace_id is used the identifier for the trace, where only the
+  // address is stored, the pointer isn't de-referenced.
+  const LogEntryBatch* entry_batch_trace_id = entry_batch.get();
+  TRACE_EVENT_FLOW_BEGIN0("log", "Batch", entry_batch_trace_id);
+  if (PREDICT_FALSE(!entry_batch_queue_.BlockingPut(std::move(entry_batch)).ok())) {
+    TRACE_EVENT_FLOW_END0("log", "Batch", entry_batch_trace_id);
     return kLogShutdownStatus;
   }
   append_thread_->Wake();
-  entry_batch.release();
   return Status::OK();
 }
 
-Status Log::AsyncAppendReplicates(const vector<ReplicateRefPtr>& replicates,
+Status Log::AsyncAppendReplicates(vector<ReplicateRefPtr> replicates,
                                   StatusCallback callback) {
-  unique_ptr<LogEntryBatchPB> batch_pb(new LogEntryBatchPB);
-  batch_pb->mutable_entry()->Reserve(replicates.size());
+  LogEntryBatchPB batch_pb;
+  batch_pb.mutable_entry()->Reserve(replicates.size());
   for (const auto& r : replicates) {
-    LogEntryPB* entry_pb = batch_pb->add_entry();
+    LogEntryPB* entry_pb = batch_pb.add_entry();
     entry_pb->set_type(REPLICATE);
     entry_pb->set_allocated_replicate(r->get());
   }
-  unique_ptr<LogEntryBatch> batch = CreateBatchFromPB(
-      REPLICATE, std::move(batch_pb), std::move(callback));
-  batch->SetReplicates(replicates);
+  unique_ptr<LogEntryBatch> batch =
+      CreateBatchFromPB(REPLICATE, batch_pb, std::move(callback));
+
+  for (LogEntryPB& entry : *batch_pb.mutable_entry()) {
+    entry.release_replicate();
+  }
+
   return AsyncAppend(std::move(batch));
 }
 
-Status Log::AsyncAppendCommit(unique_ptr<consensus::CommitMsg> commit_msg,
+Status Log::AsyncAppendCommit(const consensus::CommitMsg& commit_msg,
                               StatusCallback callback) {
   MAYBE_FAULT(FLAGS_fault_crash_before_append_commit);
 
-  unique_ptr<LogEntryBatchPB> batch_pb(new LogEntryBatchPB);
-  LogEntryPB* entry = batch_pb->add_entry();
+  LogEntryBatchPB batch_pb;
+  LogEntryPB* entry = batch_pb.add_entry();
   entry->set_type(COMMIT);
-  entry->set_allocated_commit(commit_msg.release());
+  entry->unsafe_arena_set_allocated_commit(const_cast<consensus::CommitMsg*>(&commit_msg));
 
   unique_ptr<LogEntryBatch> entry_batch = CreateBatchFromPB(
-      COMMIT, std::move(batch_pb), std::move(callback));
+      COMMIT, batch_pb, std::move(callback));
+  entry->unsafe_arena_release_commit();
   AsyncAppend(std::move(entry_batch));
   return Status::OK();
 }
 
 Status Log::WriteBatch(LogEntryBatch* entry_batch) {
+  // If there is no data to write return OK.
+  if (PREDICT_FALSE(entry_batch->type_ == FLUSH_MARKER)) {
+    return Status::OK();
+  }
+
   size_t num_entries = entry_batch->count();
   DCHECK_GT(num_entries, 0) << "Cannot call WriteBatch() with zero entries reserved";
 
@@ -892,10 +898,6 @@ Status Log::WriteBatch(LogEntryBatch* entry_batch) {
 
   Slice entry_batch_data = entry_batch->data();
   uint32_t entry_batch_bytes = entry_batch->total_size_bytes();
-  // If there is no data to write return OK.
-  if (PREDICT_FALSE(entry_batch_bytes == 0)) {
-    return Status::OK();
-  }
 
   scoped_refptr<ReadableLogSegment> finished_segment;
   scoped_refptr<ReadableLogSegment> new_readable_segment;
@@ -941,10 +943,9 @@ Status Log::UpdateIndexForBatch(const LogEntryBatch& batch,
     return Status::OK();
   }
 
-  for (const LogEntryPB& entry_pb : batch.entry_batch_pb_->entry()) {
+  for (const OpId& op_id : batch.replicate_op_ids_) {
     LogIndexEntry index_entry;
-
-    index_entry.op_id = entry_pb.replicate().id();
+    index_entry.op_id = op_id;
     index_entry.segment_sequence_number = segment_allocator_.active_segment_sequence_number();
     index_entry.offset_in_segment = start_offset;
     RETURN_NOT_OK(log_index_->AddEntry(index_entry));
@@ -1005,27 +1006,25 @@ void Log::GetSegmentsToGCUnlocked(RetentionIndexes retention_indexes,
 }
 
 Status Log::Append(LogEntryPB* entry) {
-  unique_ptr<LogEntryBatchPB> entry_batch_pb(new LogEntryBatchPB);
-  entry_batch_pb->mutable_entry()->AddAllocated(entry);
-  LogEntryBatch entry_batch(entry->type(), std::move(entry_batch_pb), 1,
-                            &DoNothingStatusCB);
-  entry_batch.Serialize();
+  LogEntryBatchPB entry_batch_pb;
+  entry_batch_pb.mutable_entry()->UnsafeArenaAddAllocated(entry);
+  LogEntryBatch entry_batch(entry->type(), entry_batch_pb, &DoNothingStatusCB);
+  entry_batch_pb.mutable_entry()->ExtractSubrange(0, 1, nullptr);
   Status s = WriteBatch(&entry_batch);
   if (s.ok()) {
     s = Sync();
   }
-  entry_batch.entry_batch_pb_->mutable_entry()->ExtractSubrange(0, 1, nullptr);
   return s;
 }
 
 Status Log::WaitUntilAllFlushed() {
   // In order to make sure we empty the queue we need to use
   // the async api.
-  unique_ptr<LogEntryBatchPB> entry_batch(new LogEntryBatchPB);
-  entry_batch->add_entry()->set_type(log::FLUSH_MARKER);
+  LogEntryBatchPB entry_batch;
+  entry_batch.add_entry()->set_type(log::FLUSH_MARKER);
   Synchronizer s;
-  unique_ptr<LogEntryBatch> reserved_entry_batch = CreateBatchFromPB(
-      FLUSH_MARKER, std::move(entry_batch), s.AsStatusCallback());
+  unique_ptr<LogEntryBatch> reserved_entry_batch =
+      CreateBatchFromPB(FLUSH_MARKER, entry_batch, s.AsStatusCallback());
   AsyncAppend(std::move(reserved_entry_batch));
   return s.Wait();
 }
@@ -1236,38 +1235,27 @@ Log::~Log() {
 }
 
 LogEntryBatch::LogEntryBatch(LogEntryTypePB type,
-                             unique_ptr<LogEntryBatchPB> entry_batch_pb,
-                             size_t count,
+                             const LogEntryBatchPB& entry_batch_pb,
                              StatusCallback cb)
     : type_(type),
-      entry_batch_pb_(std::move(entry_batch_pb)),
-      total_size_bytes_(
-          PREDICT_FALSE(count == 1 && entry_batch_pb_->entry(0).type() == FLUSH_MARKER) ?
-          0 : entry_batch_pb_->ByteSize()),
-      count_(count),
+      total_size_bytes_(entry_batch_pb.ByteSizeLong()),
+      count_(entry_batch_pb.entry().size()),
       callback_(std::move(cb)) {
-}
+  if (total_size_bytes_) {
+    buffer_.reserve(total_size_bytes_);
+    pb_util::AppendToString(entry_batch_pb, &buffer_);
+  }
 
-LogEntryBatch::~LogEntryBatch() {
-  if (type_ == REPLICATE && entry_batch_pb_) {
-    for (LogEntryPB& entry : *entry_batch_pb_->mutable_entry()) {
-      // ReplicateMsg elements are owned by and must be freed by the caller
-      // (e.g. the LogCache).
-      entry.release_replicate();
+  if (type == REPLICATE) {
+    replicate_op_ids_.reserve(entry_batch_pb.entry().size());
+    for (const auto& e : entry_batch_pb.entry()) {
+      DCHECK(e.has_replicate());
+      replicate_op_ids_.emplace_back(e.replicate().id());
     }
   }
 }
 
-void LogEntryBatch::Serialize() {
-  DCHECK_EQ(buffer_.size(), 0);
-  // FLUSH_MARKER LogEntries are markers and are not serialized.
-  if (PREDICT_FALSE(count() == 1 && entry_batch_pb_->entry(0).type() == FLUSH_MARKER)) {
-    return;
-  }
-  buffer_.reserve(total_size_bytes_);
-  pb_util::AppendToString(*entry_batch_pb_, &buffer_);
-}
-
+LogEntryBatch::~LogEntryBatch() {}
 
 }  // namespace log
 }  // namespace kudu
