@@ -51,7 +51,6 @@ DEFINE_int32(safe_time_max_lag_ms, 30 * 1000,
 TAG_FLAG(safe_time_max_lag_ms, experimental);
 
 DECLARE_int32(raft_heartbeat_interval_ms);
-DECLARE_int32(scanner_max_wait_ms);
 
 using kudu::clock::Clock;
 using std::string;
@@ -139,6 +138,19 @@ Status TimeManager::MessageReceivedFromLeader(const ReplicateMsg& message) {
   return Status::OK();
 }
 
+Status TimeManager::UpdateClockAndLastAssignedTimestamp(const Timestamp& timestamp) {
+  RETURN_NOT_OK(clock_->Update(timestamp));
+  Lock l(lock_);
+  if (PREDICT_FALSE(mode_ == NON_LEADER)) {
+    return Status::IllegalState(Substitute(
+        "Cannot bump the last assigned timestamp. Tablet is not "
+        "in leader mode. Last heard from a leader: $0 ago.",
+        (MonoTime::Now() - last_advanced_safe_time_).ToString()));
+  }
+  last_serial_ts_assigned_ = std::max(timestamp, last_serial_ts_assigned_);
+  return Status::OK();
+}
+
 void TimeManager::AdvanceSafeTimeWithMessage(const ReplicateMsg& message) {
   Lock l(lock_);
   if (GetMessageConsistencyMode(message) == CLIENT_PROPAGATED) {
