@@ -556,7 +556,7 @@ TEST_F(AlterTableTest, TestAlterOnTSRestart) {
   KuduSchema schema;
   bool alter_in_progress = false;
   ASSERT_OK(client_->GetTableSchema(kTableName, &schema));
-  ASSERT_TRUE(schema_.Equals(schema));
+  ASSERT_EQ(schema_, schema);
   ASSERT_OK(client_->IsAlterTableInProgress(kTableName, &alter_in_progress));
   ASSERT_TRUE(alter_in_progress);
 
@@ -586,10 +586,7 @@ TEST_F(AlterTableTest, TestShutdownWithPendingTasks) {
 //  - get the new schema state, and mark the alter as complete
 //  - get the old schema state, and ask the TS again to perform the alter.
 TEST_F(AlterTableTest, TestRestartTSDuringAlter) {
-  if (!AllowSlowTests()) {
-    LOG(INFO) << "Skipping slow test";
-    return;
-  }
+  SKIP_IF_SLOW_NOT_ALLOWED();
 
   ASSERT_EQ(0, tablet_replica_->tablet()->metadata()->schema_version());
 
@@ -1878,7 +1875,7 @@ TEST_F(AlterTableTest, TestAddRangePartitionConflictExhaustive) {
     if (a_lower_bound == b_lower_bound && a_upper_bound == b_upper_bound) {
       ASSERT_TRUE(s.IsAlreadyPresent()) << s.ToString();
       ASSERT_STR_CONTAINS(s.ToString(),
-                          "new range partiton duplicates another newly added one");
+                          "new range partition duplicates another newly added one");
     } else {
       ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
       ASSERT_STR_CONTAINS(s.ToString(),
