@@ -858,11 +858,7 @@ TEST_F(DeleteTableITest, TestMergeConsensusMetadata) {
 // state, which means they have not yet been committed to a majority, cannot
 // shut down during a DeleteTablet() call.
 TEST_F(DeleteTableITest, TestDeleteFollowerWithReplicatingOps) {
-  if (!AllowSlowTests()) {
-    // We will typically wait at least 5 seconds for timeouts to occur.
-    LOG(INFO) << "Skipping test in fast-test mode.";
-    return;
-  }
+  SKIP_IF_SLOW_NOT_ALLOWED();
 
   const MonoDelta timeout = MonoDelta::FromSeconds(10);
 
@@ -1154,10 +1150,7 @@ TEST_F(DeleteTableITest, TestUnknownTabletsAreNotDeleted) {
 // Ensure that the master doesn't try to delete tombstoned tablets.
 // Regression test for KUDU-2114.
 TEST_F(DeleteTableITest, TestNoDeleteTombstonedTablets) {
-  if (!AllowSlowTests()) {
-    LOG(WARNING) << "This test sleeps for several seconds and only runs in slow-test mode";
-    return;
-  }
+  SKIP_IF_SLOW_NOT_ALLOWED();
 
   const MonoDelta kTimeout = MonoDelta::FromSeconds(30);
   const vector<string> master_flags = {
@@ -1345,8 +1338,8 @@ const char* deleted_faults[] = {"fault_crash_after_blocks_deleted",
                                 "fault_crash_after_wal_deleted",
                                 "fault_crash_after_cmeta_deleted"};
 
-INSTANTIATE_TEST_CASE_P(FaultFlags, DeleteTableDeletedParamTest,
-                        ::testing::ValuesIn(deleted_faults));
+INSTANTIATE_TEST_SUITE_P(FaultFlags, DeleteTableDeletedParamTest,
+                         ::testing::ValuesIn(deleted_faults));
 
 // Parameterized test case for TABLET_DATA_TOMBSTONED deletions.
 class DeleteTableTombstonedParamTest : public DeleteTableITest,
@@ -1503,8 +1496,8 @@ TEST_P(DeleteTableTombstonedParamTest, TestTabletTombstone) {
 // Tombstoning a tablet does not delete the consensus metadata.
 const char* tombstoned_faults[] = {"fault_crash_after_blocks_deleted",
                                    "fault_crash_after_wal_deleted"};
-INSTANTIATE_TEST_CASE_P(FaultFlags, DeleteTableTombstonedParamTest,
-                        ::testing::ValuesIn(tombstoned_faults));
+INSTANTIATE_TEST_SUITE_P(FaultFlags, DeleteTableTombstonedParamTest,
+                         ::testing::ValuesIn(tombstoned_faults));
 
 
 class DeleteTableWhileScanInProgressParamTest :
@@ -1655,7 +1648,7 @@ const KuduClient::ReplicaSelection replica_selectors[] = {
   KuduClient::CLOSEST_REPLICA,
   KuduClient::FIRST_REPLICA,
 };
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     Params, DeleteTableWhileScanInProgressParamTest,
     ::testing::Combine(::testing::ValuesIn(read_modes),
                        ::testing::ValuesIn(replica_selectors)));
