@@ -20,6 +20,7 @@
 #include <ostream>
 #include <string>
 #include <thread>
+#include <tuple>
 #include <unordered_set>
 #include <utility>
 #include <vector>
@@ -729,7 +730,7 @@ class MasterAuthzITest : public MasterAuthzITestBase,
   }
 };
 
-INSTANTIATE_TEST_CASE_P(AuthzProviders, MasterAuthzITest,
+INSTANTIATE_TEST_SUITE_P(AuthzProviders, MasterAuthzITest,
     ::testing::Values(kRanger),
     [] (const testing::TestParamInfo<MasterAuthzITest::ParamType>& info) {
       return HarnessEnumToString(info.param);
@@ -745,7 +746,9 @@ TEST_P(MasterAuthzITest, TestCreateTransactionStatusTable) {
   // NotAuthorized error.
   {
     unique_ptr<TxnSystemClient> non_admin_client;
-    ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &non_admin_client));
+    ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                      cluster_->service_principal(),
+                                      &non_admin_client));
     Status s = non_admin_client->CreateTxnStatusTable(100);
     ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
     s = non_admin_client->AddTxnStatusTableRange(100, 200);
@@ -754,7 +757,9 @@ TEST_P(MasterAuthzITest, TestCreateTransactionStatusTable) {
   // But as service user, we should have no trouble making the calls.
   ASSERT_OK(this->cluster_->kdc()->Kinit(kAdminUser));
   unique_ptr<TxnSystemClient> txn_sys_client;
-  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_sys_client));
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->service_principal(),
+                                    &txn_sys_client));
   ASSERT_OK(txn_sys_client->CreateTxnStatusTable(100));
   ASSERT_OK(txn_sys_client->AddTxnStatusTableRange(100, 200));
 }
@@ -984,7 +989,7 @@ class MasterAuthzOwnerITest : public MasterAuthzITestBase,
   }
 };
 
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     AuthzProvidersWithOwner, MasterAuthzOwnerITest,
     ::testing::Combine(::testing::Values(kRanger),
                        ::testing::Values(kTestUser, "{OWNER}")),
@@ -1208,7 +1213,7 @@ static const AuthzDescriptor kAuthzCombinations[] = {
       kTableName,
     },
 };
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     AuthzCombinations, TestAuthzTable,
     ::testing::Combine(::testing::Values(kRanger),
                        ::testing::ValuesIn(kAuthzCombinations),
@@ -1301,14 +1306,13 @@ static const AuthzFuncs kAuthzFuncCombinations[] = {
     },
 };
 
-INSTANTIATE_TEST_CASE_P(AuthzFuncCombinations,
-                        AuthzErrorHandlingTest,
-                        ::testing::Combine(
-                            ::testing::Values(kRanger),
-                            ::testing::ValuesIn(kAuthzFuncCombinations)),
-                        [] (const testing::TestParamInfo<AuthzErrorHandlingTest::ParamType>& info) {
-                          return Substitute("$0_$1", HarnessEnumToString(std::get<0>(info.param)),
-                                            std::get<1>(info.param).description);
-                        });
+INSTANTIATE_TEST_SUITE_P(AuthzFuncCombinations,
+                         AuthzErrorHandlingTest,
+                         ::testing::Combine(::testing::Values(kRanger),
+                                            ::testing::ValuesIn(kAuthzFuncCombinations)),
+                         [](const testing::TestParamInfo<AuthzErrorHandlingTest::ParamType>& info) {
+                           return Substitute("$0_$1", HarnessEnumToString(std::get<0>(info.param)),
+                                             std::get<1>(info.param).description);
+                         });
 
 } // namespace kudu
