@@ -75,6 +75,7 @@ class MasterHmsTest : public ExternalMiniClusterITestBase {
     opts.num_masters = 1;
     opts.num_tablet_servers = 1;
     opts.enable_kerberos = EnableKerberos();
+    opts.principal = Principal();
     // Tune down the notification log poll period in order to speed up catalog convergence.
     opts.extra_master_flags.emplace_back("--hive_metastore_notification_log_poll_period_seconds=1");
     StartClusterWithOpts(std::move(opts));
@@ -129,6 +130,12 @@ class MasterHmsTest : public ExternalMiniClusterITestBase {
     return harness_.ChangeHmsOwner(database_name, table_name, new_table_owner);
   }
 
+  Status ChangeHmsTableComment(const std::string& database_name,
+                               const std::string& table_name,
+                               const std::string& new_table_comment) {
+    return harness_.ChangeHmsTableComment(database_name, table_name, new_table_comment);
+  }
+
   Status AlterHmsTableDropColumns(const std::string& database_name,
                                   const std::string& table_name) {
     return harness_.AlterHmsTableDropColumns(database_name, table_name);
@@ -164,6 +171,10 @@ class MasterHmsTest : public ExternalMiniClusterITestBase {
   virtual bool EnableKerberos() {
     return false;
   }
+
+  virtual string Principal() {
+    return "kudu";
+  }
 };
 
 TEST_F(MasterHmsTest, TestCreateTable) {
@@ -323,6 +334,27 @@ TEST_F(MasterHmsTest, TestAlterTableOwner) {
   });
 }
 
+TEST_F(MasterHmsTest, TestAlterTableComment) {
+  // Create the Kudu table.
+  ASSERT_OK(CreateKuduTable("default", "commentTable"));
+  NO_FATALS(CheckTable("default", "commentTable", /*user=*/ none));
+
+  // Change the table comment through the HMS, and ensure the comment is handled in Kudu.
+  const string comment_a = "comment a";
+  ASSERT_OK(ChangeHmsTableComment("default", "commentTable", comment_a));
+  ASSERT_EVENTUALLY([&] {
+    NO_FATALS(CheckTable("default", "commentTable", /*user=*/ none));
+  });
+
+  // Change the table comment through Kudu, and ensure the comment is reflected in HMS.
+  const string comment_b = "comment b";
+  unique_ptr<KuduTableAlterer> table_alterer(client_->NewTableAlterer("default.commentTable"));
+  ASSERT_OK(table_alterer->SetComment(comment_b)->Alter());
+  ASSERT_EVENTUALLY([&] {
+    NO_FATALS(CheckTable("default", "commentTable", /*user=*/ none));
+  });
+}
+
 TEST_F(MasterHmsTest, TestAlterTable) {
   // Create the Kudu table.
   ASSERT_OK(CreateKuduTable("default", "a"));
@@ -677,7 +709,9 @@ TEST_F(MasterHmsTest, TestUppercaseIdentifiers) {
 TEST_F(MasterHmsTest, TestTransactionStatusTableDoesntSync) {
   // Create a transaction status table.
   unique_ptr<TxnSystemClient> txn_sys_client;
-  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_sys_client));
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->service_principal(),
+                                    &txn_sys_client));
   ASSERT_OK(txn_sys_client->CreateTxnStatusTable(100));
 
   // We shouldn't see the table in the HMS catalog.
@@ -711,11 +745,13 @@ TEST_F(MasterHmsUpgradeTest, TestConflictingNormalizedNames) {
   // itself leader, in which case ExternalMiniCluster::Restart() can succeed. In
   // this situation a fallback to a leader-only API will deterministically fail.
   Status s = cluster_->Restart();
-  if (s.ok()) {
+  if (!s.ok()) {
+    ASSERT_TRUE(s.IsNetworkError()) << s.ToString();
+  } else {
     vector<string> tables;
     s = client_->ListTables(&tables);
+    ASSERT_TRUE(s.IsTimedOut()) << s.ToString();
   }
-  ASSERT_TRUE(s.IsNetworkError()) << s.ToString();
 
   // Disable the metastore integration and rename one of the conflicting tables.
   cluster_->ShutdownNodes(cluster::ClusterNodes::MASTERS_ONLY);
@@ -769,6 +805,9 @@ class MasterHmsKerberizedTest : public MasterHmsTest {
   bool EnableKerberos() override {
     return true;
   }
+  string Principal() override {
+    return "oryx";
+  }
 };
 
 // Checks that table ownership in a Kerberized cluster is set to the user
