@@ -21,6 +21,7 @@
 #include <cstdio>
 #include <cstdlib>
 #include <functional>
+#include <initializer_list>
 #include <memory>
 #include <ostream>
 #include <string>
@@ -81,6 +82,8 @@ using kudu::client::KuduSchema;
 using kudu::client::KuduSession;
 using kudu::client::KuduTable;
 using kudu::client::KuduTableCreator;
+using kudu::client::KuduTransaction;
+using kudu::client::sp::shared_ptr;
 using kudu::cluster::ExternalMiniCluster;
 using kudu::cluster::ExternalMiniClusterOptions;
 using kudu::rpc::Messenger;
@@ -91,7 +94,7 @@ using strings::Substitute;
 
 namespace kudu {
 
-static const char* kTableName = "test-table";
+static const char* const kTableName = "test-table";
 static const Schema kTestSchema = CreateKeyValueTestSchema();
 static const KuduSchema kTestKuduSchema = client::KuduSchema::FromSchema(kTestSchema);
 
@@ -124,7 +127,7 @@ class SecurityITest : public KuduTest {
     return proxy.SetFlag(req, &resp, &controller);
   }
 
-  Status CreateTestTable(const client::sp::shared_ptr<KuduClient>& client) {
+  static Status CreateTestTable(const shared_ptr<KuduClient>& client) {
     unique_ptr<KuduTableCreator> table_creator(client->NewTableCreator());
     return table_creator->table_name(kTableName)
         .set_range_partition_columns({ "key" })
@@ -133,8 +136,9 @@ class SecurityITest : public KuduTest {
         .Create();
   }
 
-  // Create a table, insert a row, scan it back, and delete the table.
-  void SmokeTestCluster();
+  // Create a table, insert a row, scan it back, and then drop the table.
+  void SmokeTestCluster(const shared_ptr<KuduClient>& client = {},
+                        bool transactional = false);
 
   Status TryRegisterAsTS() {
     // Make a new messenger so that we don't reuse any cached connections from
@@ -198,29 +202,43 @@ class SecurityITest : public KuduTest {
   unique_ptr<ExternalMiniCluster> cluster_;
 };
 
-void SecurityITest::SmokeTestCluster() {
-  client::sp::shared_ptr<KuduClient> client;
-  ASSERT_OK(cluster_->CreateClient(nullptr, &client));
+void SecurityITest::SmokeTestCluster(const shared_ptr<KuduClient>& client,
+                                     const bool transactional) {
+  shared_ptr<KuduClient> new_client;
+  const shared_ptr<KuduClient>& c = client ? client : new_client;
+  if (!client) {
+    ASSERT_OK(cluster_->CreateClient(nullptr, &new_client));
+  }
 
   // Create a table.
-  ASSERT_OK(CreateTestTable(client));
+  ASSERT_OK(CreateTestTable(c));
 
   // Insert a row.
-  client::sp::shared_ptr<KuduTable> table;
-  ASSERT_OK(client->OpenTable(kTableName, &table));
-  client::sp::shared_ptr<KuduSession> session = client->NewSession();
+  shared_ptr<KuduTable> table;
+  ASSERT_OK(c->OpenTable(kTableName, &table));
+  shared_ptr<KuduTransaction> txn;
+  shared_ptr<KuduSession> session;
+  if (transactional) {
+    ASSERT_OK(c->NewTransaction(&txn));
+    ASSERT_OK(txn->CreateSession(&session));
+  } else {
+    session = c->NewSession();
+  }
   session->SetTimeoutMillis(60000);
   unique_ptr<KuduInsert> ins(table->NewInsert());
   ASSERT_OK(ins->mutable_row()->SetInt32(0, 12345));
   ASSERT_OK(ins->mutable_row()->SetInt32(1, 54321));
   ASSERT_OK(session->Apply(ins.release()));
   FlushSessionOrDie(session);
+  if (transactional) {
+    ASSERT_OK(txn->Commit());
+  }
 
-  // Read it back.
+  // Read the inserted row back.
   ASSERT_EQ(1, CountTableRows(table.get()));
 
-  // Delete the table.
-  ASSERT_OK(client->DeleteTable(kTableName));
+  // Drop the table.
+  ASSERT_OK(c->DeleteTable(kTableName));
 }
 
 // Test authorizing list tablets.
@@ -240,7 +258,7 @@ TEST_F(SecurityITest, TestAuthorizationOnListTablets) {
 TEST_F(SecurityITest, TestAuthorizationOnChecksum) {
   cluster_opts_.extra_tserver_flags.emplace_back("--tserver_enforce_access_control");
   ASSERT_OK(StartCluster());
-  client::sp::shared_ptr<KuduClient> client;
+  shared_ptr<KuduClient> client;
   ASSERT_OK(cluster_->CreateClient(nullptr, &client));
   ASSERT_OK(CreateTestTable(client));
   vector<string> tablet_ids;
@@ -264,10 +282,15 @@ TEST_F(SecurityITest, TestAuthorizationOnChecksum) {
 // Test creating a table, writing some data, reading data, and dropping
 // the table.
 TEST_F(SecurityITest, SmokeTestAsAuthorizedUser) {
+  cluster_opts_.extra_master_flags.emplace_back("--txn_manager_enabled=true");
+  cluster_opts_.extra_tserver_flags.emplace_back("--enable_txn_system_client_init=true");
   ASSERT_OK(StartCluster());
 
   ASSERT_OK(cluster_->kdc()->Kinit("test-user"));
-  NO_FATALS(SmokeTestCluster());
+  shared_ptr<KuduClient> client;
+  ASSERT_OK(cluster_->CreateClient(nullptr, &client));
+  NO_FATALS(SmokeTestCluster(client));
+  NO_FATALS(SmokeTestCluster(client, /* transactional */ true));
 
   // Non-superuser clients should not be able to set flags.
   Status s = TrySetFlagOnTS();
@@ -280,12 +303,119 @@ TEST_F(SecurityITest, SmokeTestAsAuthorizedUser) {
             s.ToString());
 }
 
+// Make sure that it's possible to call TxnManagerService's txn-related RPCs
+// using credentials of a regular and a super user.
+TEST_F(SecurityITest, TxnSmokeWithDifferentUserTypes) {
+  constexpr const auto kTxnKeepaliveIntervalMs = 500;
+  const auto usernames = {
+    "test-admin",
+    "test-user",
+  };
+
+  cluster_opts_.extra_master_flags.emplace_back(
+      "--txn_manager_enabled=true");
+  cluster_opts_.extra_tserver_flags.emplace_back(
+      "--enable_txn_system_client_init=true");
+  cluster_opts_.extra_tserver_flags.emplace_back(
+      Substitute("--txn_keepalive_interval_ms=$0", kTxnKeepaliveIntervalMs));
+  ASSERT_OK(StartCluster());
+
+  for (const auto& username : usernames) {
+    SCOPED_TRACE(Substitute("Running with credentials of user $0", username));
+    ASSERT_OK(cluster_->kdc()->Kinit(username));
+
+    shared_ptr<KuduClient> client;
+    ASSERT_OK(cluster_->CreateClient(nullptr, &client));
+    NO_FATALS(SmokeTestCluster(client, /* transactional */ true));
+
+    // Now, check for the rest of TxnManagerService RPCs.
+    unique_ptr<KuduTableCreator> table_creator(client->NewTableCreator());
+    ASSERT_OK(table_creator->table_name(kTableName)
+              .set_range_partition_columns({ "key" })
+              .schema(&kTestKuduSchema)
+              .num_replicas(3)
+              .Create());
+
+    auto smoke_starter = [&](shared_ptr<KuduTable>* table,
+                             shared_ptr<KuduTransaction>* txn) {
+      RETURN_NOT_OK(client->OpenTable(kTableName, table));
+      RETURN_NOT_OK(client->NewTransaction(txn));
+      shared_ptr<KuduSession> session;
+      RETURN_NOT_OK((*txn)->CreateSession(&session));
+      RETURN_NOT_OK(session->SetFlushMode(KuduSession::MANUAL_FLUSH));
+      {
+        unique_ptr<KuduInsert> ins((*table)->NewInsert());
+        RETURN_NOT_OK(ins->mutable_row()->SetInt32(0, 0));
+        RETURN_NOT_OK(ins->mutable_row()->SetInt32(1, 1));
+        RETURN_NOT_OK(session->Apply(ins.release()));
+      }
+      return session->Flush();
+    };
+
+    // Check TxnManagerService::AbortTransaction() RPC as well.
+    {
+      shared_ptr<KuduTable> table;
+      shared_ptr<KuduTransaction> txn;
+      ASSERT_OK(smoke_starter(&table, &txn));
+      ASSERT_OK(txn->Rollback());
+
+      // Wait for the transaction to complete the rollback phase. This is useful
+      // because the next sub-scenario below starts a new transaction as well,
+      // and in case of a race it may happen that the transaction below starts
+      // before this one finalizes its abort phase. Both transactions attempt
+      // to lock the same partition(s), so the latter is automatically aborted
+      // by the deadlock prevention logic.
+      ASSERT_EVENTUALLY([&]{
+        bool is_complete = false;
+        Status completion_status;
+        ASSERT_OK(txn->IsCommitComplete(&is_complete, &completion_status));
+        ASSERT_TRUE(is_complete);
+        ASSERT_TRUE(completion_status.IsAborted()) << completion_status.ToString();
+      });
+
+      // Read the inserted row back.
+      ASSERT_EQ(0, CountTableRows(table.get()));
+    }
+
+    // Check TxnManagerService::KeepTransactionAlive() -- wait for for over than
+    // keepalive interval for a transaction and make sure it's still possible to
+    // commit a transaction using its handle. Calling StartCommit() and then
+    // calling IsCommitComplete() checks calling of the
+    // TxnManagerService::GetTransactionState() RPC.
+    {
+      shared_ptr<KuduTable> table;
+      shared_ptr<KuduTransaction> txn;
+      ASSERT_OK(smoke_starter(&table, &txn));
+
+      // Wait for a few keepalive intervals: the transaction would be aborted if
+      // calls to TxnManagerService::KeepTransactionAlive() RPC do not come
+      // through, and then KuduTransaction::StartCommit() below would fail.
+      SleepFor(MonoDelta::FromMilliseconds(3 * kTxnKeepaliveIntervalMs));
+
+      ASSERT_OK(txn->StartCommit());
+      ASSERT_EVENTUALLY([&]{
+        bool is_complete = false;
+        Status completion_status;
+        ASSERT_OK(txn->IsCommitComplete(&is_complete, &completion_status));
+        ASSERT_TRUE(is_complete);
+        ASSERT_OK(completion_status);
+      });
+
+      // Read the inserted row back.
+      ASSERT_EQ(1, CountTableRows(table.get()));
+    }
+
+    // Finally, drop the table.
+    ASSERT_OK(client->DeleteTable(kTableName));
+  }
+}
+
 // Test trying to access the cluster with no Kerberos credentials at all.
 TEST_F(SecurityITest, TestNoKerberosCredentials) {
   ASSERT_OK(StartCluster());
   ASSERT_OK(cluster_->kdc()->Kdestroy());
 
-  client::sp::shared_ptr<KuduClient> client;
+  shared_ptr<KuduClient> client;
   Status s = cluster_->CreateClient(nullptr, &client);
   ASSERT_STR_MATCHES(s.ToString(),
                      "Not authorized: Could not connect to the cluster: "
@@ -305,7 +435,7 @@ TEST_F(SecurityITest, SaslPlainFallback) {
   ASSERT_OK(StartCluster());
   ASSERT_OK(cluster_->kdc()->Kdestroy());
 
-  client::sp::shared_ptr<KuduClient> client;
+  shared_ptr<KuduClient> client;
   ASSERT_OK(cluster_->CreateClient(nullptr, &client));
 
   // Check client can successfully call ListTables().
@@ -317,7 +447,7 @@ TEST_F(SecurityITest, SaslPlainFallback) {
 TEST_F(SecurityITest, TestUnauthorizedClientKerberosCredentials) {
   ASSERT_OK(StartCluster());
   ASSERT_OK(cluster_->kdc()->Kinit("joe-interloper"));
-  client::sp::shared_ptr<KuduClient> client;
+  shared_ptr<KuduClient> client;
   Status s = cluster_->CreateClient(nullptr, &client);
   ASSERT_EQ("Remote error: Could not connect to the cluster: "
             "Not authorized: unauthorized access to method: ConnectToMaster",
@@ -376,9 +506,12 @@ void GetFullBinaryPath(string* binary) {
 TEST_F(SecurityITest, TestWorldReadableKeytab) {
   const string credentials_name = GetTestPath("insecure.keytab");
   NO_FATALS(CreateWorldReadableFile(credentials_name));
-  string binary = "kudu-master";
+  string binary = "kudu";
   NO_FATALS(GetFullBinaryPath(&binary));
-  const vector<string> argv = { binary, Substitute("--keytab_file=$0", credentials_name) };
+  const vector<string> argv = { binary,
+                                "master",
+                                "run",
+                                Substitute("--keytab_file=$0", credentials_name) };
   string stderr;
   Status s = Subprocess::Call(argv, "", nullptr, &stderr);
   ASSERT_STR_CONTAINS(stderr, Substitute(
@@ -389,9 +522,11 @@ TEST_F(SecurityITest, TestWorldReadableKeytab) {
 TEST_F(SecurityITest, TestWorldReadablePrivateKey) {
   const string credentials_name = GetTestPath("insecure.key");
   NO_FATALS(CreateWorldReadableFile(credentials_name));
-  string binary = "kudu-master";
+  string binary = "kudu";
   NO_FATALS(GetFullBinaryPath(&binary));
   const vector<string> argv = { binary,
+                                "master",
+                                "run",
                                 "--unlock_experimental_flags",
                                 Substitute("--rpc_private_key_file=$0", credentials_name),
                                 "--rpc_certificate_file=fake_file",
@@ -440,6 +575,70 @@ TEST_F(SecurityITest, TestCorruptKerberosCC) {
   }
 }
 
+TEST_F(SecurityITest, TestNonDefaultPrincipal) {
+  const string kPrincipal = "oryx";
+  cluster_opts_.principal = kPrincipal;
+  // Enable TxnManager in Kudu masters: it's necessary to test txn-related
+  // operations along with others.
+  cluster_opts_.extra_master_flags.emplace_back("--txn_manager_enabled=true");
+  cluster_opts_.extra_tserver_flags.emplace_back("--enable_txn_system_client_init=true");
+  ASSERT_OK(StartCluster());
+
+  // A client with the default SASL proto shouldn't be able to connect to
+  // a cluster using custom Kerberos principal for Kudu service user. Verify
+  // that for both the regular and the super-user.
+  for (const auto& username : {"test-user", "test-admin"}) {
+    ASSERT_OK(cluster_->kdc()->Kinit(username));
+
+    // Instantiate a KuduClientBuilder outside of this cluster's context, so
+    // the custom service principals for this cluster don't affect the default
+    // SASL proto name when creating this separate client instance.
+    shared_ptr<KuduClient> client;
+    KuduClientBuilder builder;
+    for (auto i = 0; i < cluster_->num_masters(); ++i) {
+      builder.add_master_server_addr(cluster_->master(i)->bound_rpc_addr().ToString());
+    }
+    const auto s = builder.Build(&client);
+    ASSERT_TRUE(s.IsNotAuthorized());
+    ASSERT_STR_CONTAINS(s.ToString(), "not found in Kerberos database");
+
+    // Create a client with the matching SASL proto name and verify it's able to
+    // connect to the cluster and perform basic actions.
+    {
+      // Here we don't use out-of-this-cluster KuduClientBuilder instance,
+      // so the client is created with matching SASL proto name.
+      shared_ptr<KuduClient> client;
+      ASSERT_OK(cluster_->CreateClient(nullptr, &client));
+      SmokeTestCluster(client);
+      SmokeTestCluster(client, /*transactional*/ true);
+    }
+  }
+}
+
+TEST_F(SecurityITest, TestNonExistentPrincipal) {
+  cluster_opts_.extra_master_flags.emplace_back("--principal=oryx");
+  cluster_opts_.extra_tserver_flags.emplace_back("--principal=oryx");
+  Status s = StartCluster();
+  ASSERT_TRUE(s.IsRuntimeError());
+  ASSERT_STR_CONTAINS(s.ToString(), "failed to start masters");
+}
+
+TEST_F(SecurityITest, TestMismatchingPrincipals) {
+  ASSERT_OK(StartCluster());
+  string keytab_path;
+  const string kPrincipalBase = "oryx";
+  for (auto i = 0; i < cluster_->num_tablet_servers(); i++) {
+    cluster::ExternalTabletServer* tserver = cluster_->tablet_server(i);
+    const string kPrincipal = kPrincipalBase + "/" + tserver->bound_rpc_addr().host();
+    ASSERT_OK(cluster_->kdc()->CreateServiceKeytab(kPrincipal, &keytab_path));
+    tserver->mutable_flags()->emplace_back("--principal=" + kPrincipal);
+    tserver->mutable_flags()->emplace_back("--keytab_file=" + keytab_path);
+  }
+  cluster_->Shutdown();
+  Status s = cluster_->Restart();
+  ASSERT_TRUE(s.IsTimedOut());
+}
+
 Status AssignIPToClient(bool external) {
   // If the test does not require an external IP address
   // assign loopback IP to FLAGS_local_ip_for_outbound_sockets.
@@ -491,7 +690,7 @@ class AuthTokenIssuingTest :
     public ::testing::WithParamInterface<AuthTokenIssuingTestParams> {
 };
 
-INSTANTIATE_TEST_CASE_P(, AuthTokenIssuingTest, ::testing::ValuesIn(
+INSTANTIATE_TEST_SUITE_P(, AuthTokenIssuingTest, ::testing::ValuesIn(
     vector<AuthTokenIssuingTestParams>{
       // The following 3 test cases cover passing authn token over an
       // encrypted loopback connection.
@@ -600,7 +799,7 @@ TEST_P(AuthTokenIssuingTest, ChannelConfidentiality) {
 
   // In current implementation, KuduClientBuilder calls ConnectToCluster() on
   // the newly created instance of the KuduClient.
-  client::sp::shared_ptr<KuduClient> client;
+  shared_ptr<KuduClient> client;
   ASSERT_OK(cluster_->CreateClient(nullptr, &client));
 
   string authn_creds;
@@ -626,7 +825,7 @@ class ConnectToFollowerMasterTest :
     public SecurityITest,
     public ::testing::WithParamInterface<ConnectToFollowerMasterTestParams> {
 };
-INSTANTIATE_TEST_CASE_P(, ConnectToFollowerMasterTest, ::testing::ValuesIn(
+INSTANTIATE_TEST_SUITE_P(, ConnectToFollowerMasterTest, ::testing::ValuesIn(
     vector<ConnectToFollowerMasterTestParams>{
       { "required", "optional", },
       { "required", "required", },
@@ -668,7 +867,7 @@ TEST_P(ConnectToFollowerMasterTest, AuthnTokenVerifierHaveKeys) {
   // Get authentication credentials.
   string authn_creds;
   {
-    client::sp::shared_ptr<KuduClient> client;
+    shared_ptr<KuduClient> client;
     ASSERT_OK(cluster_->CreateClient(nullptr, &client));
     ASSERT_OK(client->ExportAuthenticationCredentials(&authn_creds));
   }
@@ -679,7 +878,7 @@ TEST_P(ConnectToFollowerMasterTest, AuthnTokenVerifierHaveKeys) {
   // Make sure it's not possible to connect without authn token at this point:
   // the server side is configured to require authentication.
   {
-    client::sp::shared_ptr<KuduClient> client;
+    shared_ptr<KuduClient> client;
     KuduClientBuilder builder;
     for (auto i = 0; i < cluster_->num_masters(); ++i) {
       builder.add_master_server_addr(cluster_->master(i)->bound_rpc_addr().ToString());
@@ -696,7 +895,7 @@ TEST_P(ConnectToFollowerMasterTest, AuthnTokenVerifierHaveKeys) {
   // of masters' endpoints while trying to connect to a multi-master Kudu cluster.
   ASSERT_EVENTUALLY([&] {
     for (auto i = 1; i < cluster_->num_masters(); ++i) {
-      client::sp::shared_ptr<KuduClient> client;
+      shared_ptr<KuduClient> client;
       const auto s = KuduClientBuilder()
           .add_master_server_addr(cluster_->master(i)->bound_rpc_addr().ToString())
           .import_authentication_credentials(authn_creds)
@@ -712,7 +911,7 @@ TEST_P(ConnectToFollowerMasterTest, AuthnTokenVerifierHaveKeys) {
   // to connect and perform basic operations (like listing tables) when using
   // secondary credentials only (i.e. authn token).
   {
-    client::sp::shared_ptr<KuduClient> client;
+    shared_ptr<KuduClient> client;
     KuduClientBuilder builder;
     for (auto i = 0; i < cluster_->num_masters(); ++i) {
       builder.add_master_server_addr(cluster_->master(i)->bound_rpc_addr().ToString());
