@@ -31,6 +31,7 @@
 #include "kudu/client/client.h"
 #include "kudu/client/schema.h"
 #include "kudu/client/shared_ptr.h" // IWYU pragma: keep
+#include "kudu/common/txn_id.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/util/atomic.h"
 #include "kudu/util/countdown_latch.h"
@@ -59,9 +60,62 @@ class TestWorkload {
  public:
   static const char* const kDefaultTableName;
 
-  explicit TestWorkload(cluster::MiniCluster* cluster);
+  // Kudu provides two types of partitioning for a table: range partitioning and
+  // hash partitioning. This enum contains corresponding elements to be used
+  // to define partitioning type for the test table. Multi-level partitioning
+  // for the test table automatically created by TestWorkload isn't supported
+  // yet: as a workaround, point TestWorkload to already existing table
+  // with desired partitioning type.
+  enum class PartitioningType {
+    RANGE,
+    HASH,
+  };
+
+  // Create an test workload object to run against the specified cluster
+  // using a test table of the specified partitioning type.
+  explicit TestWorkload(
+      cluster::MiniCluster* cluster,
+      PartitioningType partitioning = PartitioningType::RANGE);
   ~TestWorkload();
 
+  // Ingest the workload as part of the given transaction. set_begin_txn() must
+  // not be called if this is set.
+  void set_txn_id(int64_t txn_id) {
+    txn_id_ = TxnId(txn_id);
+    CHECK(txn_id_.IsValid());
+  }
+
+  // Ingest the workload as a part of a new transaction. set_txn_id() must not
+  // be called if this is set.
+  void set_begin_txn() {
+    begin_txn_ = true;
+    CHECK(!txn_id_.IsValid());
+  }
+
+  // Commit the transaction that this workload is a part of upon calling
+  // StopAndJoin(). If set, either set_begin_txn() or set_txn_id() must be set
+  // as well. If not set, but either set_begin_txn() or set_txn_id() is set,
+  // the workload will ingest as a part of the transaction, but not call
+  // commit on completion.
+  //
+  // set_rollback_txn() must not be called if this is set.
+  void set_commit_txn() {
+    commit_txn_ = true;
+    CHECK(!rollback_txn_);
+  }
+
+  // Abort the transaction that this workload is a part of upon calling
+  // StopAndJoin(). If set, either set_begin_txn() or set_txn_id() must be set
+  // as well. If not set, but either set_begin_txn() or set_txn_id() is set,
+  // the workload will ingest as a part of the transaction, but not call abort
+  // on completion.
+  //
+  // set_commit_txn() must not be called if this is set.
+  void set_rollback_txn() {
+    rollback_txn_ = true;
+    CHECK(!commit_txn_);
+  }
+
   // Sets whether the read thread should crash if scanning to the cluster fails
   // for whatever reason. If set to true, errors will be populated in
   // 'read_errors_'.
@@ -137,6 +191,9 @@ class TestWorkload {
 
   // Set whether we should attempt to verify the number of rows when scanning.
   // An incorrect number of rows may be indicative of a stale read.
+  //
+  // If either set_begin_txn() or set_txn_id() has been called, does not verify
+  // the number of rows.
   void set_verify_num_rows(bool should_verify) {
     verify_num_rows_ = should_verify;
   }
@@ -154,11 +211,16 @@ class TestWorkload {
     num_replicas_ = r;
   }
 
-  // Set the number of tablets for the table created by this workload.
-  // The split points are evenly distributed through positive int32s.
-  void set_num_tablets(int tablets) {
-    CHECK_GE(tablets, 1);
-    num_tablets_ = tablets;
+  // Set the number of tablets for the test table created by this workload.
+  // In case of range partitioning, the split points are evenly distributed
+  // through positive int32s.
+  void set_num_tablets(int num_tablets) {
+    if (partitioning_ == PartitioningType::RANGE) {
+      CHECK_GE(num_tablets, 1);
+    } else {
+      CHECK_GE(num_tablets, 2);
+    }
+    num_tablets_ = num_tablets;
   }
 
   void set_table_name(const std::string& table_name) {
@@ -192,7 +254,10 @@ class TestWorkload {
 
     // Insert sequential rows.
     // This causes flushes but no compactions.
-    INSERT_SEQUENTIAL_ROWS
+    INSERT_SEQUENTIAL_ROWS,
+
+    // Insert sequential rows, then delete them.
+    INSERT_SEQUENTIAL_ROWS_WITH_DELETE,
   };
 
   void set_write_pattern(WritePattern pattern) {
@@ -205,6 +270,7 @@ class TestWorkload {
       case INSERT_RANDOM_ROWS_WITH_DELETE:
       case UPDATE_ONE_ROW:
       case INSERT_SEQUENTIAL_ROWS:
+      case INSERT_SEQUENTIAL_ROWS_WITH_DELETE:
         set_already_present_allowed(false);
         break;
       default: LOG(FATAL) << "Unsupported WritePattern.";
@@ -226,8 +292,14 @@ class TestWorkload {
   // Delete created table, etc.
   Status Cleanup();
 
+  int64_t txn_id() const {
+    CHECK(txn_id_.IsValid());
+    return txn_id_.value();
+  }
+
   // Return the number of rows inserted so far. This may be called either
-  // during or after the write workload.
+  // during or after the write workload. If writing as a part of a transaction,
+  // these rows may have not been committed.
   int64_t rows_inserted() const {
     return rows_inserted_.Load();
   }
@@ -262,6 +334,7 @@ class TestWorkload {
   size_t GetNumberOfErrors(client::KuduSession* session);
 
   cluster::MiniCluster* cluster_;
+  const PartitioningType partitioning_;
   client::KuduClientBuilder client_builder_;
   client::sp::shared_ptr<client::KuduClient> client_;
   ThreadSafeRandom rng_;
@@ -273,6 +346,10 @@ class TestWorkload {
   int write_batch_size_;
   int write_interval_millis_;
   int write_timeout_millis_;
+  TxnId txn_id_;
+  bool begin_txn_;
+  bool commit_txn_;
+  bool rollback_txn_;
   bool fault_tolerant_;
   bool verify_num_rows_;
   bool read_errors_allowed_;
@@ -296,6 +373,8 @@ class TestWorkload {
   AtomicInt<int64_t> batches_completed_;
   AtomicInt<int32_t> sequential_key_gen_;
 
+  client::sp::shared_ptr<client::KuduTransaction> txn_;
+
   std::vector<std::thread> threads_;
 
   mutable simple_spinlock read_error_lock_;
