@@ -68,6 +68,7 @@
 #include "kudu/tablet/metadata.pb.h"
 #include "kudu/tablet/tablet_replica.h"
 #include "kudu/tablet/txn_coordinator.h"
+#include "kudu/transactions/txn_status_manager.h"
 #include "kudu/transactions/txn_status_tablet.h"
 #include "kudu/tserver/heartbeater.h"
 #include "kudu/tserver/mini_tablet_server.h"
@@ -93,6 +94,7 @@ DECLARE_bool(allow_unsafe_replication_factor);
 DECLARE_bool(catalog_manager_evict_excess_replicas);
 DECLARE_bool(catalog_manager_wait_for_new_tablets_to_elect_leader);
 DECLARE_bool(enable_leader_failure_detection);
+DECLARE_bool(enable_txn_system_client_init);
 DECLARE_bool(raft_prepare_replacement_before_eviction);
 DECLARE_double(leader_failure_max_missed_heartbeat_periods);
 DECLARE_int32(heartbeat_interval_ms);
@@ -140,6 +142,7 @@ using kudu::rpc::RpcController;
 using kudu::tablet::TabletReplica;
 using kudu::tablet::ParticipantIdsByTxnId;
 using kudu::tablet::TxnCoordinator;
+using kudu::transactions::TxnStatusManager;
 using kudu::transactions::TxnStatusTablet;
 using kudu::tserver::MiniTabletServer;
 using kudu::ClusterVerifier;
@@ -294,9 +297,9 @@ TEST_P(FailedTabletsAreReplacedITest, OneReplica) {
   // Ensure the tablet eventually is replicated.
   NO_FATALS(v.CheckCluster());
 }
-INSTANTIATE_TEST_CASE_P(,
-                        FailedTabletsAreReplacedITest,
-                        ::testing::Bool());
+INSTANTIATE_TEST_SUITE_P(,
+                         FailedTabletsAreReplacedITest,
+                         ::testing::Bool());
 
 class LeadershipChangeReportingTest : public TsTabletManagerITest {
  public:
@@ -1031,6 +1034,11 @@ class TxnStatusTabletManagementTest : public TsTabletManagerITest {
   const char* kTxnStatusTabletId = "11111111111111111111111111111111";
   const MonoDelta kTimeout = MonoDelta::FromSeconds(30);
 
+  void SetUp() override {
+    NO_FATALS(TsTabletManagerITest::SetUp());
+    FLAGS_enable_txn_system_client_init = true;
+  }
+
   // Creates a request to create a transaction status tablet with the given IDs
   // and Raft config.
   static CreateTabletRequestPB CreateTxnTabletReq(const string& tablet_id, const string& replica_id,
@@ -1110,6 +1118,9 @@ class TxnStatusTabletManagementTest : public TsTabletManagerITest {
   }
 
   static Status StartTransactions(const ParticipantIdsByTxnId& txns, TxnCoordinator* coordinator) {
+    TxnStatusManager* txn_status_manager =
+        dynamic_cast<TxnStatusManager*>(coordinator);
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_status_manager);
     TabletServerErrorPB ts_error;
     for (const auto& txn_id_and_prt_ids : txns) {
       const auto& txn_id = txn_id_and_prt_ids.first;
@@ -1212,8 +1223,18 @@ TEST_F(TxnStatusTabletManagementTest, TestCopyTransactionStatusTablet) {
     ASSERT_OK(replica->WaitUntilConsensusRunning(MonoDelta::FromSeconds(3)));
     TxnCoordinator* coordinator = replica->txn_coordinator();
     ASSERT_NE(nullptr, coordinator);
+  }
+  // Restart the tserver and ensure the transactions information is
+  // correctly reloaded.
+  {
+    ts0->Shutdown();
+    ASSERT_OK(ts0->Restart());
     // Wait for the contents of the tablet to be loaded into memory.
     ASSERT_EVENTUALLY([&] {
+      scoped_refptr<TabletReplica> replica;
+      ASSERT_TRUE(ts0->server()->tablet_manager()->LookupTablet(
+          kTxnStatusTabletId, &replica));
+      TxnCoordinator* coordinator = replica->txn_coordinator();
       ASSERT_EQ(kExpectedTxns, coordinator->GetParticipantsByTxnIdForTests());
     });
   }
