@@ -34,10 +34,10 @@
 #include "kudu/clock/clock.h"
 #include "kudu/common/partial_row.h"
 #include "kudu/common/row_operations.h"
+#include "kudu/common/row_operations.pb.h"
 #include "kudu/common/timestamp.h"
 #include "kudu/common/wire_protocol-test-util.h"
 #include "kudu/common/wire_protocol.h"
-#include "kudu/common/wire_protocol.pb.h"
 #include "kudu/consensus/raft_consensus.h"
 #include "kudu/consensus/time_manager.h"
 #include "kudu/gutil/ref_counted.h"
@@ -47,6 +47,7 @@
 #include "kudu/integration-tests/cluster_itest_util.h"
 #include "kudu/integration-tests/test_workload.h"
 #include "kudu/mini-cluster/internal_mini_cluster.h"
+#include "kudu/rpc/messenger.h"
 #include "kudu/rpc/rpc_controller.h"
 #include "kudu/tablet/metadata.pb.h"
 #include "kudu/tablet/mvcc.h"
@@ -72,6 +73,7 @@
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
+DECLARE_bool(enable_txn_partition_lock);
 DECLARE_bool(raft_enable_pre_election);
 DECLARE_double(leader_failure_max_missed_heartbeat_periods);
 DECLARE_int32(consensus_inject_latency_ms_in_notifications);
@@ -97,7 +99,7 @@ using kudu::tablet::kInitializing;
 using kudu::tablet::kOpen;
 using kudu::tablet::MakeParticipantOp;
 using kudu::tablet::TabletReplica;
-using kudu::tablet::TxnState;
+using kudu::tablet::TxnMetadataPB;
 using kudu::tablet::TxnParticipant;
 using kudu::tablet::TxnState;
 using kudu::transactions::TxnSystemClient;
@@ -428,7 +430,7 @@ TEST_P(ParticipantCopyITest, TestCopyParticipantOps) {
     }
   });
 }
-INSTANTIATE_TEST_CASE_P(ShouldGCWals, ParticipantCopyITest, ::testing::Values(true, false));
+INSTANTIATE_TEST_SUITE_P(ShouldGCWals, ParticipantCopyITest, ::testing::Values(true, false));
 
 // Test to ensure that the mechanisms built to allow snapshot scans to wait for
 // safe time advancement will actually wait for transactions to commit.
@@ -549,6 +551,70 @@ TEST_F(TxnParticipantITest, TestProxyBasicCalls) {
   }
 }
 
+TEST_F(TxnParticipantITest, TestBeginCommitAfterFinalize) {
+  constexpr const int kLeaderIdx = 0;
+  constexpr const int kTxnId = 0;
+  vector<TabletReplica*> replicas = SetUpLeaderGetReplicas(kLeaderIdx);
+  ASSERT_OK(replicas[kLeaderIdx]->consensus()->WaitUntilLeaderForTests(kDefaultTimeout));
+  auto admin_proxy = cluster_->tserver_admin_proxy(kLeaderIdx);
+  const auto tablet_id = replicas[kLeaderIdx]->tablet_id();
+  {
+    TabletServerErrorPB::Code code = TabletServerErrorPB::UNKNOWN_ERROR;
+    ASSERT_OK(ParticipateInTransactionCheckResp(
+        admin_proxy.get(), tablet_id, kTxnId, ParticipantOpPB::BEGIN_TXN, &code));
+    ASSERT_EQ(TabletServerErrorPB::UNKNOWN_ERROR, code);
+  }
+  // Commit the transaction.
+  Timestamp begin_commit_ts;
+  {
+    TabletServerErrorPB::Code code = TabletServerErrorPB::UNKNOWN_ERROR;
+    ASSERT_OK(ParticipateInTransactionCheckResp(
+        admin_proxy.get(), tablet_id, kTxnId, ParticipantOpPB::BEGIN_COMMIT,
+        &code, &begin_commit_ts));
+    ASSERT_EQ(TabletServerErrorPB::UNKNOWN_ERROR, code);
+    ASSERT_NE(Timestamp::kInvalidTimestamp, begin_commit_ts);
+  }
+  {
+    TabletServerErrorPB::Code code = TabletServerErrorPB::UNKNOWN_ERROR;
+    ASSERT_OK(ParticipateInTransactionCheckResp(
+        admin_proxy.get(), tablet_id, kTxnId, ParticipantOpPB::FINALIZE_COMMIT, &code));
+    ASSERT_EQ(TabletServerErrorPB::UNKNOWN_ERROR, code);
+  }
+  // A call to BEGIN_COMMIT should yield the finalized commit timestamp.
+  Timestamp refetched_begin_commit_ts;
+  {
+    TabletServerErrorPB::Code code = TabletServerErrorPB::UNKNOWN_ERROR;
+    Status s = ParticipateInTransactionCheckResp(
+        admin_proxy.get(), tablet_id, kTxnId, ParticipantOpPB::BEGIN_COMMIT, &code,
+        &refetched_begin_commit_ts);
+    ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
+    ASSERT_EQ(TabletServerErrorPB::TXN_OP_ALREADY_APPLIED, code);
+    ASSERT_NE(Timestamp::kInvalidTimestamp, refetched_begin_commit_ts);
+    ASSERT_EQ(Timestamp(kDummyCommitTimestamp), refetched_begin_commit_ts);
+  }
+}
+
+TEST_F(TxnParticipantITest, TestProxyErrorWhenNotBegun) {
+  constexpr const int kLeaderIdx = 0;
+  auto txn_id = 0;
+  vector<TabletReplica*> replicas = SetUpLeaderGetReplicas(kLeaderIdx);
+  ASSERT_OK(replicas[kLeaderIdx]->consensus()->WaitUntilLeaderForTests(kDefaultTimeout));
+  auto admin_proxy = cluster_->tserver_admin_proxy(kLeaderIdx);
+  const auto tablet_id = replicas[kLeaderIdx]->tablet_id();
+  for (auto type : { ParticipantOpPB::BEGIN_COMMIT,
+                     ParticipantOpPB::FINALIZE_COMMIT }) {
+    TabletServerErrorPB::Code code = TabletServerErrorPB::UNKNOWN_ERROR;
+    Status s = ParticipateInTransactionCheckResp(
+        admin_proxy.get(), tablet_id, txn_id++, type, &code);
+    ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
+    ASSERT_EQ(TabletServerErrorPB::TXN_ILLEGAL_STATE, code);
+  }
+  TabletServerErrorPB::Code code = TabletServerErrorPB::UNKNOWN_ERROR;
+  ASSERT_OK(ParticipateInTransactionCheckResp(
+      admin_proxy.get(), tablet_id, txn_id++, ParticipantOpPB::ABORT_TXN, &code));
+  ASSERT_EQ(TabletServerErrorPB::UNKNOWN_ERROR, code);
+}
+
 TEST_F(TxnParticipantITest, TestProxyIllegalStatesInCommitSequence) {
   constexpr const int kLeaderIdx = 0;
   constexpr const int kTxnId = 0;
@@ -581,7 +647,7 @@ TEST_F(TxnParticipantITest, TestProxyIllegalStatesInCommitSequence) {
     ASSERT_EQ(TabletServerErrorPB::TXN_ILLEGAL_STATE, code);
   }
 
-  // Start commititng and ensure we can't start another transaction.
+  // Start committing and ensure we can't start another transaction.
   Timestamp begin_commit_ts;
   {
     TabletServerErrorPB::Code code = TabletServerErrorPB::UNKNOWN_ERROR;
@@ -609,22 +675,21 @@ TEST_F(TxnParticipantITest, TestProxyIllegalStatesInCommitSequence) {
     ASSERT_EQ(TabletServerErrorPB::TXN_ILLEGAL_STATE, code);
   }
 
-  // Finalize the commit and ensure we can do nothing else.
+  // Finalize the commit and ensure we can't begin or abort.
   {
     TabletServerErrorPB::Code code = TabletServerErrorPB::UNKNOWN_ERROR;
     ASSERT_OK(ParticipateInTransactionCheckResp(
         admin_proxy.get(), tablet_id, kTxnId, ParticipantOpPB::FINALIZE_COMMIT, &code));
     ASSERT_EQ(TabletServerErrorPB::UNKNOWN_ERROR, code);
   }
-  {
+  for (auto type : { ParticipantOpPB::BEGIN_COMMIT, ParticipantOpPB::FINALIZE_COMMIT }) {
     TabletServerErrorPB::Code code = TabletServerErrorPB::UNKNOWN_ERROR;
     Status s = ParticipateInTransactionCheckResp(
-        admin_proxy.get(), tablet_id, kTxnId, ParticipantOpPB::FINALIZE_COMMIT, &code);
+        admin_proxy.get(), tablet_id, kTxnId, type, &code);
     ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
     ASSERT_EQ(TabletServerErrorPB::TXN_OP_ALREADY_APPLIED, code);
   }
   for (auto type : { ParticipantOpPB::BEGIN_TXN,
-                     ParticipantOpPB::BEGIN_COMMIT,
                      ParticipantOpPB::ABORT_TXN }) {
     TabletServerErrorPB::Code code = TabletServerErrorPB::UNKNOWN_ERROR;
     Status s = ParticipateInTransactionCheckResp(
@@ -767,6 +832,94 @@ TEST_F(TxnParticipantITest, TestProxyTabletNotFound) {
   }
 }
 
+TEST_F(TxnParticipantITest, TestTxnSystemClientGetMetadata) {
+  unique_ptr<TxnSystemClient> txn_client;
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->messenger()->sasl_proto_name(),
+                                    &txn_client));
+  constexpr const auto kTxnId = 0;
+  constexpr const int kLeaderIdx = 0;
+  vector<TabletReplica*> replicas = SetUpLeaderGetReplicas(kLeaderIdx);
+  auto* leader_replica = replicas[kLeaderIdx];
+  const auto tablet_id = leader_replica->tablet_id();
+  ASSERT_OK(leader_replica->consensus()->WaitUntilLeaderForTests(kDefaultTimeout));
+
+  // Get commit-related metadata.
+  TxnMetadataPB meta_pb;
+  Status s = txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(kTxnId, ParticipantOpPB::GET_METADATA), kDefaultTimeout,
+      /*begin_commit_timestamp*/nullptr, &meta_pb);
+  ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
+
+  ASSERT_OK(txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(kTxnId, ParticipantOpPB::BEGIN_TXN), kDefaultTimeout));
+  ASSERT_OK(txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(kTxnId, ParticipantOpPB::GET_METADATA), kDefaultTimeout,
+      /*begin_commit_timestamp*/nullptr, &meta_pb));
+  ASSERT_FALSE(meta_pb.has_aborted());
+  ASSERT_FALSE(meta_pb.has_commit_mvcc_op_timestamp());
+  ASSERT_FALSE(meta_pb.has_commit_timestamp());
+
+  ASSERT_OK(txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(kTxnId, ParticipantOpPB::BEGIN_COMMIT), kDefaultTimeout));
+  ASSERT_OK(txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(kTxnId, ParticipantOpPB::GET_METADATA), kDefaultTimeout,
+      /*begin_commit_timestamp*/nullptr, &meta_pb));
+  ASSERT_FALSE(meta_pb.has_aborted());
+  ASSERT_TRUE(meta_pb.has_commit_mvcc_op_timestamp());
+  ASSERT_FALSE(meta_pb.has_commit_timestamp());
+
+  ASSERT_OK(txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(kTxnId, ParticipantOpPB::FINALIZE_COMMIT), kDefaultTimeout));
+  ASSERT_OK(txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(kTxnId, ParticipantOpPB::GET_METADATA), kDefaultTimeout,
+      /*begin_commit_timestamp*/nullptr, &meta_pb));
+  ASSERT_FALSE(meta_pb.has_aborted());
+  ASSERT_TRUE(meta_pb.has_commit_mvcc_op_timestamp());
+  ASSERT_TRUE(meta_pb.has_commit_timestamp());
+
+  // Get abort-related metadata.
+  constexpr const auto kAbortedTxnId = 1;
+  ASSERT_OK(txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(kAbortedTxnId, ParticipantOpPB::BEGIN_TXN), kDefaultTimeout));
+  ASSERT_OK(txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(kAbortedTxnId, ParticipantOpPB::ABORT_TXN), kDefaultTimeout));
+  ASSERT_OK(txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(kAbortedTxnId, ParticipantOpPB::GET_METADATA), kDefaultTimeout,
+      /*begin_commit_timestamp*/nullptr, &meta_pb));
+  ASSERT_TRUE(meta_pb.has_aborted());
+  ASSERT_FALSE(meta_pb.has_commit_mvcc_op_timestamp());
+  ASSERT_FALSE(meta_pb.has_commit_timestamp());
+}
+
+// Test that we can start multiple transactions on the same participant.
+TEST_F(TxnParticipantITest, TestTxnSystemClientBeginTxnDoesntLock) {
+  constexpr const int kLeaderIdx = 0;
+  constexpr const int kFirstTxn = 0;
+  constexpr const int kSecondTxn = 1;
+  vector<TabletReplica*> replicas = SetUpLeaderGetReplicas(kLeaderIdx);
+  auto* leader_replica = replicas[kLeaderIdx];
+  const auto tablet_id = leader_replica->tablet_id();
+  ASSERT_OK(leader_replica->consensus()->WaitUntilLeaderForTests(kDefaultTimeout));
+
+  // Start a transaction and make sure it results in the expected state
+  // server-side.
+  unique_ptr<TxnSystemClient> txn_client;
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->messenger()->sasl_proto_name(),
+                                    &txn_client));
+  ASSERT_OK(txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(kFirstTxn, ParticipantOpPB::BEGIN_TXN), kDefaultTimeout));
+  NO_FATALS(CheckReplicasMatchTxns(replicas, { { kFirstTxn, kOpen, -1 } }));
+
+  // Begin another transaction with a lower txn ID. This is allowed, since
+  // partition locks are only taken once we write.
+  ASSERT_OK(txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(kSecondTxn, ParticipantOpPB::BEGIN_TXN), kDefaultTimeout));
+  NO_FATALS(CheckReplicasMatchTxns(replicas,
+        { { kFirstTxn, kOpen, -1 }, { kSecondTxn, kOpen, -1 } }));
+}
+
 TEST_F(TxnParticipantITest, TestTxnSystemClientCommitSequence) {
   constexpr const int kLeaderIdx = 0;
   constexpr const int kTxnId = 0;
@@ -778,7 +931,9 @@ TEST_F(TxnParticipantITest, TestTxnSystemClientCommitSequence) {
   // Start a transaction and make sure it results in the expected state
   // server-side.
   unique_ptr<TxnSystemClient> txn_client;
-  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_client));
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->messenger()->sasl_proto_name(),
+                                    &txn_client));
   ASSERT_OK(txn_client->ParticipateInTransaction(
       tablet_id, MakeParticipantOp(kTxnId, ParticipantOpPB::BEGIN_TXN), kDefaultTimeout));
   NO_FATALS(CheckReplicasMatchTxns(replicas, { { kTxnId, kOpen, -1 } }));
@@ -816,25 +971,29 @@ TEST_F(TxnParticipantITest, TestTxnSystemClientCommitSequence) {
   ASSERT_EQ(refetched_begin_commit_ts, begin_commit_ts);
   NO_FATALS(CheckReplicasMatchTxns(replicas, { { kTxnId, kCommitInProgress, -1 } }));
 
-  // Once we finish committing, we should be unable to do anything.
+  // Once we finish committing, we should be unable to begin or abort.
   ASSERT_OK(txn_client->ParticipateInTransaction(
       tablet_id, MakeParticipantOp(kTxnId, ParticipantOpPB::FINALIZE_COMMIT, kDummyCommitTimestamp),
       kDefaultTimeout));
   NO_FATALS(CheckReplicasMatchTxns(replicas, {{kTxnId, kCommitted, kDummyCommitTimestamp}}));
-  for (const auto type : { ParticipantOpPB::BEGIN_TXN, ParticipantOpPB::BEGIN_COMMIT,
-                           ParticipantOpPB::ABORT_TXN }) {
+  for (const auto type : { ParticipantOpPB::BEGIN_TXN, ParticipantOpPB::ABORT_TXN }) {
     Status s = txn_client->ParticipateInTransaction(
         tablet_id, MakeParticipantOp(kTxnId, type), kDefaultTimeout);
     ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
   }
   NO_FATALS(CheckReplicasMatchTxns(replicas, {{kTxnId, kCommitted, kDummyCommitTimestamp}}));
-  ASSERT_OK(txn_client->ParticipateInTransaction(
-      tablet_id, MakeParticipantOp(kTxnId, ParticipantOpPB::FINALIZE_COMMIT, kDummyCommitTimestamp),
-      kDefaultTimeout));
+  for (const auto type : { ParticipantOpPB::BEGIN_COMMIT, ParticipantOpPB::FINALIZE_COMMIT }) {
+    ASSERT_OK(txn_client->ParticipateInTransaction(
+        tablet_id, MakeParticipantOp(kTxnId, type, kDummyCommitTimestamp),
+        kDefaultTimeout));
+  }
   NO_FATALS(CheckReplicasMatchTxns(replicas, {{kTxnId, kCommitted, kDummyCommitTimestamp}}));
 }
 
 TEST_F(TxnParticipantITest, TestTxnSystemClientAbortSequence) {
+  // Disable the partition lock as there are concurrent transactions.
+  // TODO(awong): update this when implementing finer grained locking.
+  FLAGS_enable_txn_partition_lock = false;
   constexpr const int kLeaderIdx = 0;
   constexpr const int kTxnOne = 0;
   constexpr const int kTxnTwo = 1;
@@ -843,7 +1002,9 @@ TEST_F(TxnParticipantITest, TestTxnSystemClientAbortSequence) {
   const auto tablet_id = leader_replica->tablet_id();
   ASSERT_OK(leader_replica->consensus()->WaitUntilLeaderForTests(kDefaultTimeout));
   unique_ptr<TxnSystemClient> txn_client;
-  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_client));
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->messenger()->sasl_proto_name(),
+                                    &txn_client));
   ASSERT_OK(txn_client->ParticipateInTransaction(
       tablet_id, MakeParticipantOp(kTxnOne, ParticipantOpPB::BEGIN_TXN),
       kDefaultTimeout));
@@ -889,6 +1050,65 @@ TEST_F(TxnParticipantITest, TestTxnSystemClientAbortSequence) {
       { { kTxnOne, kAborted, -1 }, { kTxnTwo, kAborted, -1 } }));
 }
 
+TEST_F(TxnParticipantITest, TestTxnSystemClientErrorWhenNotBegun) {
+  constexpr const int kLeaderIdx = 0;
+  int txn_id = 0;
+  vector<TabletReplica*> replicas = SetUpLeaderGetReplicas(kLeaderIdx);
+  auto* leader_replica = replicas[kLeaderIdx];
+  const auto tablet_id = leader_replica->tablet_id();
+  ASSERT_OK(leader_replica->consensus()->WaitUntilLeaderForTests(kDefaultTimeout));
+  unique_ptr<TxnSystemClient> txn_client;
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->messenger()->sasl_proto_name(),
+                                    &txn_client));
+
+  for (auto type : { ParticipantOpPB::BEGIN_COMMIT,
+                     ParticipantOpPB::FINALIZE_COMMIT }) {
+    Status s = txn_client->ParticipateInTransaction(
+        tablet_id, MakeParticipantOp(txn_id++, type), kDefaultTimeout);
+    ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
+    NO_FATALS(CheckReplicasMatchTxns(replicas, {}));
+  }
+
+  ASSERT_OK(txn_client->ParticipateInTransaction(
+      tablet_id, MakeParticipantOp(txn_id++, ParticipantOpPB::ABORT_TXN), kDefaultTimeout));
+  NO_FATALS(CheckReplicasMatchTxns(replicas, { { 2, kAborted, -1 } }));
+}
+
+TEST_F(TxnParticipantITest, TestTxnSystemClientRepeatCalls) {
+  constexpr const int kLeaderIdx = 0;
+  constexpr const int kTxnOne = 1;
+  constexpr const int kTxnTwo = 2;
+  vector<TabletReplica*> replicas = SetUpLeaderGetReplicas(kLeaderIdx);
+  auto* leader_replica = replicas[kLeaderIdx];
+  const auto tablet_id = leader_replica->tablet_id();
+  ASSERT_OK(leader_replica->consensus()->WaitUntilLeaderForTests(kDefaultTimeout));
+  unique_ptr<TxnSystemClient> txn_client;
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->messenger()->sasl_proto_name(),
+                                    &txn_client));
+  // Repeat each op twice. There should be no issues here since each op is
+  // idempotent. There should also be no issues with the partition lock.
+  for (const auto& type : kCommitSequence) {
+    ASSERT_OK(txn_client->ParticipateInTransaction(
+        tablet_id, MakeParticipantOp(kTxnOne, type, kDummyCommitTimestamp),
+        kDefaultTimeout));
+    ASSERT_OK(txn_client->ParticipateInTransaction(
+        tablet_id, MakeParticipantOp(kTxnOne, type, kDummyCommitTimestamp),
+        kDefaultTimeout));
+  }
+  for (const auto& type : kAbortSequence) {
+    ASSERT_OK(txn_client->ParticipateInTransaction(
+        tablet_id, MakeParticipantOp(kTxnTwo, type, kDummyCommitTimestamp),
+        kDefaultTimeout));
+    ASSERT_OK(txn_client->ParticipateInTransaction(
+        tablet_id, MakeParticipantOp(kTxnTwo, type, kDummyCommitTimestamp),
+        kDefaultTimeout));
+  }
+  NO_FATALS(CheckReplicasMatchTxns(
+      replicas, { { kTxnOne, kCommitted, kDummyCommitTimestamp }, { kTxnTwo, kAborted, -1 } }));
+}
+
 TEST_F(TxnParticipantITest, TestTxnSystemClientTimeoutWhenNoMajority) {
   constexpr const int kLeaderIdx = 0;
   constexpr const int kTxnId = 0;
@@ -902,7 +1122,9 @@ TEST_F(TxnParticipantITest, TestTxnSystemClientTimeoutWhenNoMajority) {
     cluster_->mini_tablet_server(i)->Shutdown();
   }
   unique_ptr<TxnSystemClient> txn_client;
-  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_client));
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->messenger()->sasl_proto_name(),
+                                    &txn_client));
   Status s = txn_client->ParticipateInTransaction(
       tablet_id, MakeParticipantOp(kTxnId, ParticipantOpPB::BEGIN_TXN),
       MonoDelta::FromSeconds(1));
@@ -958,7 +1180,9 @@ TEST_F(TxnParticipantITest, TestTxnSystemClientSucceedsOnBootstrap) {
   // Start a thread that sends participant ops to the tablet.
   int next_txn_id = 0;
   unique_ptr<TxnSystemClient> txn_client;
-  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_client));
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->messenger()->sasl_proto_name(),
+                                    &txn_client));
   CountDownLatch stop(1);
   Status client_error;
   thread t([&] {
@@ -1010,7 +1234,9 @@ TEST_F(TxnParticipantITest, TestTxnSystemClientRetriesWhenReplicaNotFound) {
   // Start a thread that sends participant ops to the tablet.
   int next_txn_id = 0;
   unique_ptr<TxnSystemClient> txn_client;
-  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_client));
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->messenger()->sasl_proto_name(),
+                                    &txn_client));
   CountDownLatch stop(1);
   Status client_error;
   thread t([&] {
@@ -1225,7 +1451,9 @@ TEST_F(TxnParticipantElectionStormITest, TestTxnSystemClientRetriesThroughStorm)
   }
   const auto kTimeout = MonoDelta::FromSeconds(10);
   unique_ptr<TxnSystemClient> txn_client;
-  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_client));
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->messenger()->sasl_proto_name(),
+                                    &txn_client));
 
   // Start injecting latency to Raft-related traffic to spur elections.
   FLAGS_raft_enable_pre_election = false;
