@@ -17,6 +17,7 @@
 
 #include <algorithm>
 #include <atomic>
+#include <cstddef>
 #include <cstdint>
 #include <functional>
 #include <map>
@@ -39,6 +40,7 @@
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/stl_util.h"
+#include "kudu/gutil/strings/join.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/integration-tests/cluster_itest_util.h"
 #include "kudu/integration-tests/test_workload.h"
@@ -47,9 +49,11 @@
 #include "kudu/master/ts_manager.h"
 #include "kudu/mini-cluster/internal_mini_cluster.h"
 #include "kudu/mini-cluster/mini_cluster.h"
+#include "kudu/rpc/messenger.h"
 #include "kudu/tablet/metadata.pb.h"
 #include "kudu/tablet/tablet_metadata.h"
 #include "kudu/tablet/tablet_replica.h"
+#include "kudu/tools/tool_test_util.h"
 #include "kudu/transactions/transactions.pb.h"
 #include "kudu/transactions/txn_status_tablet.h"
 #include "kudu/transactions/txn_system_client.h"
@@ -57,12 +61,21 @@
 #include "kudu/tserver/tablet_server.h"
 #include "kudu/tserver/ts_tablet_manager.h"
 #include "kudu/util/monotime.h"
+#include "kudu/util/net/net_util.h"
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/status.h"
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
+DECLARE_bool(enable_txn_system_client_init);
+DECLARE_bool(raft_enable_pre_election);
+DECLARE_bool(txn_schedule_background_tasks);
+DECLARE_bool(txn_status_tablet_failover_inject_timeout_error);
+DECLARE_bool(txn_status_tablet_inject_load_failure_error);
+DECLARE_bool(txn_status_tablet_inject_uninitialized_leader_status_error);
 DECLARE_double(leader_failure_max_missed_heartbeat_periods);
+DECLARE_int32(consensus_inject_latency_ms_in_notifications);
+DECLARE_int32(raft_heartbeat_interval_ms);
 DECLARE_string(superuser_acl);
 DECLARE_string(user_acl);
 DECLARE_uint32(txn_keepalive_interval_ms);
@@ -77,10 +90,12 @@ using kudu::cluster::InternalMiniClusterOptions;
 using kudu::itest::TServerDetails;
 using kudu::itest::TabletServerMap;
 using kudu::tablet::TabletReplica;
+using kudu::tools::RunKuduTool;
 using kudu::transactions::TxnStatePB;
 using kudu::transactions::TxnStatusEntryPB;
 using kudu::transactions::TxnStatusTablet;
 using kudu::transactions::TxnSystemClient;
+using std::atomic;
 using std::map;
 using std::string;
 using std::thread;
@@ -103,11 +118,17 @@ class TxnStatusTableITest : public KuduTest {
 
   void SetUp() override {
     KuduTest::SetUp();
+    // Several of these tests rely on checking transaction state, which is
+    // easier to work with without committing in the background.
+    FLAGS_txn_schedule_background_tasks = false;
+    FLAGS_enable_txn_system_client_init = true;
     cluster_.reset(new InternalMiniCluster(env_, {}));
     ASSERT_OK(cluster_->Start());
 
     // Create the txn system client with which to communicate with the cluster.
-    ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_sys_client_));
+    ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                      cluster_->messenger()->sasl_proto_name(),
+                                      &txn_sys_client_));
   }
 
   // Ensures that all replicas have the right table type set.
@@ -195,6 +216,54 @@ TEST_F(TxnStatusTableITest, TestTxnStatusTableNotListed) {
   ASSERT_NE(nullptr, table);
 }
 
+// Test that the transaction status table is visible in a separate section in
+// ksck, and that its health is reported as other tables are.
+TEST_F(TxnStatusTableITest, TestTxnStatusTableInKsck) {
+  vector<string> master_addrs;
+  for (const auto& hp : cluster_->master_rpc_addrs()) {
+    master_addrs.emplace_back(hp.ToString());
+  }
+  string out;
+  string err;
+  vector<string> ksck_args = { "cluster", "ksck", JoinStrings(master_addrs, ",") };
+  ASSERT_OK(RunKuduTool(ksck_args, &out, &err));
+  ASSERT_STR_CONTAINS(out, "The cluster doesn't have any matching system tables");
+
+  // Nothing should be logged on error for finding the txn status table.
+  ASSERT_STR_NOT_CONTAINS(err, TxnStatusTablet::kTxnStatusTableName) << err;
+  ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(100));
+
+  ASSERT_OK(RunKuduTool(ksck_args, &out));
+  ASSERT_STR_MATCHES(out,
+      "^Summary by system table.*\n"
+      "^             Name              | RF | Status  | Total Tablets | Healthy .*\n"
+      "^-------------------------------+----+---------+---------------+---------.*\n"
+      "^ kudu_system.kudu_transactions | 1  | HEALTHY | 1             | 1       .*\n");
+
+  // Now bring down a tablet server and we should see the health update.
+  cluster_->mini_tablet_server(0)->Shutdown();
+  Status s = RunKuduTool(ksck_args, &out);
+  ASSERT_TRUE(s.IsRuntimeError()) << s.ToString();
+  ASSERT_STR_MATCHES(out,
+      "^Summary by system table.*\n"
+      "^             Name              | RF | Status      | Total Tablets | Healthy .*\n"
+      "^-------------------------------+----+-------------+---------------+---------.*\n"
+      "^ kudu_system.kudu_transactions | 1  | UNAVAILABLE | 1             | 0       .*\n");
+}
+
+// Test that despite being unable to list the transaction status table, we are
+// able to run the tool. In previous iterations of the rebalancer, the tool may
+// have crashed attempting to find the RF for a table it didn't know about.
+TEST_F(TxnStatusTableITest, TestTxnStatusTableInRebalancer) {
+  vector<string> master_addrs;
+  for (const auto& hp : cluster_->master_rpc_addrs()) {
+    master_addrs.emplace_back(hp.ToString());
+  }
+  ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(100));
+  vector<string> rebalancer_args = { "cluster", "rebalance", JoinStrings(master_addrs, ",") };
+  ASSERT_OK(RunKuduTool(rebalancer_args));
+}
+
 // Test that only the service- or super-user can create or alter the
 // transaction status table.
 TEST_F(TxnStatusTableITest, TestProtectCreateAndAlter) {
@@ -249,10 +318,10 @@ TEST_F(TxnStatusTableITest, TestProtectAccess) {
     Status _s = (s); \
     results.emplace_back(_s.CloneAndPrepend(msg)); \
   } while (0)
-  const auto attempt_accesses_and_delete = [&] (KuduClient* client) {
-    // Go through various table access methods on the transaction status table.
-    // Collect the results so we can verify whether we were authorized
-    // properly.
+  const auto attempt_accesses = [&] (KuduClient* client) {
+    // Go through various table access methods on the transaction status table,
+    // except for dropping/deleting it. Collect the results so we can verify
+    // whether we were authorized properly.
     vector<Status> results;
     bool unused;
     STORE_AND_PREPEND(client->TableExists(kTableName, &unused), "failed to check existence");
@@ -261,6 +330,13 @@ TEST_F(TxnStatusTableITest, TestProtectAccess) {
     client::KuduTableStatistics* unused_stats = nullptr;
     STORE_AND_PREPEND(client->GetTableStatistics(kTableName, &unused_stats), "failed to get stats");
     unique_ptr<client::KuduTableStatistics> unused_unique_stats(unused_stats);
+    return results;
+  };
+  const auto attempt_accesses_and_delete = [&] (KuduClient* client) {
+    // Go through various table access methods on the transaction status table,
+    // including dropping the table. Collect the results so we can verify
+    // whether we were authorized properly.
+    vector<Status> results = attempt_accesses(client);
     STORE_AND_PREPEND(client->DeleteTable(kTableName), "failed to delete table");
     return results;
   };
@@ -275,13 +351,21 @@ TEST_F(TxnStatusTableITest, TestProtectAccess) {
   // Even though we're the service user, we shouldn't be able to access
   // anything because most RPCs require us to be a super-user or user.
   NO_FATALS(SetSuperuserAndUser("nobody", "nobody"));
-  results = attempt_accesses_and_delete(client_sp.get());
+  results = attempt_accesses(client_sp.get());
   for (const auto& r : results) {
     // NOTE: we don't check the type because authorization errors may surface
     // as RemoteErrors or NotAuthorized errors.
-    ASSERT_FALSE(r.ok());
-    ASSERT_STR_CONTAINS(r.ToString(), "Not authorized");
+    ASSERT_TRUE(r.ok()) << r.ToString();
   }
+
+  {
+    const auto s = client_sp->DeleteTable(kTableName);
+    const auto errmsg = s.ToString();
+    ASSERT_TRUE(s.IsRemoteError()) << errmsg;
+    ASSERT_STR_CONTAINS(
+        errmsg, "Not authorized: unauthorized access to method: DeleteTable");
+  }
+
   // As a sanity check, our last delete have failed, and we should be unable to
   // create another transaction status table.
   Status s = txn_sys_client_->CreateTxnStatusTable(100);
@@ -386,6 +470,44 @@ TEST_F(TxnStatusTableITest, TestSystemClientFindTablets) {
   ASSERT_OK(txn_sys_client_->AbortTransaction(100, kUser));
 }
 
+TEST_F(TxnStatusTableITest, TestSystemClientMasterDown) {
+  ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(100));
+  ASSERT_OK(txn_sys_client_->OpenTxnStatusTable());
+
+  cluster_->mini_master(0)->Shutdown();
+
+  // When the only server is down, the system client should keep trying until
+  // it times out.
+  {
+    int64_t highest_seen_txn_id = -1;
+    auto s = txn_sys_client_->BeginTransaction(
+        1, kUser, nullptr /* txn_keepalive_ms */, &highest_seen_txn_id,
+        MonoDelta::FromMilliseconds(100));
+    ASSERT_TRUE(s.IsTimedOut()) << s.ToString();
+    // The 'highest_seen_txn_id' should be left untouched.
+    ASSERT_EQ(-1, highest_seen_txn_id);
+  }
+
+  // Now try with a longer timeout and ensure that if the server comes back up,
+  // the system client will succeed.
+  thread t([&] {
+    // Wait a bit to give some time for the system client to make its request
+    // and retry some.
+    SleepFor(MonoDelta::FromMilliseconds(500));
+    CHECK_OK(cluster_->mini_master(0)->Restart());
+  });
+  SCOPED_CLEANUP({
+    t.join();
+  });
+
+  int64_t highest_seen_txn_id = -1;
+  ASSERT_OK(txn_sys_client_->BeginTransaction(
+      1, kUser, nullptr /* txn_keepalive_ms */, &highest_seen_txn_id,
+      MonoDelta::FromSeconds(10)));
+  // Make sure the highest txn ID we've seen matches the one we just started.
+  ASSERT_EQ(1, highest_seen_txn_id);
+}
+
 TEST_F(TxnStatusTableITest, TestSystemClientTServerDown) {
   ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(100));
   ASSERT_OK(txn_sys_client_->OpenTxnStatusTable());
@@ -410,7 +532,7 @@ TEST_F(TxnStatusTableITest, TestSystemClientTServerDown) {
     // Wait a bit to give some time for the system client to make its request
     // and retry some.
     SleepFor(MonoDelta::FromMilliseconds(500));
-    ASSERT_OK(cluster_->mini_tablet_server(0)->Restart());
+    CHECK_OK(cluster_->mini_tablet_server(0)->Restart());
   });
   SCOPED_CLEANUP({
     t.join();
@@ -419,8 +541,9 @@ TEST_F(TxnStatusTableITest, TestSystemClientTServerDown) {
   int64_t highest_seen_txn_id = -1;
   ASSERT_OK(txn_sys_client_->BeginTransaction(
       1, kUser, nullptr /* txn_keepalive_ms */, &highest_seen_txn_id,
-      MonoDelta::FromSeconds(3)));
-  ASSERT_EQ(highest_seen_txn_id, 1);
+      MonoDelta::FromSeconds(10)));
+  // Make sure the highest txn ID we've seen matches the one we just started.
+  ASSERT_EQ(1, highest_seen_txn_id);
 }
 
 TEST_F(TxnStatusTableITest, TestSystemClientBeginTransactionErrors) {
@@ -458,7 +581,7 @@ TEST_F(TxnStatusTableITest, TestSystemClientRegisterParticipantErrors) {
   ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(100));
   ASSERT_OK(txn_sys_client_->OpenTxnStatusTable());
   Status s = txn_sys_client_->RegisterParticipant(1, "participant", kUser);
-  ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+  ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
   ASSERT_STR_MATCHES(s.ToString(), "transaction ID.*not found, current highest txn ID:.*");
 
   ASSERT_OK(txn_sys_client_->BeginTransaction(1, kUser));
@@ -500,14 +623,14 @@ TEST_F(TxnStatusTableITest, SystemClientCommitAndAbortTransaction) {
   // an error.
   {
     auto s = txn_sys_client_->BeginCommitTransaction(2, kUser);
-    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 2 not found");
   }
 
   // An attempt to abort a non-existent transaction should report an error.
   {
     auto s = txn_sys_client_->AbortTransaction(2, kUser);
-    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 2 not found");
   }
 
@@ -532,6 +655,38 @@ TEST_F(TxnStatusTableITest, SystemClientCommitAndAbortTransaction) {
   }
 }
 
+TEST_F(TxnStatusTableITest, TServerInitializesTxnSystemClient) {
+  auto* mts = cluster_->mini_tablet_server(0);
+  auto* client_initializer = mts->server()->txn_client_initializer();
+  TxnSystemClient* txn_client = nullptr;
+  ASSERT_EVENTUALLY([&] {
+    ASSERT_OK(client_initializer->GetClient(&txn_client));
+  });
+  // We should be able to get the client, and use it to make a call.
+  ASSERT_NE(txn_client, nullptr);
+  ASSERT_OK(txn_client->CreateTxnStatusTable(100));
+  txn_client = nullptr;
+
+  // Try shutting down the master, and then restarting the tablet server to
+  // attempt to rebuild a TxnSystemClient. This should fail until the master
+  // comes back up.
+  cluster_->mini_master()->Shutdown();
+  mts->Shutdown();
+  ASSERT_OK(mts->Restart());
+  client_initializer = mts->server()->txn_client_initializer();
+  Status s = client_initializer->GetClient(&txn_client);
+  ASSERT_TRUE(s.IsServiceUnavailable()) << s.ToString();
+  ASSERT_EQ(txn_client, nullptr);
+
+  ASSERT_OK(cluster_->mini_master()->Restart());
+  ASSERT_EVENTUALLY([&] {
+    ASSERT_OK(client_initializer->GetClient(&txn_client));
+  });
+  // We should be able to get the client, and use it to make a call.
+  ASSERT_NE(txn_client, nullptr);
+  ASSERT_OK(txn_client->OpenTxnStatusTable());
+}
+
 TEST_F(TxnStatusTableITest, GetTransactionStatus) {
   const auto verify_state = [&](TxnStatePB state) {
     TxnStatusEntryPB txn_status;
@@ -552,12 +707,12 @@ TEST_F(TxnStatusTableITest, GetTransactionStatus) {
   NO_FATALS(verify_state(TxnStatePB::COMMIT_IN_PROGRESS));
 
   ASSERT_OK(txn_sys_client_->AbortTransaction(1, kUser));
-  NO_FATALS(verify_state(TxnStatePB::ABORTED));
+  NO_FATALS(verify_state(TxnStatePB::ABORT_IN_PROGRESS));
 
   {
     auto s = txn_sys_client_->BeginCommitTransaction(1, kUser);
     ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
-    NO_FATALS(verify_state(TxnStatePB::ABORTED));
+    NO_FATALS(verify_state(TxnStatePB::ABORT_IN_PROGRESS));
   }
 
   // In the negative scenarios below, check for the expected status code
@@ -573,7 +728,7 @@ TEST_F(TxnStatusTableITest, GetTransactionStatus) {
   {
     TxnStatusEntryPB empty;
     auto s = txn_sys_client_->GetTransactionStatus(2, kUser, &empty);
-    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
     ASSERT_FALSE(empty.has_user());
     ASSERT_FALSE(empty.has_state());
   }
@@ -581,7 +736,7 @@ TEST_F(TxnStatusTableITest, GetTransactionStatus) {
   {
     TxnStatusEntryPB empty;
     auto s = txn_sys_client_->GetTransactionStatus(2, "", &empty);
-    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
     ASSERT_FALSE(empty.has_user());
     ASSERT_FALSE(empty.has_state());
   }
@@ -618,15 +773,106 @@ TEST_F(TxnStatusTableITest, TestSystemClientConcurrentCalls) {
   }
 }
 
+// Check the operation of the CheckOpenTxnStatusTable() method; make sure
+// it's possible to register txn participants even with stale information
+// on the range parititions.
+TEST_F(TxnStatusTableITest, CheckOpenTxnStatusTable) {
+  static constexpr auto kPartitionWidth = 1000;
+  static constexpr const char* const kUser = "CheckOpen";
+
+  {
+    // At this point, the transaction status table doesn't exist yet.
+    auto s = txn_sys_client_->CheckOpenTxnStatusTable();
+    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+  }
+
+  ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(kPartitionWidth));
+  ASSERT_OK(txn_sys_client_->CheckOpenTxnStatusTable());
+  const auto* sys_table_ptr_ref = txn_sys_client_->txn_status_table().get();
+
+  cluster_->mini_master()->Shutdown();
+  // Now, once CheckOpenTxnStatusTable() succeeded, there should be no RPC calls
+  // to the cluster upon subsequent invocations, and CheckOpenTxnStatusTable()
+  // should return Status::OK(), keeping the original shared pointer.
+  ASSERT_OK(txn_sys_client_->CheckOpenTxnStatusTable());
+  const auto* sys_table_ptr = txn_sys_client_->txn_status_table().get();
+  ASSERT_EQ(sys_table_ptr_ref, sys_table_ptr);
+
+  ASSERT_OK(cluster_->mini_master()->Restart());
+  ASSERT_OK(txn_sys_client_->BeginTransaction(0, kUser));
+
+  static constexpr auto kNewTxnId = kPartitionWidth + 1;
+  {
+    // Behind the scenes, create tablets for the next transaction IDs range
+    // and start a new transaction.
+    unique_ptr<TxnSystemClient> tsc;
+    ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                      cluster_->messenger()->sasl_proto_name(),
+                                      &tsc));
+    // Re-open the system table.
+    ASSERT_OK(tsc->OpenTxnStatusTable());
+    ASSERT_OK(tsc->AddTxnStatusTableRange(kPartitionWidth, 2 * kPartitionWidth));
+    ASSERT_OK(tsc->BeginTransaction(kNewTxnId, kUser));
+  }
+
+  // It would be great if it were possible to use the same client which was not
+  // aware of a new range in the txn status table to register a participant
+  // in a transaction with ID in the new range. This assumes the client would
+  // automatically re-fetch metadata from masters upon getting a request
+  // for a non-covered range. However, this isn't implemented yet.
+  //
+  // TODO(aserbin): change this to expected Status::OK() after implementing that
+  auto s = txn_sys_client_->RegisterParticipant(
+      kNewTxnId, "txn_participant", kUser, MonoDelta::FromSeconds(10));
+  ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+  ASSERT_STR_CONTAINS(s.ToString(),
+                      "No tablet covering the requested range partition");
+}
+
+TEST_F(TxnStatusTableITest, CheckOpenTxnStatusTableConcurrent) {
+  static constexpr auto kNumThreads = 8;
+  static constexpr const char* const kUser = "CheckOpenConcurrent";
+  static constexpr const char* const kParticipant = "txn_participant";
+
+  ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(2 * kNumThreads));
+  atomic<size_t> success_count_begin = 0;
+  atomic<size_t> success_count_register = 0;
+  vector<thread> threads;
+  threads.reserve(kNumThreads);
+  for (int idx = 0; idx < kNumThreads; ++idx) {
+    threads.emplace_back([&] {
+      CHECK_OK(txn_sys_client_->CheckOpenTxnStatusTable());
+      // The call below is of "best effort" approach: the essence here is just
+      // to make concurrent requests to the txn system client once
+      // CheckOpenTxnStatusTable() has been called.
+      auto s = txn_sys_client_->BeginTransaction(0, kUser);
+      if (s.ok()) {
+        ++success_count_begin;
+      }
+      s = txn_sys_client_->RegisterParticipant(
+          0, kParticipant, kUser, MonoDelta::FromSeconds(10));
+      if (s.ok()) {
+        ++success_count_register;
+      }
+    });
+  }
+  std::for_each(threads.begin(), threads.end(), [&] (thread& t) { t.join(); });
+  ASSERT_EQ(1, success_count_begin);
+  ASSERT_LE(1, success_count_register);
+}
+
 class MultiServerTxnStatusTableITest : public TxnStatusTableITest {
  public:
   void SetUp() override {
     KuduTest::SetUp();
+    FLAGS_enable_txn_system_client_init = true;
     InternalMiniClusterOptions opts;
     opts.num_tablet_servers = 4;
     cluster_.reset(new InternalMiniCluster(env_, std::move(opts)));
     ASSERT_OK(cluster_->Start());
-    ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_sys_client_));
+    ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                      cluster_->messenger()->sasl_proto_name(),
+                                      &txn_sys_client_));
 
     // Create the initial transaction status table partitions and start an
     // initial transaction.
@@ -708,14 +954,143 @@ TEST_F(MultiServerTxnStatusTableITest, TestSystemClientCrashedNodes) {
   ASSERT_FALSE(leader_uuid.empty());
   FLAGS_leader_failure_max_missed_heartbeat_periods = 1;
   cluster_->mini_tablet_server_by_uuid(leader_uuid)->Shutdown();
-  // We have to wait for a leader to be elected. Until that happens, the system
-  // client may try to start transactions on followers, and in doing so use up
-  // transaction IDs. Have the system client try again with a higher
-  // transaction ID until a leader is elected.
-  int txn_id = 2;
+  ASSERT_OK(txn_sys_client_->BeginTransaction(2, kUser));
+}
+
+enum InjectedErrorType {
+  kFailoverTimeoutError,
+  kLeaderStatusError,
+  kLoadFromTabletError
+};
+class TxnStatusTableRetryITest : public MultiServerTxnStatusTableITest,
+                                 public ::testing::WithParamInterface<InjectedErrorType> {
+};
+
+TEST_P(TxnStatusTableRetryITest, TestRetryOnError) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+
+  auto error_type = GetParam();
+  switch (error_type) {
+    case kFailoverTimeoutError:
+      FLAGS_txn_status_tablet_failover_inject_timeout_error = true;
+      break;
+    case kLeaderStatusError:
+      FLAGS_txn_status_tablet_inject_uninitialized_leader_status_error = true;
+      break;
+    case kLoadFromTabletError:
+      FLAGS_txn_status_tablet_inject_load_failure_error = true;
+      break;
+  }
+  // Find the leader and force it to step down. The system client should be
+  // able to find the new leader.
+  const string& tablet_id = GetTabletId();
+  ASSERT_FALSE(tablet_id.empty());
+  string orig_leader_uuid;
+  ASSERT_OK(FindLeaderId(tablet_id, &orig_leader_uuid));
+  ASSERT_FALSE(orig_leader_uuid.empty());
+  cluster_->mini_tablet_server_by_uuid(
+      orig_leader_uuid)->server()->mutable_quiescing()->store(true);
+
+  string new_leader_uuid;
+  TxnStatusEntryPB txn_status;
   ASSERT_EVENTUALLY([&] {
-    ASSERT_OK(txn_sys_client_->BeginTransaction(++txn_id, kUser));
+    ASSERT_OK(FindLeaderId(tablet_id, &new_leader_uuid));
+    ASSERT_NE(new_leader_uuid, orig_leader_uuid);
+    Status s = txn_sys_client_->GetTransactionStatus(1, kUser, &txn_status);
+    ASSERT_TRUE(s.IsTimedOut()) << s.ToString();
   });
+  orig_leader_uuid = new_leader_uuid;
+  // After disabling fault injection flag, calls should succeed given
+  // the failed ones will be retried.
+  switch (error_type) {
+    case kFailoverTimeoutError:
+      FLAGS_txn_status_tablet_failover_inject_timeout_error = false;
+      break;
+    case kLeaderStatusError:
+      FLAGS_txn_status_tablet_inject_uninitialized_leader_status_error = false;
+      break;
+    case kLoadFromTabletError:
+      FLAGS_txn_status_tablet_inject_load_failure_error = false;
+      break;
+  }
+  cluster_->mini_tablet_server_by_uuid(
+      orig_leader_uuid)->server()->mutable_quiescing()->store(true);
+  ASSERT_EVENTUALLY([&] {
+    ASSERT_OK(FindLeaderId(tablet_id, &new_leader_uuid));
+    ASSERT_NE(new_leader_uuid, orig_leader_uuid);
+    ASSERT_OK(txn_sys_client_->GetTransactionStatus(1, kUser, &txn_status));
+  });
+}
+
+// Test that calls to transaction status tablets will retry when:
+//   1) timeout on waiting the replica to catch up with all replicated
+//      operations in previous term.
+//   2) leader status is not initialized yet.
+INSTANTIATE_TEST_SUITE_P(TestTxnStatusTableRetryOnError,
+                         TxnStatusTableRetryITest,
+                         ::testing::Values(kFailoverTimeoutError,
+                                           kLeaderStatusError,
+                                           kLoadFromTabletError));
+
+class TxnStatusTableElectionStormITest : public TxnStatusTableITest {
+ public:
+  void SetUp() override {
+    KuduTest::SetUp();
+    FLAGS_enable_txn_system_client_init = true;
+
+    // Make leader elections more frequent to get through this test a bit more
+    // quickly.
+    FLAGS_leader_failure_max_missed_heartbeat_periods = 1;
+    FLAGS_raft_heartbeat_interval_ms = 30;
+    InternalMiniClusterOptions opts;
+    opts.num_tablet_servers = 3;
+    cluster_.reset(new InternalMiniCluster(env_, std::move(opts)));
+    ASSERT_OK(cluster_->Start());
+    ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                      cluster_->messenger()->sasl_proto_name(),
+                                      &txn_sys_client_));
+
+    // Create the initial transaction status table partitions.
+    ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(100, 3));
+    ASSERT_OK(txn_sys_client_->OpenTxnStatusTable());
+
+   // Inject latency so elections become more frequent and wait a bit for our
+   // latency injection to kick in.
+   FLAGS_raft_enable_pre_election = false;
+   FLAGS_consensus_inject_latency_ms_in_notifications = 1.5 * FLAGS_raft_heartbeat_interval_ms;
+   SleepFor(MonoDelta::FromMilliseconds(FLAGS_raft_heartbeat_interval_ms * 2));
+  }
+};
+
+TEST_F(TxnStatusTableElectionStormITest, TestFrequentElections) {
+  // Ensure concurrent transaction read and write work as expected under frequent
+  // leader elections.
+  const int kNumTxnsPerThread = 5;
+  const int kNumThreads = 5;
+  vector<thread> threads;
+  for (int t = 0; t < kNumThreads; t++) {
+    threads.emplace_back([&, t] {
+      for (int i = 0; i < kNumTxnsPerThread; i++) {
+        int txn_id = t * kNumTxnsPerThread + i;
+        TxnStatusEntryPB txn_status;
+        Status s = txn_sys_client_->BeginTransaction(txn_id, kUser);
+        // As we don't have guarantee of the threads' execution order, transactions to
+        // be created later can have lower txn ID than previous created ones, which is
+        // not allowed.
+        if (s.ok()) {
+          // Make sure a re-election happens before the following read.
+          SleepFor(MonoDelta::FromMilliseconds(3 * FLAGS_raft_heartbeat_interval_ms));
+
+          CHECK_OK(txn_sys_client_->GetTransactionStatus(txn_id, kUser, &txn_status));
+          CHECK(txn_status.has_user());
+          CHECK_STREQ(kUser, txn_status.user().c_str());
+          CHECK(txn_status.has_state());
+          CHECK_EQ(TxnStatePB::OPEN, txn_status.state());
+        }
+      }
+    });
+  }
+  std::for_each(threads.begin(), threads.end(), [&] (thread& t) { t.join(); });
 }
 
 } // namespace itest
