@@ -255,6 +255,11 @@ struct PersistentTableInfo {
     return pb.owner();
   }
 
+  // Return the table's comment.
+  const std::string& comment() const {
+    return pb.comment();
+  }
+
   // Helper to set the state of the tablet with a custom message.
   void set_state(SysTablesEntryPB::State state, const std::string& msg);
 
@@ -270,6 +275,14 @@ struct PersistentTableInfo {
 // spin-lock.
 class TableInfo : public RefCountedThreadSafe<TableInfo> {
  public:
+  enum {
+    // It is useful to place size limits on tables that may be ingested to
+    // by long-lived applications. To avoid breaking existing tables, there
+    // is no limit enforced.
+    // The default value -1 means no limit.
+    TABLE_WRITE_DEFAULT_LIMIT = -1L
+  };
+
   typedef PersistentTableInfo cow_state;
   typedef std::map<std::string, scoped_refptr<TabletInfo>> TabletInfoMap;
 
@@ -619,6 +632,7 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
                         const std::string& table_name,
                         const boost::optional<std::string>& new_table_name,
                         const boost::optional<std::string>& new_table_owner,
+                        const boost::optional<std::string>& new_table_comment,
                         int64_t notification_log_event_id) WARN_UNUSED_RESULT;
 
   // Get the information about an in-progress alter operation. If 'user' is
@@ -808,7 +822,8 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   static std::string NormalizeTableName(const std::string& table_name);
 
   enum ChangeConfigOp {
-    kAddMaster
+    kAddMaster,
+    kRemoveMaster
   };
 
   // Add/remove a master specified by 'hp' and 'uuid' by initiating change config request.
@@ -836,6 +851,11 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   typedef std::unordered_map<std::string, scoped_refptr<TableInfo>> TableInfoMap;
   typedef std::unordered_map<std::string, scoped_refptr<TabletInfo>> TabletInfoMap;
 
+  // Check whether the table's write limit is reached,
+  // if true, the write permission should be disabled.
+  static bool IsTableWriteDisabled(const scoped_refptr<TableInfo>& table,
+                                   const std::string& table_name);
+
   // Delete the specified table in the catalog. If 'user' is provided,
   // checks that the user is authorized to delete the table. Otherwise,
   // it indicates its an internal operation (originates from catalog
