@@ -43,6 +43,7 @@
 #include "kudu/common/common.pb.h"
 #include "kudu/common/partial_row.h"
 #include "kudu/common/row_operations.h"
+#include "kudu/common/row_operations.pb.h"
 #include "kudu/common/schema.h"
 #include "kudu/common/types.h"
 #include "kudu/common/wire_protocol.h"
@@ -118,12 +119,14 @@ using strings::Substitute;
 
 DECLARE_bool(catalog_manager_check_ts_count_for_create_table);
 DECLARE_bool(enable_per_range_hash_schemas);
+DECLARE_bool(master_client_location_assignment_enabled);
 DECLARE_bool(master_support_authz_tokens);
 DECLARE_bool(mock_table_metrics_for_testing);
 DECLARE_bool(raft_prepare_replacement_before_eviction);
 DECLARE_double(sys_catalog_fail_during_write);
 DECLARE_int32(diagnostics_log_stack_traces_interval_ms);
 DECLARE_int32(master_inject_latency_on_tablet_lookups_ms);
+DECLARE_int32(max_table_comment_length);
 DECLARE_int32(rpc_service_queue_length);
 DECLARE_int64(live_row_count_for_testing);
 DECLARE_int64(on_disk_size_for_testing);
@@ -179,6 +182,7 @@ class MasterTest : public KuduTest {
                      const vector<KuduPartialRow>& split_rows,
                      const vector<pair<KuduPartialRow, KuduPartialRow>>& bounds,
                      const optional<string>& owner,
+                     const optional<string>& comment = boost::none,
                      const optional<TableTypePB>& table_type = boost::none,
                      const vector<vector<HashBucketSchema>>& range_hash_schema = {});
 
@@ -546,7 +550,7 @@ Status MasterTest::CreateTable(const string& table_name,
   RETURN_NOT_OK(split2.SetInt32("key", 20));
 
   return CreateTable(
-      table_name, schema, { split1, split2 }, {}, boost::none, table_type);
+      table_name, schema, { split1, split2 }, {}, boost::none, boost::none, table_type);
 }
 
 Status MasterTest::CreateTable(const string& table_name,
@@ -554,6 +558,7 @@ Status MasterTest::CreateTable(const string& table_name,
                                const vector<KuduPartialRow>& split_rows,
                                const vector<pair<KuduPartialRow, KuduPartialRow>>& bounds,
                                const optional<string>& owner,
+                               const optional<string>& comment,
                                const optional<TableTypePB>& table_type,
                                const vector<vector<HashBucketSchema>>& range_hash_schema) {
   CreateTableRequestPB req;
@@ -589,6 +594,9 @@ Status MasterTest::CreateTable(const string& table_name,
   if (owner) {
     req.set_owner(*owner);
   }
+  if (comment) {
+    req.set_comment(*comment);
+  }
   if (!bounds.empty()) {
     controller.RequireServerFeature(MasterFeatures::RANGE_PARTITION_BOUNDS);
   }
@@ -837,7 +845,7 @@ TEST_F(MasterTest, TestCreateTableCheckRangeInvariants) {
     ASSERT_OK(a_upper.SetInt32("key", 100));
     vector<vector<HashBucketSchema>> range_hash_schema = { vector<HashBucketSchema>() };
     Status s = CreateTable(kTableName, kTableSchema, { split1 }, { { a_lower, a_upper } },
-                           none, none, range_hash_schema);
+                           none, none, none, range_hash_schema);
     ASSERT_TRUE(s.IsInvalidArgument());
     ASSERT_STR_CONTAINS(s.ToString(),
                         "Both 'split_rows' and 'range_hash_schemas' cannot be "
@@ -856,7 +864,7 @@ TEST_F(MasterTest, TestCreateTableCheckRangeInvariants) {
     vector<vector<HashBucketSchema>> range_hash_schema = {std::move(hash_schemas_2),
                                                           std::move(hash_schemas_4)};
     Status s = CreateTable(kTableName, kTableSchema, { }, { { a_lower, a_upper } },
-                           none, none, range_hash_schema);
+                           none, none, none, range_hash_schema);
     ASSERT_TRUE(s.IsInvalidArgument());
     ASSERT_STR_CONTAINS(s.ToString(),
                         "The number of range bounds does not match the number of per "
@@ -995,6 +1003,15 @@ TEST_F(MasterTest, TestCreateTableOwnerNameTooLong) {
   ASSERT_STR_CONTAINS(s.ToString(), "invalid owner name");
 }
 
+TEST_F(MasterTest, TestCreateTableCommentTooLong) {
+  constexpr const char* const kTableName = "testb";
+  const string kComment = string(FLAGS_max_table_comment_length + 1, 'x');
+  const Schema kTableSchema({ ColumnSchema("key", INT32), ColumnSchema("val", INT32) }, 1);
+  Status s = CreateTable(kTableName, kTableSchema, {}, {}, none, kComment);
+  ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
+  ASSERT_STR_CONTAINS(s.ToString(), "invalid table comment");
+}
+
 // Regression test for KUDU-253/KUDU-592: crash if the schema passed to CreateTable
 // is invalid.
 TEST_F(MasterTest, TestCreateTableInvalidSchema) {
@@ -1430,8 +1447,8 @@ TEST_P(ConcurrentGetTableSchemaTest, DirectMethodCall) {
       total / kRunInterval.ToSeconds(), supports_authz_ ? "enabled" : "disabled");
 }
 
-INSTANTIATE_TEST_CASE_P(SupportsAuthzTokens,
-                        ConcurrentGetTableSchemaTest, ::testing::Bool());
+INSTANTIATE_TEST_SUITE_P(SupportsAuthzTokens,
+                         ConcurrentGetTableSchemaTest, ::testing::Bool());
 
 // Verifies that on-disk master metadata is self-consistent and matches a set
 // of expected contents.
@@ -2054,6 +2071,7 @@ TEST_F(MasterTest, TestConnectToMasterAndAssignLocation) {
                                                    "testdata/first_argument.sh");
   const string location = "/foo";
   FLAGS_location_mapping_cmd = Substitute("$0 $1", kLocationCmdPath, location);
+  FLAGS_master_client_location_assignment_enabled = true;
   {
     // Restarting the master to take into account the new setting for the
     // --location_mapping_cmd flag.
@@ -2422,7 +2440,7 @@ TEST_P(AuthzTokenMasterTest, TestGenerateAuthzTokens) {
   }
 }
 
-INSTANTIATE_TEST_CASE_P(SupportsAuthzTokens, AuthzTokenMasterTest, ::testing::Bool());
+INSTANTIATE_TEST_SUITE_P(SupportsAuthzTokens, AuthzTokenMasterTest, ::testing::Bool());
 
 } // namespace master
 } // namespace kudu
