@@ -261,6 +261,7 @@ void MasterPathHandlers::HandleCatalogManager(const Webserver::WebRequest& req,
     table_json["name"] = EscapeForHtmlToString(l.data().name());
     table_json["id"] = EscapeForHtmlToString(table->id());
     table_json["owner"] = EscapeForHtmlToString(l.data().owner());
+    table_json["comment"] = EscapeForHtmlToString(l.data().comment());
     table_json["state"] = state;
     table_json["message"] = EscapeForHtmlToString(l.data().pb.state_msg());
     table_json["tablet_count"] = HumanReadableInt::ToString(table->num_tablets());
@@ -357,6 +358,7 @@ void MasterPathHandlers::HandleTablePage(const Webserver::WebRequest& req,
     TableMetadataLock l(table.get(), LockMode::READ);
     (*output)["name"] = l.data().name();
     (*output)["owner"] = l.data().owner();
+    (*output)["comment"] = l.data().comment();
 
     // Not all Kudu tablenames are also valid Impala identifiers. We need to
     // replace such names with a placeholder when they are used as Impala
