@@ -39,12 +39,12 @@
 #include "kudu/common/partial_row.h"
 #include "kudu/common/partition.h"
 #include "kudu/common/row_operations.h"
+#include "kudu/common/row_operations.pb.h"
 #include "kudu/common/rowblock.h"
 #include "kudu/common/rowblock_memory.h"
 #include "kudu/common/scan_spec.h"
 #include "kudu/common/schema.h"
 #include "kudu/common/wire_protocol.h"
-#include "kudu/common/wire_protocol.pb.h"
 #include "kudu/consensus/consensus_meta.h"
 #include "kudu/consensus/consensus_meta_manager.h"
 #include "kudu/consensus/consensus_peers.h"
@@ -88,20 +88,12 @@ DEFINE_double(sys_catalog_fail_during_write, 0.0,
               "Fraction of the time when system table writes will fail");
 TAG_FLAG(sys_catalog_fail_during_write, hidden);
 
-// Following flags related to dynamic multi-master are hidden till the feature is ready to
-// be advertised.
+// This flag is meant to be used by the tool orchestrating addition of a Kudu master and not
+// for external documentation and hence it's hidden.
 DEFINE_string(master_address_add_new_master, "",
               "Address of master to add as a NON_VOTER on creating a distributed master config.");
-TAG_FLAG(master_address_add_new_master, unsafe);
 TAG_FLAG(master_address_add_new_master, hidden);
 
-DEFINE_bool(master_consensus_allow_status_msg_for_failed_peer, false,
-            "Allows status-only Raft messages to be sent to a master peer in FAILED_UNRECOVERABLE "
-            "state.");
-TAG_FLAG(master_consensus_allow_status_msg_for_failed_peer, advanced);
-TAG_FLAG(master_consensus_allow_status_msg_for_failed_peer, hidden);
-TAG_FLAG(master_consensus_allow_status_msg_for_failed_peer, runtime);
-
 DECLARE_bool(master_support_change_config);
 DECLARE_int64(rpc_max_message_size);
 
@@ -471,6 +463,7 @@ Status SysCatalogTable::SetupTablet(
       cmeta_manager_,
       local_peer_pb_,
       master_->tablet_apply_pool(),
+      /*reload_txn_status_tablet_pool*/nullptr,
       /*txn_coordinator_factory*/ nullptr,
       [this, tablet_id](const string& reason) {
         this->SysCatalogStateChanged(tablet_id, reason);
@@ -479,7 +472,10 @@ Status SysCatalogTable::SetupTablet(
   consensus::ServerContext server_ctx{/*quiescing*/nullptr,
                                       master_->num_raft_leaders(),
                                       master_->raft_pool(),
-                                      &FLAGS_master_consensus_allow_status_msg_for_failed_peer};
+                                      // Allow sending status-only Raft messages to a master peer
+                                      // in FAILED_UNRECOVERABLE state if we allow dynamically
+                                      // adding/removing masters.
+                                      &FLAGS_master_support_change_config};
   RETURN_NOT_OK_SHUTDOWN(tablet_replica_->Init(std::move(server_ctx)),
                          "failed to initialize system catalog replica");
 
@@ -544,7 +540,7 @@ Status SysCatalogTable::WaitUntilRunning() {
                                << seconds_waited << " seconds)";
       continue;
     }
-    // if the status is not OK or TimedOut return it.
+    // If the status is not OK or not TimedOut, then return it.
     return status;
   }
   return Status::OK();
