@@ -61,6 +61,7 @@ using strings::Substitute;
 
 DECLARE_bool(txn_manager_enabled);
 DECLARE_bool(txn_manager_lazily_initialized);
+DECLARE_bool(txn_schedule_background_tasks);
 DECLARE_int32(rpc_service_queue_length);
 DECLARE_int64(txn_manager_status_table_range_partition_span);
 DECLARE_uint32(txn_manager_status_table_num_replicas);
@@ -85,6 +86,10 @@ class TxnManagerTest : public KuduTest {
     FLAGS_txn_manager_enabled = true;
     FLAGS_txn_manager_lazily_initialized = true;
 
+    // To facilitate testing just the TxnManager, switch off
+    // transaction-related background tasks.
+    FLAGS_txn_schedule_background_tasks = false;
+
     // In this test, there is just a single tablet servers in the cluster.
     FLAGS_txn_manager_status_table_num_replicas = 1;
 
@@ -173,7 +178,7 @@ TEST_F(TxnManagerTest, LazyInitialization) {
     ASSERT_OK(proxy_->GetTransactionState(req, &resp, &ctx));
     ASSERT_TRUE(resp.has_error());
     auto s = StatusFromPB(resp.error().status());
-    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 0 not found");
   }
 
@@ -213,7 +218,7 @@ TEST_F(TxnManagerTest, LazyInitialization) {
     ASSERT_OK(proxy_->CommitTransaction(req, &resp, &ctx));
     ASSERT_TRUE(resp.has_error());
     auto s = StatusFromPB(resp.error().status());
-    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 0 not found");
   }
   ASSERT_OK(master_->WaitForTxnManagerInit());
@@ -242,7 +247,7 @@ TEST_F(TxnManagerTest, LazyInitializationConcurrentCalls) {
       CHECK_OK(proxy_->GetTransactionState(req, &resp, &ctx));
       CHECK(resp.has_error());
       auto s = StatusFromPB(resp.error().status());
-      CHECK(s.IsNotFound()) << s.ToString();
+      CHECK(s.IsInvalidArgument()) << s.ToString();
     }
   };
 
@@ -293,7 +298,7 @@ TEST_F(TxnManagerTest, NonlazyInitialization) {
     ASSERT_OK(proxy_->CommitTransaction(req, &resp, &ctx));
     ASSERT_TRUE(resp.has_error());
     auto s = StatusFromPB(resp.error().status());
-    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 0 not found");
   }
 }
@@ -371,7 +376,7 @@ TEST_F(TxnManagerTest, AbortedTransactionLifecycle) {
         << StatusFromPB(resp.error().status()).ToString();
     TxnStatePB txn_state;
     NO_FATALS(fetch_txn_status(txn_id, &txn_state));
-    ASSERT_EQ(TxnStatePB::ABORTED, txn_state);
+    ASSERT_EQ(TxnStatePB::ABORT_IN_PROGRESS, txn_state);
   }
 
   // Try to send keep-alive for already aborted transaction.
@@ -387,11 +392,11 @@ TEST_F(TxnManagerTest, AbortedTransactionLifecycle) {
     ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
     ASSERT_STR_CONTAINS(
         s.ToString(),
-        Substitute("transaction ID $0 is already in terminal state", txn_id));
+        Substitute("transaction ID $0 is not available for further commits", txn_id));
     // The transaction should stay in ABORTED state, of course.
     TxnStatePB txn_state;
     NO_FATALS(fetch_txn_status(txn_id, &txn_state));
-    ASSERT_EQ(TxnStatePB::ABORTED, txn_state);
+    ASSERT_EQ(TxnStatePB::ABORT_IN_PROGRESS, txn_state);
   }
 }
 
@@ -538,7 +543,7 @@ TEST_F(TxnManagerTest, KeepTransactionAliveNonExistingTxnId) {
   ASSERT_OK(proxy_->KeepTransactionAlive(req, &resp, &ctx));
   ASSERT_TRUE(resp.has_error());
   auto s = StatusFromPB(resp.error().status());
-  ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+  ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 123 not found");
 }
 
