@@ -90,6 +90,7 @@ using kudu::tserver::ListTabletsResponsePB;
 using kudu::tserver::TabletServerAdminServiceProxy;
 using kudu::tserver::TabletServerServiceProxy;
 using std::copy;
+using std::map;
 using std::pair;
 using std::string;
 using std::unique_ptr;
@@ -107,26 +108,27 @@ DEFINE_bool(perf_record, false,
 namespace kudu {
 namespace cluster {
 
-static const char* const kMasterBinaryName = "kudu-master";
-static const char* const kTabletServerBinaryName = "kudu-tserver";
+static const char* const kKuduBinaryName = "kudu";
 static double kTabletServerRegistrationTimeoutSeconds = 15.0;
 static double kMasterCatalogManagerTimeoutSeconds = 60.0;
 
 ExternalMiniClusterOptions::ExternalMiniClusterOptions()
     : num_masters(1),
-      supply_single_master_addr(false),
+      supply_single_master_addr(true),
       num_tablet_servers(1),
       bind_mode(kDefaultBindMode),
       num_data_dirs(1),
       enable_kerberos(false),
+      principal("kudu"),
       hms_mode(HmsMode::NONE),
       enable_ranger(false),
       logtostderr(true),
       start_process_timeout(MonoDelta::FromSeconds(70)),
       rpc_negotiation_timeout(MonoDelta::FromSeconds(3))
 #if !defined(NO_CHRONY)
-      , num_ntp_servers(1)
-      , ntp_config_mode(BuiltinNtpConfigMode::ALL_SERVERS)
+      ,
+      num_ntp_servers(1),
+      ntp_config_mode(BuiltinNtpConfigMode::ALL_SERVERS)
 #endif // #if !defined(NO_CHRONY) ...
 {
 }
@@ -220,13 +222,14 @@ Status ExternalMiniCluster::Start() {
       << tablet_servers_.size() << "). Maybe you meant Restart()?";
   RETURN_NOT_OK(HandleOptions());
 
-  RETURN_NOT_OK_PREPEND(rpc::MessengerBuilder("minicluster-messenger")
-                        .set_num_reactors(1)
-                        .set_max_negotiation_threads(1)
-                        .set_rpc_negotiation_timeout_ms(
-                            opts_.rpc_negotiation_timeout.ToMilliseconds())
-                        .Build(&messenger_),
-                        "Failed to start Messenger for minicluster");
+  RETURN_NOT_OK_PREPEND(
+      rpc::MessengerBuilder("minicluster-messenger")
+          .set_num_reactors(1)
+          .set_max_negotiation_threads(1)
+          .set_rpc_negotiation_timeout_ms(opts_.rpc_negotiation_timeout.ToMilliseconds())
+          .set_sasl_proto_name(opts_.principal)
+          .Build(&messenger_),
+      "Failed to start Messenger for minicluster");
 
   Status s = env()->CreateDir(opts_.cluster_root);
   if (!s.ok() && !s.IsAlreadyPresent()) {
@@ -331,6 +334,10 @@ Status ExternalMiniCluster::Start() {
                             "could not create keytab");
       hms_->EnableKerberos(kdc_->GetEnvVars()["KRB5_CONFIG"], spn, ktpath,
                            rpc::SaslProtection::kAuthentication);
+
+      // Set the protocol name in the environment so that the KuduMetastorePlugin
+      // can communicate with Kudu when a custom name is used.
+      hms_->AddEnvVar("KUDU_SASL_PROTOCOL_NAME", opts_.principal);
     }
 
     RETURN_NOT_OK_PREPEND(hms_->Start(),
@@ -410,10 +417,10 @@ void ExternalMiniCluster::DisableMetastoreIntegration() {
 void ExternalMiniCluster::SetDaemonBinPath(string daemon_bin_path) {
   opts_.daemon_bin_path = std::move(daemon_bin_path);
   for (auto& master : masters_) {
-    master->SetExePath(GetBinaryPath(kMasterBinaryName));
+    master->SetExePath(GetBinaryPath(kKuduBinaryName));
   }
   for (auto& ts : tablet_servers_) {
-    ts->SetExePath(GetBinaryPath(kTabletServerBinaryName));
+    ts->SetExePath(GetBinaryPath(kKuduBinaryName));
   }
 }
 
@@ -534,7 +541,7 @@ Status ExternalMiniCluster::StartMasters() {
        std::back_inserter(flags));
 
   // Start the masters.
-  const string& exe = GetBinaryPath(kMasterBinaryName);
+  const string& exe = GetBinaryPath(kKuduBinaryName);
   for (int i = 0; i < num_masters; i++) {
     string daemon_id = Substitute("master-$0", i);
 
@@ -574,8 +581,9 @@ Status ExternalMiniCluster::StartMasters() {
 
     scoped_refptr<ExternalMaster> peer = new ExternalMaster(opts);
     if (opts_.enable_kerberos) {
-      RETURN_NOT_OK_PREPEND(peer->EnableKerberos(kdc_.get(), master_rpc_addrs[i].host()),
-                            "could not enable Kerberos");
+      RETURN_NOT_OK_PREPEND(
+          peer->EnableKerberos(kdc_.get(), opts_.principal, master_rpc_addrs[i].host()),
+          "could not enable Kerberos");
     }
     RETURN_NOT_OK_PREPEND(peer->Start(),
                           Substitute("Unable to start Master at index $0", i));
@@ -611,7 +619,7 @@ Status ExternalMiniCluster::AddTabletServer() {
   ExternalDaemonOptions opts;
   opts.messenger = messenger_;
   opts.block_manager_type = opts_.block_manager_type;
-  opts.exe = GetBinaryPath(kTabletServerBinaryName);
+  opts.exe = GetBinaryPath(kKuduBinaryName);
   opts.wal_dir = GetWalPath(daemon_id);
   opts.data_dirs = GetDataPaths(daemon_id);
   opts.log_dir = GetLogPath(daemon_id);
@@ -631,7 +639,7 @@ Status ExternalMiniCluster::AddTabletServer() {
   vector<HostPort> master_hostports = master_rpc_addrs();
   scoped_refptr<ExternalTabletServer> ts = new ExternalTabletServer(opts, master_hostports);
   if (opts_.enable_kerberos) {
-    RETURN_NOT_OK_PREPEND(ts->EnableKerberos(kdc_.get(), bind_host),
+    RETURN_NOT_OK_PREPEND(ts->EnableKerberos(kdc_.get(), opts_.principal, bind_host),
                           "could not enable Kerberos");
   }
 
@@ -913,6 +921,7 @@ Status ExternalMiniCluster::CreateClient(client::KuduClientBuilder* builder,
   for (const scoped_refptr<ExternalMaster>& master : masters_) {
     builder->add_master_server_addr(master->bound_rpc_hostport().ToString());
   }
+  builder->sasl_protocol_name(opts_.principal);
   return builder->Build(client);
 }
 
@@ -946,18 +955,14 @@ string ExternalMiniCluster::UuidForTS(int ts_idx) const {
   return tablet_server(ts_idx)->uuid();
 }
 
-Status ExternalMiniCluster::AddMaster(const scoped_refptr<ExternalMaster>& new_master) {
-  const auto& new_master_uuid = new_master->instance_id().permanent_uuid();
-  for (const auto& m : masters_) {
-    if (m->instance_id().permanent_uuid() == new_master_uuid) {
-      CHECK(m->bound_rpc_hostport() == new_master->bound_rpc_hostport());
-      return Status::AlreadyPresent(Substitute(
-          "Master $0, uuid: $1 already present in ExternalMiniCluster",
-          m->bound_rpc_hostport().ToString(), new_master_uuid));
+Status ExternalMiniCluster::RemoveMaster(const HostPort& hp) {
+  for (auto it = masters_.begin(); it != masters_.end(); ++it) {
+    if ((*it)->bound_rpc_hostport() == hp) {
+      masters_.erase(it);
+      return Status::OK();
     }
   }
-  masters_.emplace_back(new_master);
-  return Status::OK();
+  return Status::NotFound(Substitute("Master $0 not found in ExternalMiniCluster", hp.ToString()));
 }
 
 //------------------------------------------------------------
@@ -973,61 +978,50 @@ ExternalDaemon::ExternalDaemon(ExternalDaemonOptions opts)
 ExternalDaemon::~ExternalDaemon() {
 }
 
-Status ExternalDaemon::StartProcess(const vector<string>& user_flags) {
-  CHECK(!process_);
-  const auto this_tid = std::this_thread::get_id();
-  CHECK_EQ(parent_tid_, this_tid)
-    << "Process being started from thread " << this_tid << " which is different"
-    << " from the instantiating thread " << parent_tid_;
+std::vector<std::string> ExternalDaemon::GetDaemonFlags(const ExternalDaemonOptions& opts) {
+  const string info_path = JoinPathSegments(opts.data_dirs[0], "info.pb");
+  vector<string> flags = {
+      // Basic flags for a Kudu server.
+      "--fs_wal_dir=" + opts.wal_dir,
+      "--fs_data_dirs=" + JoinStrings(opts.data_dirs, ","),
+      "--block_manager=" + opts.block_manager_type,
+      "--webserver_interface=localhost",
 
-  RETURN_NOT_OK(env_util::CreateDirsRecursively(Env::Default(), log_dir()));
-  const string info_path = JoinPathSegments(data_dirs()[0], "info.pb");
+      // Disable fsync to dramatically speed up runtime. This is safe as no tests
+      // rely on forcefully cutting power to a machine or equivalent.
+      "--never_fsync",
 
-  vector<string> argv = {
-    // First the exe for argv[0].
-    opts_.exe,
+      // Disable minidumps by default since many tests purposely inject faults.
+      "--enable_minidumps=false",
 
-    // Basic flags for Kudu server.
-    "--fs_wal_dir=" + wal_dir(),
-    "--fs_data_dirs=" + JoinStrings(data_dirs(), ","),
-    "--block_manager=" + opts_.block_manager_type,
-    "--webserver_interface=localhost",
+      // Disable redaction of the information in logs and Web UI.
+      "--redact=none",
 
-    // Disable fsync to dramatically speed up runtime. This is safe as no tests
-    // rely on forcefully cutting power to a machine or equivalent.
-    "--never_fsync",
+      // Enable metrics logging.
+      "--metrics_log_interval_ms=1000",
 
-    // Disable minidumps by default since many tests purposely inject faults.
-    "--enable_minidumps=false",
+      // Even if we are logging to stderr, metrics logs and minidumps end up being
+      // written based on -log_dir. So, we have to set that too.
+      "--log_dir=" + opts.log_dir,
 
-    // Disable redaction of the information in logs and Web UI.
-    "--redact=none",
+      // Tell the server to dump its port information so we can pick it up.
+      "--server_dump_info_path=" + info_path,
+      "--server_dump_info_format=pb",
 
-    // Enable metrics logging.
-    "--metrics_log_interval_ms=1000",
+      // We use ephemeral ports in many tests. They don't work for production,
+      // but are OK in unit tests.
+      "--rpc_server_allow_ephemeral_ports",
 
-    // Even if we are logging to stderr, metrics logs and minidumps end up being
-    // written based on -log_dir. So, we have to set that too.
-    "--log_dir=" + log_dir(),
-
-    // Tell the server to dump its port information so we can pick it up.
-    "--server_dump_info_path=" + info_path,
-    "--server_dump_info_format=pb",
-
-    // We use ephemeral ports in many tests. They don't work for production,
-    // but are OK in unit tests.
-    "--rpc_server_allow_ephemeral_ports",
-
-    // Allow unsafe and experimental flags from tests, since we often use
-    // fault injection, etc.
-    "--unlock_experimental_flags",
-    "--unlock_unsafe_flags",
+      // Allow unsafe and experimental flags from tests, since we often use
+      // fault injection, etc.
+      "--unlock_experimental_flags",
+      "--unlock_unsafe_flags",
   };
 
-  if (opts_.logtostderr) {
+  if (opts.logtostderr) {
     // Ensure that logging goes to the test output and doesn't get buffered.
-    argv.emplace_back("--logtostderr");
-    argv.emplace_back("--logbuflevel=-1");
+    flags.emplace_back("--logtostderr");
+    flags.emplace_back("--logbuflevel=-1");
   }
 
   // If large keys are not enabled.
@@ -1039,14 +1033,32 @@ Status ExternalDaemon::StartProcess(const vector<string>& user_flags) {
     // since we are using strong/high TLS v1.2 cipher suites, so the minimum
     // size of TLS-related RSA key is 768 bits (due to the usage of
     // the ECDHE-RSA-AES256-GCM-SHA384 suite).
-    argv.emplace_back("--ipki_server_key_size=768");
+    flags.emplace_back("--ipki_server_key_size=768");
 
     // The RSA key of 768 bits is too short if OpenSSL security level is set to
     // 1 or higher (applicable for OpenSSL 1.1.0 and newer). Lowering the
     // security level to 0 makes possible ot use shorter keys in such cases.
-    argv.emplace_back("--openssl_security_level_override=0");
+    flags.emplace_back("--openssl_security_level_override=0");
   }
 
+  return flags;
+}
+
+Status ExternalDaemon::StartProcess(const vector<string>& user_flags) {
+  CHECK(!process_);
+  const auto this_tid = std::this_thread::get_id();
+  CHECK_EQ(parent_tid_, this_tid)
+    << "Process being started from thread " << this_tid << " which is different"
+    << " from the instantiating thread " << parent_tid_;
+
+  RETURN_NOT_OK(env_util::CreateDirsRecursively(Env::Default(), log_dir()));
+  const string info_path = JoinPathSegments(data_dirs()[0], "info.pb");
+
+  vector<string> argv = { /* First the exe for argv[0] */ opts_.exe };
+  vector<string> flags = GetDaemonFlags(opts_);
+  argv.insert(argv.end(), std::make_move_iterator(flags.begin()),
+              std::make_move_iterator(flags.end()));
+
   // Add all the flags coming from the minicluster framework.
   argv.insert(argv.end(), user_flags.begin(), user_flags.end());
 
@@ -1145,23 +1157,35 @@ void ExternalDaemon::SetMetastoreIntegration(const string& hms_uris,
   opts_.extra_flags.emplace_back(Substitute("--hive_metastore_sasl_enabled=$0", enable_kerberos));
 }
 
-Status ExternalDaemon::EnableKerberos(MiniKdc* kdc, const string& bind_host) {
-  string spn = "kudu/" + bind_host;
+Status ExternalDaemon::CreateKerberosConfig(MiniKdc* kdc,
+                                            const string& principal_base,
+                                            const string& bind_host,
+                                            vector<string>* flags,
+                                            map<string, string>* env_vars) {
+  string spn = principal_base + "/" + bind_host;
   string ktpath;
   RETURN_NOT_OK_PREPEND(kdc->CreateServiceKeytab(spn, &ktpath),
                         "could not create keytab");
-  extra_env_ = kdc->GetEnvVars();
+  *env_vars = kdc->GetEnvVars();
+  *flags =  {
+      Substitute("--keytab_file=$0", ktpath),
+      Substitute("--principal=$0", spn),
+      "--rpc_authentication=required",
+      "--superuser_acl=test-admin",
+      "--user_acl=test-user",
+  };
 
+  return Status::OK();
+}
+
+Status ExternalDaemon::EnableKerberos(MiniKdc* kdc, const string& principal_base,
+                                      const string& bind_host) {
+  vector<string> flags;
+  RETURN_NOT_OK(CreateKerberosConfig(kdc, principal_base, bind_host, &flags, &extra_env_));
   // Insert Kerberos flags at the front of extra_flags, so that user specified
   // flags will override them.
-  opts_.extra_flags.insert(opts_.extra_flags.begin(), {
-    Substitute("--keytab_file=$0", ktpath),
-    Substitute("--principal=$0", spn),
-    "--rpc_authentication=required",
-    "--superuser_acl=test-admin",
-    "--user_acl=test-user",
-  });
-
+  opts_.extra_flags.insert(opts_.extra_flags.begin(), std::make_move_iterator(flags.begin()),
+                           std::make_move_iterator(flags.end()));
   return Status::OK();
 }
 
@@ -1400,10 +1424,10 @@ ExternalMaster::~ExternalMaster() {
 }
 
 Status ExternalMaster::Start() {
-  vector<string> flags(GetCommonFlags());
-  flags.push_back(Substitute("--rpc_bind_addresses=$0", rpc_bind_address().ToString()));
-  flags.push_back(Substitute("--webserver_interface=$0", rpc_bind_address().host()));
-  flags.emplace_back("--webserver_port=0");
+  vector<string> flags { "master", "run" };
+  auto common_flags = GetCommonFlags(rpc_bind_address());
+  flags.insert(flags.end(), std::make_move_iterator(common_flags.begin()),
+               std::make_move_iterator(common_flags.end()));
   return StartProcess(flags);
 }
 
@@ -1413,16 +1437,10 @@ Status ExternalMaster::Restart() {
     return Status::IllegalState("Master cannot be restarted. Must call Shutdown() first.");
   }
 
-  vector<string> flags(GetCommonFlags());
-  flags.push_back(Substitute("--rpc_bind_addresses=$0", bound_rpc_.ToString()));
-  if (bound_http_.Initialized()) {
-    flags.push_back(Substitute("--webserver_interface=$0", bound_http_.host()));
-    flags.push_back(Substitute("--webserver_port=$0", bound_http_.port()));
-  } else {
-    flags.push_back(Substitute("--webserver_interface=$0", bound_rpc_.host()));
-    flags.emplace_back("--webserver_port=0");
-  }
-
+  vector<string> flags { "master", "run" };
+  auto common_flags = GetCommonFlags(bound_rpc_, bound_http_);
+  flags.insert(flags.end(), std::make_move_iterator(common_flags.begin()),
+               std::make_move_iterator(common_flags.end()));
   return StartProcess(flags);
 }
 
@@ -1471,20 +1489,37 @@ Status ExternalMaster::WaitForCatalogManager(WaitMode wait_mode) {
   return Status::OK();
 }
 
-const vector<string>& ExternalMaster::GetCommonFlags() {
-  static vector<string> kFlags;
+vector<string> ExternalMaster::GetCommonFlags(const HostPort& rpc_bind_addr,
+                                              const HostPort& http_addr) {
+  vector<string> flags;
   if (!UseLargeKeys()) {
-    kFlags = {
-        // See the in-line comment for "--ipki_server_key_size" flag in
-        // ExternalDaemon::StartProcess() method.
-        "--ipki_ca_key_size=768",
-
-        // As for the TSK keys, 512 bits is the minimum since we are using
-        // SHA256 digest for token signing/verification.
-        "--tsk_num_rsa_bits=512",
-    };
+    // See the in-line comment for "--ipki_server_key_size" flag in
+    // ExternalDaemon::StartProcess() method.
+    flags.emplace_back("--ipki_ca_key_size=768");
+    // As for the TSK keys, 512 bits is the minimum since we are using
+    // SHA256 digest for token signing/verification.
+    flags.emplace_back("--tsk_num_rsa_bits=512");
   }
-  return kFlags;
+
+  flags.emplace_back(Substitute("--rpc_bind_addresses=$0", rpc_bind_addr.ToString()));
+
+  if (http_addr.Initialized()) {
+    flags.emplace_back(Substitute("--webserver_interface=$0", http_addr.host()));
+    flags.emplace_back(Substitute("--webserver_port=$0", http_addr.port()));
+  } else {
+    flags.emplace_back(Substitute("--webserver_interface=$0", rpc_bind_addr.host()));
+    flags.emplace_back("--webserver_port=0");
+  }
+
+  return flags;
+}
+
+vector<string> ExternalMaster::GetMasterFlags(const ExternalDaemonOptions& opts) {
+  vector<string> flags(ExternalDaemon::GetDaemonFlags(opts));
+  auto common_flags = GetCommonFlags(opts.rpc_bind_address);
+  flags.insert(flags.end(), std::make_move_iterator(common_flags.begin()),
+               std::make_move_iterator(common_flags.end()));
+  return flags;
 }
 
 //------------------------------------------------------------
@@ -1503,6 +1538,7 @@ ExternalTabletServer::~ExternalTabletServer() {
 
 Status ExternalTabletServer::Start() {
   vector<string> flags {
+    "tserver", "run",
     Substitute("--rpc_bind_addresses=$0", rpc_bind_address().ToString()),
     Substitute("--local_ip_for_outbound_sockets=$0", rpc_bind_address().host()),
     Substitute("--webserver_interface=$0", rpc_bind_address().host()),
@@ -1519,6 +1555,7 @@ Status ExternalTabletServer::Restart() {
     return Status::IllegalState("Tablet server cannot be restarted. Must call Shutdown() first.");
   }
   vector<string> flags {
+    "tserver", "run",
     Substitute("--rpc_bind_addresses=$0", bound_rpc_.ToString()),
     Substitute("--local_ip_for_outbound_sockets=$0", rpc_bind_address().host()),
     Substitute("--tserver_master_addrs=$0",
