@@ -31,7 +31,6 @@
 #include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 
-#include "kudu/client/shared_ptr.h" // IWYU pragma: keep
 #include "kudu/common/common.pb.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/port.h"
@@ -138,7 +137,7 @@ struct ExternalMiniClusterOptions {
   int num_masters;
 
   // Whether to supply 'master_addresses' field for single master configuration.
-  // Default: False
+  // Default: True
   bool supply_single_master_addr;
 
   // Number of TS to start.
@@ -195,6 +194,11 @@ struct ExternalMiniClusterOptions {
   // Default: false.
   bool enable_kerberos;
 
+  // Service principal name for the servers.
+  //
+  // Default: "kudu".
+  std::string principal;
+
   // Tri state mode flag that indicates whether to set up a Hive Metastore as
   // part of this ExternalMiniCluster and enable Kudu Hive Metastore integration.
   //
@@ -365,6 +369,13 @@ class ExternalMiniCluster : public MiniCluster {
     return opts_.cluster_root;
   }
 
+  // Kerberos principal prefix name whose credentials are used to run Kudu
+  // servers in the cluster. Matches the SASL protocol name used for connection
+  // negotiation.
+  const std::string& service_principal() const {
+    return opts_.principal;
+  }
+
   int num_tablet_servers() const override {
     return tablet_servers_.size();
   }
@@ -476,9 +487,10 @@ class ExternalMiniCluster : public MiniCluster {
   // files that reside in the log dir.
   std::string GetLogPath(const std::string& daemon_id) const;
 
-  // Adds a master to the ExternalMiniCluster when the new master has been added
-  // dynamically after bringing up the ExternalMiniCluster.
-  Status AddMaster(const scoped_refptr<ExternalMaster>& master);
+  // Removes any bookkeeping of the master specified by 'hp' from the ExternalMiniCluster
+  // after already having run through a successful master Raft change config to remove it.
+  // This helps keep the state of the actual cluster in sync with the state in ExternalMiniCluster.
+  Status RemoveMaster(const HostPort& hp);
 
  private:
   Status StartMasters();
@@ -557,15 +569,27 @@ class ExternalDaemon : public RefCountedThreadSafe<ExternalDaemon> {
   void SetMetastoreIntegration(const std::string& hms_uris,
                                bool enable_kerberos);
 
+  // Create a Kerberos principal and keytab using the 'principal_base' and 'bind_host' hostname
+  // of the form <principal_base>/<bind_host>.
+  // Returns the generated keytab file and service principal as 'flags' and the appropriate
+  // environment variables in 'env_vars' output parameters.
+  static Status CreateKerberosConfig(MiniKdc* kdc, const std::string& principal_base,
+                                     const std::string& bind_host,
+                                     std::vector<std::string>* flags,
+                                     std::map<std::string, std::string>* env_vars);
+
   // Enable Kerberos for this daemon. This creates a Kerberos principal
   // and keytab, and sets the appropriate environment variables in the
   // subprocess such that the server will use Kerberos authentication.
   //
   // 'bind_host' is the hostname that will be used to generate the Kerberos
-  // service principal.
+  // service principal, 'principal_base' will be the first part (i.e.
+  // <principal_base>/<bind_host>).
   //
   // Must be called before 'StartProcess()'.
-  Status EnableKerberos(MiniKdc* kdc, const std::string& bind_host);
+  Status EnableKerberos(MiniKdc* kdc,
+                        const std::string& principal_base,
+                        const std::string& bind_host);
 
   // Sends a SIGSTOP signal to the daemon.
   Status Pause() WARN_UNUSED_RESULT;
@@ -646,6 +670,10 @@ class ExternalDaemon : public RefCountedThreadSafe<ExternalDaemon> {
   // LOG(FATAL) if there are any leaks.
   void CheckForLeaks();
 
+  // Get flags for an ExternalDaemon from the supplied 'opts'.
+  // It does not include executable or any positional arguments.
+  static std::vector<std::string> GetDaemonFlags(const ExternalDaemonOptions& opts);
+
   // Get RPC bind address for daemon.
   const HostPort& rpc_bind_address() const {
     return opts_.rpc_bind_address;
@@ -715,9 +743,16 @@ class ExternalMaster : public ExternalDaemon {
   Status WaitForCatalogManager(
       WaitMode wait_mode = DONT_WAIT_FOR_LEADERSHIP) WARN_UNUSED_RESULT;
 
+  // Get all flags for a master from the supplied 'opts'.
+  // It does not include executable or any positional arguments like "master run".
+  static std::vector<std::string> GetMasterFlags(const ExternalDaemonOptions& opts);
  private:
   friend class RefCountedThreadSafe<ExternalMaster>;
-  static const std::vector<std::string>& GetCommonFlags();
+  // Get flags specific to ExternalMaster where 'rpc_bind_addr' and 'http_addr' are
+  // the RPC and HTTP addresses to bind in case of start or the corresponding bound
+  // addresses in case of restart.
+  static std::vector<std::string> GetCommonFlags(const HostPort& rpc_bind_addr,
+                                                 const HostPort& http_addr = HostPort());
   virtual ~ExternalMaster();
 };
 
