@@ -43,6 +43,7 @@ class Messenger;
 } // namespace rpc
 
 namespace tserver {
+class TabletServerAdminServiceProxy;
 class TabletServerServiceProxy;
 } // namespace tserver
 
@@ -118,6 +119,11 @@ class MiniCluster {
   // 'idx' is running.
   virtual std::shared_ptr<tserver::TabletServerServiceProxy> tserver_proxy(int idx) const = 0;
 
+  // Returns an RPC proxy to the tserver admin service at 'idx'. Requires that
+  // the tserver at 'idx' is running.
+  virtual std::shared_ptr<tserver::TabletServerAdminServiceProxy> tserver_admin_proxy(
+      int idx) const = 0;
+
   // Returns the UUID for the tablet server 'ts_idx'
   virtual std::string UuidForTS(int ts_idx) const = 0;
 
