@@ -17,11 +17,7 @@
 
 #include "kudu/server/webserver.h"
 
-#if defined(KUDU_OPENSSL_REQUIRE_FIPS_HEADER)
-#include <openssl/fips.h>
-#else
 #include <openssl/crypto.h>
-#endif
 
 #include <cstdlib>
 #include <functional>
@@ -178,7 +174,7 @@ TEST_F(PasswdWebserverTest, TestCrashInFIPSMode) {
 
   Status s = server_->Start();
   ASSERT_TRUE(s.IsIllegalState());
-  ASSERT_STR_CONTAINS("Digest authentication in FIPS approved mode", s.ToString());
+  ASSERT_STR_CONTAINS(s.ToString(), "Digest authentication in FIPS approved mode");
 }
 
 class SpnegoWebserverTest : public WebserverTest {
