@@ -20,6 +20,7 @@
 #include <memory>
 #include <ostream>
 
+#include <gflags/gflags.h>
 #include <glog/logging.h>
 #include <google/protobuf/arena.h>
 
@@ -29,6 +30,7 @@
 #include "kudu/consensus/opid.pb.h"
 #include "kudu/consensus/raft_consensus.h"
 #include "kudu/consensus/time_manager.h"
+#include "kudu/gutil/macros.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/rpc/rpc_header.pb.h"
@@ -39,10 +41,16 @@
 #include "kudu/tablet/txn_participant.h"
 #include "kudu/tserver/tserver.pb.h"
 #include "kudu/util/debug/trace_event.h"
+#include "kudu/util/flag_tags.h"
 #include "kudu/util/pb_util.h"
 #include "kudu/util/status.h"
 #include "kudu/util/trace.h"
 
+DEFINE_bool(enable_txn_partition_lock, true,
+            "Whether or not to enable partition lock for transactions");
+TAG_FLAG(enable_txn_partition_lock, unsafe);
+TAG_FLAG(enable_txn_partition_lock, hidden);
+
 using kudu::consensus::CommitMsg;
 using kudu::consensus::ReplicateMsg;
 using kudu::consensus::OperationType;
@@ -156,13 +164,37 @@ Status ParticipantOp::Prepare() {
   state_->AcquireTxnAndLock();
   RETURN_NOT_OK(state_->ValidateOp());
 
+  const auto& op = state_->request()->op();
+  auto* replica = state_->tablet_replica();
+
   // Before we assign a timestamp, bump the clock so further ops get assigned
   // higher timestamps (including this one).
-  if (state_->request()->op().type() == ParticipantOpPB::FINALIZE_COMMIT &&
-      type() == consensus::LEADER) {
-    DCHECK(!state_->consensus_round()->replicate_msg()->has_timestamp());
-    RETURN_NOT_OK(state_->tablet_replica()->time_manager()->UpdateClockAndLastAssignedTimestamp(
-        state_->commit_timestamp()));
+  switch (op.type()) {
+    case ParticipantOpPB::BEGIN_COMMIT:
+      // To avoid inconsistencies, TxnOpDispatcher should not contain any
+      // pending write requests at this point. Those pending requests must be
+      // submitted and replied accordingly before BEGIN_COMMIT can be processed.
+      // Even if UnregisterTxnOpDispatcher() returns non-OK, the TxnOpDispatcher
+      // is marked for removal, so no write requests are accepted by the replica
+      // in the context of the specified transaction after a call to
+      // TabletReplica::UnregisterTxnOpDispatcher().
+      RETURN_NOT_OK(replica->UnregisterTxnOpDispatcher(
+          op.txn_id(), false/*abort_pending_ops*/));
+      break;
+    case ParticipantOpPB::FINALIZE_COMMIT:
+      if (type() == consensus::LEADER) {
+        DCHECK(!state_->consensus_round()->replicate_msg()->has_timestamp());
+        RETURN_NOT_OK(state_->tablet_replica()->time_manager()->
+            UpdateClockAndLastAssignedTimestamp(state_->commit_timestamp()));
+      }
+      break;
+    case ParticipantOpPB::ABORT_TXN:
+      RETURN_NOT_OK(replica->UnregisterTxnOpDispatcher(
+          op.txn_id(), true/*abort_pending_ops*/));
+      break;
+    default:
+      // Nothing to do in all other cases.
+      break;
   }
   TRACE("PREPARE: Finished.");
   return Status::OK();
@@ -183,7 +215,7 @@ Status ParticipantOp::Start() {
 
 Status ParticipantOpState::PerformOp(const consensus::OpId& op_id, Tablet* tablet) {
   const auto& op = request()->op();
-  const auto& op_type = request()->op().type();
+  const auto& op_type = op.type();
   Status s;
   switch (op_type) {
     // NOTE: these can currently never fail because we are only updating
@@ -207,6 +239,7 @@ Status ParticipantOpState::PerformOp(const consensus::OpId& op_id, Tablet* table
       if (txn_->commit_op()) {
         txn_->commit_op()->FinishApplying();
       }
+      txn_->ReleasePartitionLock();
       break;
     }
     case ParticipantOpPB::ABORT_TXN: {
@@ -216,6 +249,7 @@ Status ParticipantOpState::PerformOp(const consensus::OpId& op_id, Tablet* table
       if (txn_->commit_op()) {
         txn_->commit_op()->Abort();
       }
+      txn_->ReleasePartitionLock();
       break;
     }
     case ParticipantOpPB::UNKNOWN: {
