@@ -65,6 +65,7 @@
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/rpc/result_tracker.h"
 #include "kudu/rpc/rpc_header.pb.h"
+#include "kudu/tablet/lock_manager.h"
 #include "kudu/tablet/metadata.pb.h"
 #include "kudu/tablet/mvcc.h"
 #include "kudu/tablet/ops/alter_schema_op.h"
@@ -93,6 +94,7 @@
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/stopwatch.h"
 
+DECLARE_bool(prevent_kudu_2233_corruption);
 DECLARE_int32(group_commit_queue_size_bytes);
 
 DEFINE_double(fault_crash_during_log_replay, 0.0,
@@ -1151,7 +1153,8 @@ Status TabletBootstrap::HandleEntryPair(const IOContext* io_context, LogEntryPB*
     default:
       break;
   }
-  if (!timestamp_assigned_in_opid_order) {
+  if (!timestamp_assigned_in_opid_order ||
+      PREDICT_FALSE(!FLAGS_prevent_kudu_2233_corruption)) {
     return Status::OK();
   }
 
@@ -1627,7 +1630,19 @@ Status TabletBootstrap::PlayRowOperations(const IOContext* io_context,
                         Substitute("Could not decode row operations: $0",
                                    SecureDebugString(op_state->request()->row_operations())));
 
-  // Run AcquireRowLocks, Apply, etc!
+  // If the write is a part of a transaction that's currently open (i.e., it
+  // has an in-flight Txn associated with it), lock the Txn now and make sure
+  // it's open.
+  // NOTE: we shouldn't take this lock if we've persisted that the transaction
+  // has completed, since there is no corresponding in-flight Txn.
+  if (op_state->txn_id() && !ContainsKey(terminal_txn_ids_, *op_state->txn_id())) {
+    RETURN_NOT_OK_PREPEND(tablet_->AcquireTxnLock(*op_state->txn_id(), op_state),
+                          "Failed to acquire txn lock");;
+  }
+
+  // Acquire partition/row locks, Apply, etc!
+  RETURN_NOT_OK_PREPEND(tablet_->AcquirePartitionLock(op_state, LockManager::WAIT_FOR_LOCK),
+                        "Failed to acquire partition lock");
   RETURN_NOT_OK_PREPEND(tablet_->AcquireRowLocks(op_state),
                         "Failed to acquire row locks");
 
