@@ -415,11 +415,11 @@ Status TabletMetadata::LoadFromSuperBlock(const TabletSuperBlockPB& superblock)
       PartitionSchema partition_schema;
       RETURN_NOT_OK(PartitionSchema::FromPB(superblock.partition_schema(),
                                             *schema_, &partition_schema));
-      CHECK(partition_schema_.Equals(partition_schema));
+      CHECK(partition_schema_ == partition_schema);
 
       Partition partition;
       Partition::FromPB(superblock.partition(), &partition);
-      CHECK(partition_.Equals(partition));
+      CHECK(partition_ == partition);
     }
 
     tablet_data_state_ = superblock.tablet_data_state();
@@ -654,7 +654,7 @@ Status TabletMetadata::UpdateUnlocked(
   }
   for (const auto& txn_id : txns_being_flushed) {
     auto txn_meta = FindOrDie(txn_metadata_by_txn_id_, txn_id);
-    txn_meta->set_flushed_committed_mrs_unlocked();
+    txn_meta->set_flushed_committed_mrs();
   }
 
   RowSetMetadataVector new_rowsets = rowsets_;
@@ -738,7 +738,7 @@ Status TabletMetadata::ToSuperBlockUnlocked(TabletSuperBlockPB* super_block,
   partition_.ToPB(pb.mutable_partition());
   pb.set_last_durable_mrs_id(last_durable_mrs_id_);
   pb.set_schema_version(schema_version_);
-  partition_schema_.ToPB(pb.mutable_partition_schema());
+  RETURN_NOT_OK(partition_schema_.ToPB(*schema_, pb.mutable_partition_schema()));
   pb.set_table_name(table_name_);
 
   for (const shared_ptr<RowSetMetadata>& meta : rowsets) {
@@ -746,22 +746,8 @@ Status TabletMetadata::ToSuperBlockUnlocked(TabletSuperBlockPB* super_block,
   }
 
   for (const auto& txn_id_and_metadata : txn_metadata_by_txn_id_) {
-    TxnMetadataPB meta_pb;
     const auto& txn_meta = txn_id_and_metadata.second;
-    const auto& commit_ts = txn_meta->commit_timestamp();
-    if (commit_ts) {
-      meta_pb.set_commit_timestamp(commit_ts->value());
-    }
-    const auto& commit_mvcc_op_ts = txn_meta->commit_mvcc_op_timestamp();
-    if (commit_mvcc_op_ts) {
-      meta_pb.set_commit_mvcc_op_timestamp(commit_mvcc_op_ts->value());
-    }
-    if (txn_meta->aborted()) {
-      meta_pb.set_aborted(true);
-    }
-    if (txn_meta->flushed_committed_mrs_unlocked()) {
-      meta_pb.set_flushed_committed_mrs(true);
-    }
+    TxnMetadataPB meta_pb = txn_meta->ToPB();
     InsertOrDie(pb.mutable_txn_metadata(), txn_id_and_metadata.first, meta_pb);
   }
 
@@ -822,8 +808,7 @@ void TabletMetadata::AddTxnMetadata(int64_t txn_id, unique_ptr<MinLogIndexAnchor
 void TabletMetadata::BeginCommitTransaction(int64_t txn_id, Timestamp mvcc_op_timestamp,
                                             unique_ptr<MinLogIndexAnchorer> log_anchor) {
   std::lock_guard<LockType> l(data_lock_);
-  auto txn_metadata = FindPtrOrNull(txn_metadata_by_txn_id_, txn_id);
-  CHECK(txn_metadata);
+  auto txn_metadata = FindOrDie(txn_metadata_by_txn_id_, txn_id);
   // NOTE: we may already have an MVCC op timestamp if we are bootstrapping and
   // the timestamp was persisted already, in which case, we don't need to
   // anchor the WAL to ensure the timestamp's persistence in metadata.
@@ -836,31 +821,41 @@ void TabletMetadata::BeginCommitTransaction(int64_t txn_id, Timestamp mvcc_op_ti
 void TabletMetadata::AddCommitTimestamp(int64_t txn_id, Timestamp commit_timestamp,
                                         unique_ptr<MinLogIndexAnchorer> log_anchor) {
   std::lock_guard<LockType> l(data_lock_);
-  auto txn_metadata = FindPtrOrNull(txn_metadata_by_txn_id_, txn_id);
-  CHECK(txn_metadata);
+  auto txn_metadata = FindOrDie(txn_metadata_by_txn_id_, txn_id);
   txn_metadata->set_commit_timestamp(commit_timestamp);
   anchors_needing_flush_.emplace_back(std::move(log_anchor));
 }
 
 void TabletMetadata::AbortTransaction(int64_t txn_id, unique_ptr<MinLogIndexAnchorer> log_anchor) {
   std::lock_guard<LockType> l(data_lock_);
-  auto txn_metadata = FindPtrOrNull(txn_metadata_by_txn_id_, txn_id);
+  // NOTE: we can't emplace with a raw pointer here; if the lookup succeeds, we
+  // wouldn't use it and we'd have a memory leak, so use scoped_refptr.
+  auto txn_metadata = LookupOrEmplace(&txn_metadata_by_txn_id_, txn_id,
+                                      scoped_refptr<TxnMetadata>(new TxnMetadata));
   CHECK(txn_metadata);
   txn_metadata->set_aborted();
   anchors_needing_flush_.emplace_back(std::move(log_anchor));
 }
 
-bool TabletMetadata::HasTxnMetadata(int64_t txn_id, TxnState* state) {
+bool TabletMetadata::HasTxnMetadata(int64_t txn_id, TxnState* state, Timestamp* timestamp) {
   std::lock_guard<LockType> l(data_lock_);
   auto txn_meta = FindPtrOrNull(txn_metadata_by_txn_id_, txn_id);
   if (txn_meta) {
     if (!state) return true;
     if (txn_meta->commit_timestamp()) {
       *state = kCommitted;
+      if (timestamp) {
+        DCHECK(txn_meta->commit_timestamp());
+        *timestamp = *txn_meta->commit_timestamp();
+      }
     } else if (txn_meta->aborted()) {
       *state = kAborted;
     } else if (txn_meta->commit_mvcc_op_timestamp()) {
       *state = kCommitInProgress;
+      if (timestamp) {
+        DCHECK(txn_meta->commit_mvcc_op_timestamp());
+        *timestamp = *txn_meta->commit_mvcc_op_timestamp();
+      }
     } else {
       *state = kOpen;
     }
@@ -889,7 +884,7 @@ void TabletMetadata::GetTxnIds(unordered_set<int64_t>* in_flight_txn_ids,
     // If we have not flushed the MRS after committing, the bootstrap process
     // will need to create an MRS for it, even if the transaction is committed.
     if (txn_ids_with_mrs &&
-        !txn_meta->flushed_committed_mrs_unlocked() &&
+        !txn_meta->flushed_committed_mrs() &&
         !txn_meta->aborted()) {
       EmplaceOrDie(&needs_mrs, txn_id);
     }
@@ -908,6 +903,16 @@ unordered_map<int64_t, scoped_refptr<TxnMetadata>> TabletMetadata::GetTxnMetadat
   return txn_metadata_by_txn_id_;
 }
 
+bool TabletMetadata::GetTxnMetadataPB(int64_t txn_id, TxnMetadataPB* pb) const {
+  std::lock_guard<LockType> l(data_lock_);
+  auto txn_meta = FindPtrOrNull(txn_metadata_by_txn_id_, txn_id);
+  if (!txn_meta) {
+    return false;
+  }
+  *pb = txn_meta->ToPB();
+  return true;
+}
+
 const RowSetMetadata *TabletMetadata::GetRowSetForTests(int64_t id) const {
   for (const shared_ptr<RowSetMetadata>& rowset_meta : rowsets_) {
     if (rowset_meta->id() == id) {
