@@ -17,10 +17,9 @@
 
 #include "kudu/tools/tool_action_common.h"
 
-#include <stdlib.h>
-
 #include <algorithm>
 #include <cstddef>
+#include <cstdlib>
 #include <iomanip>
 #include <iostream>
 #include <iterator>
@@ -146,6 +145,12 @@ DEFINE_int64(negotiation_timeout_ms, 3000,
              "Timeout for negotiating an RPC connection to a Kudu server, "
              "in milliseconds");
 
+DEFINE_string(sasl_protocol_name,
+              "kudu",
+              "SASL protocol name used to connnect to a Kerberos-enabled cluster. Must match the "
+              "servers' service principal name base (e.g. if it's \"kudu/_HOST\", then "
+              "sasl_protocol_name must be \"kudu\" to be able to connect.");
+
 bool ValidateTimeoutSettings() {
   if (FLAGS_timeout_ms < FLAGS_negotiation_timeout_ms) {
     LOG(ERROR) << strings::Substitute(
@@ -413,9 +418,10 @@ TServerActionBuilder::TServerActionBuilder(std::string name, ActionRunner runner
 
 Status BuildMessenger(std::string name, shared_ptr<Messenger>* messenger) {
   shared_ptr<Messenger> m;
-  MessengerBuilder b(std::move(name));
-  b.set_rpc_negotiation_timeout_ms(FLAGS_negotiation_timeout_ms);
-  auto s = b.Build(&m);
+  Status s = MessengerBuilder(std::move(name))
+                 .set_rpc_negotiation_timeout_ms(FLAGS_negotiation_timeout_ms)
+                 .set_sasl_proto_name(FLAGS_sasl_protocol_name)
+                 .Build(&m);
   if (s.ok()) {
     *messenger = std::move(m);
   }
@@ -582,11 +588,11 @@ Status CreateKuduClient(const vector<string>& master_addresses,
   if (can_see_all_replicas) {
     ReplicaController::SetVisibility(&b, ReplicaController::Visibility::ALL);
   }
-  return b
-      .master_server_addrs(master_addresses)
+  return b.master_server_addrs(master_addresses)
       .default_rpc_timeout(rpc_timeout)
       .default_admin_operation_timeout(rpc_timeout)
       .connection_negotiation_timeout(negotiation_timeout)
+      .sasl_protocol_name(FLAGS_sasl_protocol_name)
       .Build(client);
 }
 
@@ -648,7 +654,24 @@ Status ParseMasterAddresses(const RunnerContext& context,
   CHECK(master_addresses);
   string master_addresses_str;
   RETURN_NOT_OK(ParseMasterAddressesStr(context, master_addresses_arg, &master_addresses_str));
-  *master_addresses = strings::Split(master_addresses_str, ",");
+  vector<string> master_addresses_local = strings::Split(master_addresses_str, ",");
+  std::unordered_set<string> unique_masters;
+  std::unordered_set<string> duplicate_masters;
+  // Loop through the master addresses to find the duplicate. If there is no master specified
+  // do not report as a duplicate
+  for (const auto& master : master_addresses_local) {
+    if (master.empty()) continue;
+    if (ContainsKey(unique_masters, master)) {
+      duplicate_masters.insert(master);
+    } else {
+      unique_masters.insert(master);
+    }
+  }
+  if (!duplicate_masters.empty()) {
+    return Status::InvalidArgument(
+      "Duplicate master addresses specified: " + JoinStrings(duplicate_masters, ","));
+  }
+  *master_addresses = std::move(master_addresses_local);
   return Status::OK();
 }
 
@@ -744,6 +767,20 @@ Status DumpMemTrackers(const string& address, uint16_t default_port) {
   return Status::OK();
 }
 
+Status GetKuduToolAbsolutePathSafe(string* path) {
+  static const char* const kKuduCtlFileName = "kudu";
+  string exe;
+  RETURN_NOT_OK(Env::Default()->GetExecutablePath(&exe));
+  const string binroot = DirName(exe);
+  string tool_abs_path = JoinPathSegments(binroot, kKuduCtlFileName);
+  if (!Env::Default()->FileExists(tool_abs_path)) {
+    return Status::NotFound(Substitute(
+        "$0 binary not found at $1", kKuduCtlFileName, tool_abs_path));
+  }
+  *path = std::move(tool_abs_path);
+  return Status::OK();
+}
+
 namespace {
 
 // Pretty print a table using the psql format. For example:
@@ -922,16 +959,30 @@ Status LeaderMasterProxy::SyncRpc(const Req& req,
                                   const std::function<void(master::MasterServiceProxy*,
                                                            const Req&, Resp*,
                                                            rpc::RpcController*,
-                                                           const ResponseCallback&)>& func) {
+                                                           const ResponseCallback&)>& func,
+                                  std::vector<uint32_t> required_feature_flags) {
   MonoTime deadline = MonoTime::Now() + MonoDelta::FromMilliseconds(FLAGS_timeout_ms);
   Synchronizer sync;
   AsyncLeaderMasterRpc<Req, Resp> rpc(deadline, client_.get(), BackoffType::EXPONENTIAL,
-      req, resp, func, std::move(func_name), sync.AsStatusCallback(), {});
+      req, resp, func, std::move(func_name), sync.AsStatusCallback(),
+      std::move(required_feature_flags));
   rpc.SendRpc();
   return sync.Wait();
 }
 
 // Explicit specializations for callers outside this compilation unit.
+template
+Status LeaderMasterProxy::SyncRpc(
+    const master::AddMasterRequestPB& req,
+    master::AddMasterResponsePB* resp,
+    string func_name,
+    const std::function<void(MasterServiceProxy*,
+                             const master::AddMasterRequestPB&,
+                             master::AddMasterResponsePB*,
+                             RpcController*,
+                             const ResponseCallback&)>& func,
+    std::vector<uint32_t> required_feature_flags);
+
 template
 Status LeaderMasterProxy::SyncRpc(
     const master::ChangeTServerStateRequestPB& req,
@@ -941,7 +992,9 @@ Status LeaderMasterProxy::SyncRpc(
                              const master::ChangeTServerStateRequestPB&,
                              master::ChangeTServerStateResponsePB*,
                              RpcController*,
-                             const ResponseCallback&)>& func);
+                             const ResponseCallback&)>& func,
+    std::vector<uint32_t> required_feature_flags);
+
 template
 Status LeaderMasterProxy::SyncRpc(
     const master::ListTabletServersRequestPB& req,
@@ -951,7 +1004,9 @@ Status LeaderMasterProxy::SyncRpc(
                              const master::ListTabletServersRequestPB&,
                              master::ListTabletServersResponsePB*,
                              RpcController*,
-                             const ResponseCallback&)>& func);
+                             const ResponseCallback&)>& func,
+    std::vector<uint32_t> required_feature_flags);
+
 template
 Status LeaderMasterProxy::SyncRpc(
     const master::ListMastersRequestPB& req,
@@ -961,7 +1016,21 @@ Status LeaderMasterProxy::SyncRpc(
                              const master::ListMastersRequestPB&,
                              master::ListMastersResponsePB*,
                              RpcController*,
-                             const ResponseCallback&)>& func);
+                             const ResponseCallback&)>& func,
+    std::vector<uint32_t> required_feature_flags);
+
+template
+Status LeaderMasterProxy::SyncRpc(
+    const master::RemoveMasterRequestPB& req,
+    master::RemoveMasterResponsePB* resp,
+    string func_name,
+    const std::function<void(MasterServiceProxy*,
+                             const master::RemoveMasterRequestPB&,
+                             master::RemoveMasterResponsePB*,
+                             RpcController*,
+                             const ResponseCallback&)>& func,
+    std::vector<uint32_t> required_feature_flags);
+
 template
 Status LeaderMasterProxy::SyncRpc(
     const master::ReplaceTabletRequestPB& req,
@@ -971,7 +1040,8 @@ Status LeaderMasterProxy::SyncRpc(
                              const master::ReplaceTabletRequestPB&,
                              master::ReplaceTabletResponsePB*,
                              RpcController*,
-                             const ResponseCallback&)>& func);
+                             const ResponseCallback&)>& func,
+    std::vector<uint32_t> required_feature_flags);
 
 } // namespace tools
 } // namespace kudu
