@@ -78,7 +78,6 @@ class TransactionsVisitor {
 // Expected usage of this class is to have a management layer that reads and
 // writes to the underlying replica only if it is leader.
 //
-// TODO(awong): ensure that only the leader TxnStatusManager can call these.
 // TODO(awong): delete transactions that are entirely aborted or committed.
 // TODO(awong): consider batching writes.
 class TxnStatusTablet {
@@ -108,7 +107,7 @@ class TxnStatusTablet {
   // error. If there was otherwise a logical error with the request (e.g. row
   // already exists), returns an error without populating 'ts_error'.
   Status AddNewTransaction(int64_t txn_id, const std::string& user,
-                           tserver::TabletServerErrorPB* ts_error);
+                           int64_t start_timestamp, tserver::TabletServerErrorPB* ts_error);
   Status UpdateTransaction(int64_t txn_id, const TxnStatusEntryPB& pb,
                            tserver::TabletServerErrorPB* ts_error);
   Status AddNewParticipant(int64_t txn_id, const std::string& tablet_id,
