@@ -20,9 +20,11 @@
 #include <functional>
 #include <memory>
 #include <mutex>
+#include <ostream>
 #include <string>
 
 #include <boost/optional/optional.hpp>
+#include <gflags/gflags.h>
 #include <glog/logging.h>
 
 #include "kudu/client/client-internal.h"
@@ -33,15 +35,40 @@
 #include "kudu/common/common.pb.h"
 #include "kudu/common/partial_row.h"
 #include "kudu/common/partition.h"
+#include "kudu/gutil/macros.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
+#include "kudu/gutil/strings/substitute.h"
+#include "kudu/master/master.pb.h"
+#include "kudu/master/master.proxy.h"
+#include "kudu/rpc/messenger.h"
+#include "kudu/rpc/rpc_controller.h"
 #include "kudu/transactions/coordinator_rpc.h"
 #include "kudu/transactions/participant_rpc.h"
-#include "kudu/transactions/txn_status_tablet.h"
 #include "kudu/transactions/transactions.pb.h"
+#include "kudu/transactions/txn_status_tablet.h"
 #include "kudu/tserver/tserver_admin.pb.h"
 #include "kudu/util/async_util.h"
+#include "kudu/util/flag_tags.h"
+#include "kudu/util/logging.h"
+#include "kudu/util/net/dns_resolver.h"
 #include "kudu/util/net/net_util.h"
+#include "kudu/util/net/sockaddr.h"
+#include "kudu/util/threadpool.h"
+
+DEFINE_bool(enable_txn_system_client_init, false,
+            "Whether or not background TxnSystemClient initialization should "
+            "be enabled. Setting this to 'true' allows the server's "
+            "TxnSystemClient to participate in orchestrating transactions.");
+TAG_FLAG(enable_txn_system_client_init, experimental);
+
+DEFINE_int64(txn_system_client_op_timeout_ms, 10 * 1000,
+             "Op timeout used by the TxnSystemClient when making transactions-related "
+             "RPCs to the TxnStatusManager.");
+TAG_FLAG(txn_system_client_op_timeout_ms, advanced);
+TAG_FLAG(txn_system_client_op_timeout_ms, runtime);
+
+DECLARE_int64(rpc_negotiation_timeout_ms);
 
 using kudu::client::KuduClient;
 using kudu::client::KuduSchema;
@@ -50,17 +77,26 @@ using kudu::client::KuduTable;
 using kudu::client::KuduTableAlterer;
 using kudu::client::KuduTableCreator;
 using kudu::client::internal::MetaCache;
+using kudu::master::MasterServiceProxy;
+using kudu::master::PingRequestPB;
+using kudu::master::PingResponsePB;
+using kudu::rpc::Messenger;
+using kudu::rpc::RpcController;
+using kudu::tablet::TxnMetadataPB;
 using kudu::tserver::CoordinatorOpPB;
 using kudu::tserver::CoordinatorOpResultPB;
 using kudu::tserver::ParticipantOpPB;
+using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
 using std::vector;
+using strings::Substitute;
 
 namespace kudu {
 namespace transactions {
 
 Status TxnSystemClient::Create(const vector<HostPort>& master_addrs,
+                               const string& sasl_protocol_name,
                                unique_ptr<TxnSystemClient>* sys_client) {
   vector<string> master_strings;
   for (const auto& hp : master_addrs) {
@@ -69,6 +105,7 @@ Status TxnSystemClient::Create(const vector<HostPort>& master_addrs,
   DCHECK(!master_addrs.empty());
   KuduClientBuilder builder;
   builder.master_server_addrs(master_strings);
+  builder.sasl_protocol_name(sasl_protocol_name);
   client::sp::shared_ptr<KuduClient> client;
   RETURN_NOT_OK(builder.Build(&client));
   sys_client->reset(new TxnSystemClient(std::move(client)));
@@ -130,11 +167,37 @@ Status TxnSystemClient::OpenTxnStatusTable() {
   return Status::OK();
 }
 
+Status TxnSystemClient::CheckOpenTxnStatusTable() {
+  {
+    std::lock_guard<simple_spinlock> l(table_lock_);
+    if (txn_status_table_) {
+      return Status::OK();
+    }
+  }
+
+  // TODO(aserbin): enqueue concurrent calls to the OpenTable() above, if any
+  client::sp::shared_ptr<KuduTable> table;
+  RETURN_NOT_OK(client_->OpenTable(TxnStatusTablet::kTxnStatusTableName, &table));
+
+  {
+    std::lock_guard<simple_spinlock> l(table_lock_);
+    // Extra check to handle concurrent callers.
+    if (!txn_status_table_) {
+      txn_status_table_ = std::move(table);
+    }
+  }
+
+  return Status::OK();
+}
+
 Status TxnSystemClient::BeginTransaction(int64_t txn_id,
                                          const string& user,
                                          uint32_t* txn_keepalive_ms,
                                          int64_t* highest_seen_txn_id,
                                          MonoDelta timeout) {
+  if (!timeout.Initialized()) {
+    timeout = MonoDelta::FromMilliseconds(FLAGS_txn_system_client_op_timeout_ms);
+  }
   CoordinatorOpPB coordinate_txn_op;
   coordinate_txn_op.set_type(CoordinatorOpPB::BEGIN_TXN);
   coordinate_txn_op.set_txn_id(txn_id);
@@ -163,6 +226,9 @@ Status TxnSystemClient::BeginTransaction(int64_t txn_id,
 
 Status TxnSystemClient::RegisterParticipant(int64_t txn_id, const string& participant_id,
                                             const string& user, MonoDelta timeout) {
+  if (!timeout.Initialized()) {
+    timeout = MonoDelta::FromMilliseconds(FLAGS_txn_system_client_op_timeout_ms);
+  }
   CoordinatorOpPB coordinate_txn_op;
   coordinate_txn_op.set_type(CoordinatorOpPB::REGISTER_PARTICIPANT);
   coordinate_txn_op.set_txn_id(txn_id);
@@ -178,6 +244,9 @@ Status TxnSystemClient::RegisterParticipant(int64_t txn_id, const string& partic
 Status TxnSystemClient::BeginCommitTransaction(int64_t txn_id,
                                                const string& user,
                                                MonoDelta timeout) {
+  if (!timeout.Initialized()) {
+    timeout = MonoDelta::FromMilliseconds(FLAGS_txn_system_client_op_timeout_ms);
+  }
   CoordinatorOpPB coordinate_txn_op;
   coordinate_txn_op.set_type(CoordinatorOpPB::BEGIN_COMMIT_TXN);
   coordinate_txn_op.set_txn_id(txn_id);
@@ -192,6 +261,9 @@ Status TxnSystemClient::BeginCommitTransaction(int64_t txn_id,
 Status TxnSystemClient::AbortTransaction(int64_t txn_id,
                                          const string& user,
                                          MonoDelta timeout) {
+  if (!timeout.Initialized()) {
+    timeout = MonoDelta::FromMilliseconds(FLAGS_txn_system_client_op_timeout_ms);
+  }
   CoordinatorOpPB coordinate_txn_op;
   coordinate_txn_op.set_type(CoordinatorOpPB::ABORT_TXN);
   coordinate_txn_op.set_txn_id(txn_id);
@@ -207,6 +279,9 @@ Status TxnSystemClient::GetTransactionStatus(int64_t txn_id,
                                              const string& user,
                                              TxnStatusEntryPB* txn_status,
                                              MonoDelta timeout) {
+  if (!timeout.Initialized()) {
+    timeout = MonoDelta::FromMilliseconds(FLAGS_txn_system_client_op_timeout_ms);
+  }
   DCHECK(txn_status);
   CoordinatorOpPB coordinate_txn_op;
   coordinate_txn_op.set_type(CoordinatorOpPB::GET_TXN_STATUS);
@@ -218,22 +293,25 @@ Status TxnSystemClient::GetTransactionStatus(int64_t txn_id,
                                            timeout,
                                            s.AsStatusCallback(),
                                            &result));
-  const auto ret = s.Wait();
-  if (ret.ok()) {
+  const auto rs = s.Wait();
+  if (rs.ok()) {
     // Retrieve the response and set corresponding output parameters.
     DCHECK(!result.has_op_error());
     DCHECK(result.has_txn_status());
+    DCHECK(result.txn_status().has_state());
     TxnStatusEntryPB ret;
-    ret.set_state(result.txn_status().state());
-    ret.set_allocated_user(result.mutable_txn_status()->release_user());
+    ret.Swap(result.mutable_txn_status());
     *txn_status = std::move(ret);
   }
-  return ret;
+  return rs;
 }
 
 Status TxnSystemClient::KeepTransactionAlive(int64_t txn_id,
                                              const string& user,
                                              MonoDelta timeout) {
+  if (!timeout.Initialized()) {
+    timeout = MonoDelta::FromMilliseconds(FLAGS_txn_system_client_op_timeout_ms);
+  }
   CoordinatorOpPB coordinate_txn_op;
   coordinate_txn_op.set_type(CoordinatorOpPB::KEEP_TXN_ALIVE);
   coordinate_txn_op.set_txn_id(txn_id);
@@ -296,10 +374,11 @@ Status TxnSystemClient::CoordinateTransactionAsync(CoordinatorOpPB coordinate_tx
 Status TxnSystemClient::ParticipateInTransaction(const string& tablet_id,
                                                  const ParticipantOpPB& participant_op,
                                                  const MonoDelta& timeout,
-                                                 Timestamp* begin_commit_timestamp) {
+                                                 Timestamp* begin_commit_timestamp,
+                                                 TxnMetadataPB* metadata_pb) {
   Synchronizer sync;
   ParticipateInTransactionAsync(tablet_id, participant_op, timeout,
-                                sync.AsStatusCallback(), begin_commit_timestamp);
+                                sync.AsStatusCallback(), begin_commit_timestamp, metadata_pb);
   return sync.Wait();
 }
 
@@ -307,7 +386,8 @@ void TxnSystemClient::ParticipateInTransactionAsync(const string& tablet_id,
                                                     ParticipantOpPB participant_op,
                                                     const MonoDelta& timeout,
                                                     StatusCallback cb,
-                                                    Timestamp* begin_commit_timestamp) {
+                                                    Timestamp* begin_commit_timestamp,
+                                                    TxnMetadataPB* metadata_pb) {
   MonoTime deadline = MonoTime::Now() + timeout;
   unique_ptr<TxnParticipantContext> ctx(
       new TxnParticipantContext({
@@ -322,7 +402,8 @@ void TxnSystemClient::ParticipateInTransactionAsync(const string& tablet_id,
   // See https://taylorconor.com/blog/noncopyable-lambdas/ for more details.
   client_->data_->meta_cache_->LookupTabletById(
       client_.get(), tablet_id, deadline, &ctx_raw->tablet,
-      [cb = std::move(cb), deadline, ctx_raw, begin_commit_timestamp] (const Status& s) mutable {
+      [cb = std::move(cb), deadline, ctx_raw, begin_commit_timestamp, metadata_pb]
+          (const Status& s) mutable {
         unique_ptr<TxnParticipantContext> unique_ctx(ctx_raw);
         if (PREDICT_FALSE(!s.ok())) {
           cb(s);
@@ -332,10 +413,119 @@ void TxnSystemClient::ParticipateInTransactionAsync(const string& tablet_id,
             std::move(unique_ctx),
             deadline,
             std::move(cb),
-            begin_commit_timestamp);
+            begin_commit_timestamp,
+            metadata_pb);
         rpc->SendRpc();
       });
 }
 
+TxnSystemClientInitializer::TxnSystemClientInitializer()
+    : init_complete_(false),
+      shutting_down_(false) {}
+
+TxnSystemClientInitializer::~TxnSystemClientInitializer() {
+  Shutdown();
+}
+
+Status TxnSystemClientInitializer::Init(const shared_ptr<Messenger>& messenger,
+                                        vector<HostPort> master_addrs) {
+  if (PREDICT_FALSE(!FLAGS_enable_txn_system_client_init)) {
+    // If we're set to not enable the system client, return early. Further
+    // attempts to access the client will fail since initialization never
+    // completes.
+    LOG(INFO) << "TxnSystemClient initialization is disabled...";
+    return Status::OK();
+  }
+  RETURN_NOT_OK(ThreadPoolBuilder("txn-client-init")
+      .set_max_threads(1)
+      .Build(&txn_client_init_pool_));
+
+  return txn_client_init_pool_->Submit([this, messenger, master_addrs = std::move(master_addrs)] {
+      unique_ptr<TxnSystemClient> txn_client;
+      while (!shutting_down_) {
+        // HACK: if the master addresses are all totally unreachable,
+        // KuduClientBuilder::Build() will hang, attempting fruitlessly to
+        // retry, in the below call to Create(). So first, make sure we can at
+        // least reach the masters; if not, try again.
+        // TODO(awong): there's still a small window between these pings and
+        // client creation. If this ends up being a problem, we may need to
+        // come to a more robust solution, e.g. adding a timeout to Create().
+        DnsResolver dns_resolver;
+        Status s;
+        for (const auto& hp : master_addrs) {
+          vector<Sockaddr> addrs;
+          s = dns_resolver.ResolveAddresses(hp, &addrs).AndThen([&] {
+            unique_ptr<MasterServiceProxy> proxy(
+                new MasterServiceProxy(messenger, addrs[0], hp.host()));
+            PingRequestPB req;
+            PingResponsePB resp;
+            RpcController rpc;
+            rpc.set_timeout(MonoDelta::FromMilliseconds(FLAGS_rpc_negotiation_timeout_ms));
+            return proxy->Ping(req, &resp, &rpc);
+          });
+          if (s.ok()) {
+            break;
+          }
+        }
+        // Only if we can reach at least one of the masters should we try
+        // connecting.
+        if (PREDICT_TRUE(s.ok())) {
+          s = TxnSystemClient::Create(master_addrs,
+                                      messenger->sasl_proto_name(),
+                                      &txn_client);
+        }
+        if (PREDICT_TRUE(s.ok())) {
+          txn_client_ = std::move(txn_client);
+          init_complete_ = true;
+          return;
+        }
+        static const MonoDelta kRetryInterval = MonoDelta::FromSeconds(1);
+        KLOG_EVERY_N_SECS(WARNING, 60) <<
+            Substitute("unable to initialize TxnSystemClient, will retry in $0: $1",
+                       kRetryInterval.ToString(), s.ToString());
+        SleepFor(kRetryInterval);
+      }
+  });
+}
+
+Status TxnSystemClientInitializer::GetClient(TxnSystemClient** client) const {
+  // NOTE: the shutdown check is best effort. There's still room for a TOCTOU.
+  if (PREDICT_FALSE(shutting_down_)) {
+    return Status::ServiceUnavailable("could not get TxnSystemClient, shutting down");
+  }
+  if (PREDICT_TRUE(init_complete_)) {
+    *client = DCHECK_NOTNULL(txn_client_.get());
+    return Status::OK();
+  }
+  return Status::ServiceUnavailable("could not get TxnSystemClient, still initializing");
+}
+
+Status TxnSystemClientInitializer::WaitForClient(const MonoDelta& timeout,
+                                                 TxnSystemClient** client) const {
+  const auto deadline = MonoTime::Now() + timeout;
+  Status s;
+  do {
+    if (shutting_down_) {
+      return Status::ServiceUnavailable("could not get TxnSystemClient, shutting down");
+    }
+    s = GetClient(client);
+    if (PREDICT_TRUE(s.ok())) {
+      DCHECK(*client);
+      return Status::OK();
+    }
+    SleepFor(MonoDelta::FromMilliseconds(100));
+  } while (MonoTime::Now() < deadline);
+  return Status::TimedOut(Substitute("Unable to get client in $0: $1",
+                                     timeout.ToString(), s.ToString()));
+}
+
+void TxnSystemClientInitializer::Shutdown() {
+  shutting_down_ = true;
+  if (FLAGS_enable_txn_system_client_init) {
+    txn_client_init_pool_->Wait();
+    txn_client_init_pool_->Shutdown();
+  }
+}
+
 } // namespace transactions
 } // namespace kudu
