@@ -16,6 +16,7 @@
 // under the License.
 #pragma once
 
+#include <atomic>
 #include <cstdint>
 #include <memory>
 #include <mutex>
@@ -34,6 +35,7 @@
 namespace kudu {
 class HostPort;
 class Timestamp;
+class ThreadPool;
 
 namespace client {
 class KuduClient;
@@ -43,8 +45,17 @@ class KuduTable;
 namespace itest {
 class TxnStatusTableITest;
 class TxnStatusTableITest_TestProtectCreateAndAlter_Test;
+class TxnStatusTableITest_CheckOpenTxnStatusTable_Test;
 } // namespace itest
 
+namespace rpc {
+class Messenger;
+} // namespace rpc
+
+namespace tablet {
+class TxnMetadataPB;
+} // namespace tablet
+
 namespace tserver {
 class CoordinatorOpPB;
 class CoordinatorOpResultPB;
@@ -60,6 +71,7 @@ class TxnStatusEntryPB;
 class TxnSystemClient {
  public:
   static Status Create(const std::vector<HostPort>& master_addrs,
+                       const std::string& sasl_protocol_name,
                        std::unique_ptr<TxnSystemClient>* sys_client);
 
   // Creates the transaction status table with a single range partition of the
@@ -95,24 +107,24 @@ class TxnSystemClient {
                           std::string& user,
                           uint32_t* txn_keepalive_ms = nullptr,
                           int64_t* highest_seen_txn_id = nullptr,
-                          MonoDelta timeout = MonoDelta::FromSeconds(10));
+                          MonoDelta timeout = MonoDelta());
 
   // Attempts to register the given participant with the given transaction.
   // Returns an error if the transaction hasn't yet been started, or if the
   // 'user' isn't permitted to modify the transaction.
   Status RegisterParticipant(int64_t txn_id, const std::string& participant_id,
                              const std::string& user,
-                             MonoDelta timeout = MonoDelta::FromSeconds(10));
+                             MonoDelta timeout = MonoDelta());
 
   // Initiates committing a transaction with the given identifier.
   Status BeginCommitTransaction(int64_t txn_id,
                                 const std::string& user,
-                                MonoDelta timeout = MonoDelta::FromSeconds(10));
+                                MonoDelta timeout = MonoDelta());
 
   // Aborts a transaction with the given identifier.
   Status AbortTransaction(int64_t txn_id,
                           const std::string& user,
-                          MonoDelta timeout = MonoDelta::FromSeconds(10));
+                          MonoDelta timeout = MonoDelta());
 
   // Retrieves transactions status. On success, returns Status::OK() and stores
   // the result status in the 'txn_status' output parameter. On failure,
@@ -120,17 +132,22 @@ class TxnSystemClient {
   Status GetTransactionStatus(int64_t txn_id,
                               const std::string& user,
                               TxnStatusEntryPB* txn_status,
-                              MonoDelta timeout = MonoDelta::FromSeconds(10));
+                              MonoDelta timeout = MonoDelta());
 
   // Send keep-alive heartbeat for the specified transaction as the given user.
   Status KeepTransactionAlive(int64_t txn_id,
                               const std::string& user,
-                              MonoDelta timeout = MonoDelta::FromSeconds(10));
+                              MonoDelta timeout = MonoDelta());
 
   // Opens the transaction status table, refreshing metadata with that from the
   // masters.
   Status OpenTxnStatusTable();
 
+  // Check if the transaction status table is already open, returning
+  // Status::OK() if so. Otherwise, open the transaction status table. In the
+  // latter case, the result status of opening the table is returned.
+  Status CheckOpenTxnStatusTable();
+
   // Sends an RPC to the leader of the given tablet to participate in a
   // transaction.
   //
@@ -139,11 +156,19 @@ class TxnSystemClient {
   Status ParticipateInTransaction(const std::string& tablet_id,
                                   const tserver::ParticipantOpPB& participant_op,
                                   const MonoDelta& timeout,
-                                  Timestamp* begin_commit_timestamp = nullptr);
+                                  Timestamp* begin_commit_timestamp = nullptr,
+                                  tablet::TxnMetadataPB* metadata_pb = nullptr);
+  void ParticipateInTransactionAsync(const std::string& tablet_id,
+                                     tserver::ParticipantOpPB participant_op,
+                                     const MonoDelta& timeout,
+                                     StatusCallback cb,
+                                     Timestamp* begin_commit_timestamp = nullptr,
+                                     tablet::TxnMetadataPB* metadata_pb = nullptr);
  private:
 
   friend class itest::TxnStatusTableITest;
   FRIEND_TEST(itest::TxnStatusTableITest, TestProtectCreateAndAlter);
+  FRIEND_TEST(itest::TxnStatusTableITest, CheckOpenTxnStatusTable);
 
   explicit TxnSystemClient(client::sp::shared_ptr<client::KuduClient> client)
       : client_(std::move(client)) {}
@@ -158,12 +183,6 @@ class TxnSystemClient {
                                     const StatusCallback& cb,
                                     tserver::CoordinatorOpResultPB* result = nullptr);
 
-  void ParticipateInTransactionAsync(const std::string& tablet_id,
-                                     tserver::ParticipantOpPB participant_op,
-                                     const MonoDelta& timeout,
-                                     StatusCallback cb,
-                                     Timestamp* begin_commit_timestamp = nullptr);
-
   client::sp::shared_ptr<client::KuduTable> txn_status_table() {
     std::lock_guard<simple_spinlock> l(table_lock_);
     return txn_status_table_;
@@ -175,5 +194,48 @@ class TxnSystemClient {
   client::sp::shared_ptr<client::KuduTable> txn_status_table_;
 };
 
+// Wrapper around a TxnSystemClient that allows callers to asynchronously
+// create a client.
+// TODO(awong): the problem at hand is similar to TxnManager initialization,
+// minus table creation. Refactor for code reuse?
+class TxnSystemClientInitializer {
+ public:
+  TxnSystemClientInitializer();
+  ~TxnSystemClientInitializer();
+
+  // Starts attempts to initialize the transaction system client.
+  Status Init(const std::shared_ptr<rpc::Messenger>& messenger,
+              std::vector<HostPort> master_addrs);
+
+  // Returns a ServiceUnavailable error if the client has not yet been
+  // initialized or if Shutdown() has been called. Otherwise, returns OK and
+  // sets 'client'. Callers should ensure that 'client' is only used while the
+  // TxnSystemClientInitializer is still in scope.
+  Status GetClient(TxnSystemClient** client) const;
+
+  // Like the above, but retries periodically for the client to be initialized
+  // for up to 'timeout', returning a TimedOut error if unable to. Returns a
+  // ServiceUnavailable error if the initializer has been being shut down.
+  Status WaitForClient(const MonoDelta& timeout, TxnSystemClient** client) const;
+
+  // Stops the initialization, preventing success of further calls to
+  // GetClient().
+  void Shutdown();
+
+ private:
+  // Whether or not 'txn_client_' has been initialized.
+  std::atomic<bool> init_complete_;
+
+  // Whether or not the client initializer is shutting down, in which case
+  // attempts to access 'txn_client_' should fail.
+  std::atomic<bool> shutting_down_;
+
+  // Threadpool on which to schedule attempts to initialize 'txn_client_'.
+  std::unique_ptr<ThreadPool> txn_client_init_pool_;
+
+  // The TxnSystemClient, initialized asynchronously via calls to Init().
+  std::unique_ptr<TxnSystemClient> txn_client_;
+};
+
 } // namespace transactions
 } // namespace kudu
