@@ -59,6 +59,10 @@ class PartitionSchema;
 class Schema;
 class ThreadPool;
 
+namespace transactions {
+class TxnSystemClient;
+}  // namespace transactions
+
 namespace consensus {
 class ConsensusMetadataManager;
 class OpId;
@@ -240,6 +244,19 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
   // Update the tablet statistics if necessary.
   void UpdateTabletStatsIfNecessary();
 
+  // Schedule preliminary tasks to begin transaction 'txn_id' started by 'user'
+  // with 'replica' as a participant, with the given deadline. Calls 'cb' if
+  // any of the tasks fail.
+  Status SchedulePreliminaryTasksForTxnWrite(
+      scoped_refptr<tablet::TabletReplica> replica,
+      int64_t txn_id,
+      const std::string& user,
+      MonoTime deadline,
+      tablet::RegisteredTxnCallback cb);
+
+  // Schedule the rollback of the given transaction as the given user.
+  Status ScheduleAbortTxn(int64_t txn_id, const std::string& user);
+
  private:
   FRIEND_TEST(LeadershipChangeReportingTest, TestReportStatsDuringLeadershipChange);
   FRIEND_TEST(TsTabletManagerTest, TestPersistBlocks);
@@ -258,6 +275,14 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
     return LogPrefix(tablet_id, fs_manager_);
   }
 
+  static void RegisterAndBeginParticipantTxnTask(
+      transactions::TxnSystemClient* txn_system_client,
+      scoped_refptr<tablet::TabletReplica> replica,
+      int64_t txn_id,
+      const std::string& user,
+      MonoTime deadline,
+      tablet::RegisteredTxnCallback began_txn_cb);
+
   // Returns Status::OK() iff state_ == MANAGER_RUNNING.
   Status CheckRunningUnlocked(TabletServerErrorPB::Code* error_code) const;
 
@@ -359,7 +384,7 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
 
   const scoped_refptr<consensus::ConsensusMetadataManager> cmeta_manager_;
 
-  TabletServer* server_;
+  TabletServer* const server_;
 
   consensus::RaftPeerPB local_peer_pb_;
 
@@ -401,6 +426,11 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
   // Holds cached tablet states from tablet_map_.
   std::map<tablet::TabletStatePB, int> tablet_state_counts_;
 
+  // Set of transactions that have a pending call to abort, indicating that
+  // further attempts to schedule such a call can be ignored.
+  simple_spinlock txn_aborts_lock_;
+  std::unordered_set<int64_t> txn_aborts_in_progress_;
+
   TSTabletManagerStatePB state_;
 
   // Thread pool used to run tablet copy operations.
@@ -412,6 +442,17 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
   // Thread pool used to delete tablets asynchronously.
   std::unique_ptr<ThreadPool> delete_tablet_pool_;
 
+  // Thread pool used to reload transaction status tablets asynchronously.
+  std::unique_ptr<ThreadPool> reload_txn_status_tablet_pool_;
+
+  // Thread pool used to perform background tasks on transactions, e.g. to commit.
+  std::unique_ptr<ThreadPool> txn_commit_pool_;
+
+  // Thread pool to perform preliminary tasks when processing write operations
+  // in the context of a multi-row transaction. Such tasks include registering
+  // tablet as a participant in the corresponding transaction, etc.
+  std::unique_ptr<ThreadPool> txn_participant_registration_pool_;
+
   // Thread pool to run TxnStatusManager tasks. As of now, this pool is
   // to run a long-running single periodic task to abort stale transactions
   // registered with corresponding transaction status tablets.
