@@ -89,11 +89,17 @@ fixup_libtool() {
 }
 
 build_cmake() {
+  # Use system curl if it is available: this shortens build times a bit.
+  local use_system_curl=""
+  if pkg-config --exists libcurl; then
+    use_system_curl="--system-curl"
+  fi
   CMAKE_BDIR=$TP_BUILD_DIR/$CMAKE_NAME$MODE_SUFFIX
   mkdir -p $CMAKE_BDIR
   pushd $CMAKE_BDIR
   $CMAKE_SOURCE/bootstrap \
     --prefix=$PREFIX \
+    $use_system_curl \
     --parallel=$PARALLEL -- \
     -DBUILD_TESTING=OFF
   # Unfortunately, cmake's bootstrap always uses Makefiles
@@ -108,9 +114,7 @@ build_libcxxabi() {
   pushd $LIBCXXABI_BDIR
   rm -Rf CMakeCache.txt CMakeFiles/
 
-  # libcxxabi requires gcc5 or newer. Since we can't guarantee that universally,
-  # let's always build it with clang.
-  CC="$CLANG" CXX="$CLANGXX" cmake \
+  cmake \
     -DCMAKE_BUILD_TYPE=Release \
     -DCMAKE_INSTALL_PREFIX=$PREFIX \
     -DCMAKE_CXX_FLAGS="$EXTRA_CXXFLAGS $EXTRA_LDFLAGS" \
@@ -140,9 +144,7 @@ build_libcxx() {
   mkdir -p $LIBCXX_BDIR
   pushd $LIBCXX_BDIR
   rm -Rf CMakeCache.txt CMakeFiles/
-  # Since libcxxabi requires gcc5 or newer, we build it with clang. As libcxx is
-  # a dependency, let's also always use clang to build it.
-  CC="$CLANG" CXX="$CLANGXX" cmake \
+  cmake \
     -DCMAKE_BUILD_TYPE=Release \
     -DCMAKE_INSTALL_PREFIX=$PREFIX \
     -DCMAKE_CXX_FLAGS="$EXTRA_CXXFLAGS" \
@@ -475,7 +477,8 @@ build_gperftools() {
   popd
 }
 
-build_gmock() {
+build_gmock_gtest() {
+  # Build both gmock and gtest
   GMOCK_SHARED_BDIR=$TP_BUILD_DIR/$GMOCK_NAME.shared$MODE_SUFFIX
   GMOCK_STATIC_BDIR=$TP_BUILD_DIR/$GMOCK_NAME.static$MODE_SUFFIX
   for SHARED in ON OFF; do
@@ -488,7 +491,6 @@ build_gmock() {
     pushd $GMOCK_BDIR
     rm -rf CMakeCache.txt CMakeFiles/
     cmake \
-      -DCMAKE_BUILD_TYPE=Debug \
       -DCMAKE_POSITION_INDEPENDENT_CODE=On \
       -DCMAKE_CXX_FLAGS="$EXTRA_CXXFLAGS" \
       -DCMAKE_EXE_LINKER_FLAGS="$EXTRA_LDFLAGS $EXTRA_LIBS" \
@@ -496,7 +498,7 @@ build_gmock() {
       -DCMAKE_SHARED_LINKER_FLAGS="$EXTRA_LDFLAGS $EXTRA_LIBS" \
       -DBUILD_SHARED_LIBS=$SHARED \
       $EXTRA_CMAKE_FLAGS \
-      $GMOCK_SOURCE/googlemock
+      $GMOCK_SOURCE
     ${NINJA:-make} -j$PARALLEL $EXTRA_MAKEFLAGS
     popd
   done
@@ -504,9 +506,11 @@ build_gmock() {
   # Install gmock/gtest libraries and headers manually instead of using make
   # install. Make install results in libraries with a malformed lib name on
   # macOS.
-  echo Installing gmock...
-  cp -a $GMOCK_SHARED_BDIR/libgmock.$DYLIB_SUFFIX $PREFIX/lib/
-  cp -a $GMOCK_STATIC_BDIR/libgmock.a $PREFIX/lib/
+  echo Installing gmock and gtest...
+  cp -a $GMOCK_SHARED_BDIR/lib/libgmock.$DYLIB_SUFFIX $PREFIX/lib/
+  cp -a $GMOCK_STATIC_BDIR/lib/libgmock.a $PREFIX/lib/
+  cp -a $GMOCK_SHARED_BDIR/lib/libgtest.$DYLIB_SUFFIX $PREFIX/lib/
+  cp -a $GMOCK_STATIC_BDIR/lib/libgtest.a $PREFIX/lib/
   rsync -av $GMOCK_SOURCE/googlemock/include/ $PREFIX/include/
   rsync -av $GMOCK_SOURCE/googletest/include/ $PREFIX/include/
 }
@@ -830,6 +834,14 @@ build_boost() {
   # Compile with PIC and set nanosecond resolution for impala compatibility.
   BOOST_CFLAGS="$EXTRA_CFLAGS -fPIC -DBOOST_DATE_TIME_POSIX_TIME_STD_CONFIG"
   BOOST_CXXFLAGS="$EXTRA_CXXFLAGS -fPIC -DBOOST_DATE_TIME_POSIX_TIME_STD_CONFIG"
+
+  # Force Boost UUID to use /dev/random instead of getrandom(2) when compiled on
+  # Linux 3.17 or later. Using getrandom(2) has caused crashes, especially in the
+  # binary jar, when getrandom(2) is not available.
+  # https://www.boost.org/doc/libs/1_68_0/libs/uuid/doc/uuid.html#Random%20Generator
+  BOOST_CFLAGS="$BOOST_CFLAGS -DBOOST_UUID_RANDOM_PROVIDER_FORCE_POSIX"
+  BOOST_CXXFLAGS="$BOOST_CFLAGS -DBOOST_UUID_RANDOM_PROVIDER_FORCE_POSIX"
+
   BOOST_LDFLAGS="$EXTRA_LDFLAGS"
 
   case $BUILD_TYPE in
